const gulp = require('gulp');
const fs = require('fs');
const concat = require('gulp-concat');
const rename = require('gulp-rename');
const uglify = require('gulp-uglify');

const sourceFolder = './webroot/js/dss/dss-setup-interface';
const entryPoint = 'interface-module.js';
const outputFolder = './webroot/js/dss';
const output = 'dss-interface-module.js'

function getSourcesRecursively (path) {
    const file = fs.readFileSync(`${sourceFolder}/${path}`, 'utf8');
    return file.split('\n')
        .filter(function (line) {
            return line.indexOf('//= require') === 0;
        })
        .map(function (line) {
            const match = line.trim().match(/\<([^>]+)\>$/);
            return match[1].indexOf('.js') !== -1
                ? match[1]
                : `${match[1]}.js`;
        })
        .reduce(function (list, fileRelPath) {
            const files = getSourcesRecursively(fileRelPath);
            return list.concat(files)
                .filter(function (filePath, index, allFilePaths) {
                    return allFilePaths.indexOf(filePath) === index;
                });
        }, [])
        .concat([ `${sourceFolder}/${path}` ]);
}

gulp.task('dev', function () {
    const fileList = getSourcesRecursively(entryPoint);
    console.log(fileList);

    return gulp.src(fileList, { sourcemaps: true })
        .pipe(concat(output))
        .pipe(gulp.dest(outputFolder, { sourcemaps: '.' }));
});

gulp.task('build', function () {
    const fileList = getSourcesRecursively(entryPoint);
    console.log(fileList);

    return gulp.src(fileList, { sourcemaps: true })
        .pipe(concat(output))
        .pipe(uglify())
        .pipe(gulp.dest(outputFolder, { sourcemaps: '.' }));
});

gulp.task('default', function () {
    gulp.watch(`${sourceFolder}/**/*.js`, gulp.series('dev'));
    return;
});
