//= require <dSS/util/Util>

Ext.namespace('dSS', 'dSS.grid');

dSS.grid.DeviceCacheDetails = Ext.extend(Ext.grid.GridPanel,
{
    initComponent: function()
    {
        var cacheReadoutRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            return value === "" ? "" : String(Math.round(value, 10)) + "%";
        };

        var columns =
        [
            {
                header: dSS._("Room name"),
                tooltip: dSS._("Room name"),
                width: 80,
                sortable: true,
                dataIndex: 'zoneName',
                renderer: dSS.util.gridZoneNameRenderer
            },
            {
                header: "",
                width: 20,
                resizable: false,
                sortable: false,
                hideable: false,
                menuDisabled: true,
                fixed: true,
                dataIndex: 'functionID',
                renderer: dSS.util.gridDeviceTypeRenderer
            },
            {
                header: dSS._("Name"),
                tooltip: dSS._("Name"),
                width: 80,
                sortable: true,
                dataIndex: 'name',
                    renderer: dSS.util.gridDeviceNameRenderer
            },
            {
                header: dSS._("ID"),
                tooltip: dSS._("ID"),
                width: 70,
                sortable: true,
                    dataIndex: 'dsuid',
                renderer: dSS.util.gridDeviceDSUIDRenderer
            },
            {
                header: dSS._("Cache readout"),
                tooltip: dSS._("Cache readout"),
                width: 70,
                sortable: true,
                dataIndex: 'cacheReadout',
                renderer: cacheReadoutRenderer
            }
        ];

        Ext.apply(this, {
            cls: 'dsClsDeviceCacheDetails',

            store: dSS.data.Loader.getDeviceStore(),
            columns: columns,
            stripeRows: true,
            disableSelection: true,
            viewConfig: {
                forceFit: true
            }
        });

        dSS.grid.DeviceCacheDetails.superclass.initComponent.apply(this, arguments);
        this.on('afterrender', function(me) {
            me.sortState = this.getStore().getSortState();
            me.getStore().clearFilter();
            me.getStore().filterBy(function(record) {
                return !isNaN(parseInt(record.get('cacheReadout'), 10));
            }, this);
            me.getStore().sort('zoneName', 'ASC');
        }, this);

        this.on('beforedestroy', function(me) {
            if (me.sortState !== undefined) {
                me.getStore().sort(me.sortState.field,
                                   me.sortState.direction);
            }
        }, this);
    }
});

Ext.reg('dssdevicecachedetails', dSS.grid.DeviceCacheDetails);

dSS.DeviceCacheInfo = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        var MIN_BUTTON_WIDTH = 110;

        this.detailsButton = new Ext.Button({
            cls: 'dsClsDeviceCacheInfo_DetailsButton',

            text: dSS._("Details"),
            handler: this.showDetails.createDelegate(this),
            minWidth: MIN_BUTTON_WIDTH
        });

        this.updateButton = new Ext.Button({
            cls: 'dsClsDeviceCacheInfo_UpdateButton',

            text: dSS._("Update Cache"),
            handler: this.updateCache.createDelegate(this),
            ref: 'updateButton',
            minWidth: MIN_BUTTON_WIDTH,
        });

        this.resetCacheButton = new Ext.Button({
            cls: 'dsClsDeviceCacheInfo_ResetCacheButton',

            text: dSS._("Reset Cache"),
            minWidth: MIN_BUTTON_WIDTH,
            handler: this.confirmResetCache.createDelegate(this)
        });

        Ext.apply(this, {
            cls: 'dsClsDeviceCacheInfo',

            layout: 'form',
            bodyBorder: false,
            items:
            [
                {
                    xtype: 'fieldset',
                    title: dSS._("Device Cache"),
                    layout: 'form',
                    scope: this,
                    ref: 'mainSet',
                    items:
                    [
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            html: dSS._("The system caches device data to " +
                                        "speed up device configuration and " +
                                        "to determine room, group and device " +
                                        "states. The states may differ from " +
                                        "the real states until finishing " +
                                        "readout. Automatic cache update is " +
                                        "performed periodically and on " +
                                        "changes.")
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            height: 25,
                            ref: 'deviceLabel'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            ref: 'statusLabel',
                            html: dSS._("Readout status") + ":"
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'progress',
                            ref: 'progress'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            html: dSS._("Remaining readout time depends on " +
                                        "system usage.")
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            ref: 'estimationLabel'
                        },
                        {
                            xtype: 'box',
                            height: 0,
                            ref: 'estimationLabelSpacer'
                        },
                        {
                            xtype: 'panel',
                            layout: 'hbox',
                            border: false,
                            items: [
                                this.detailsButton,
                                {
                                    xtype: 'label',
                                    flex: 3
                                },
                                this.updateButton
                            ]
                        },
                        {
                            xtype: 'box',
                            height: 20
                        },
                        {
                            cls: 'dsClsDeviceCacheInfo_CacheResetPanel',

                            xtype: 'panel',
                            layout: 'hbox',
                            border: false,
                            ref: 'cacheResetPanel',
                            items: [
                                {
                                    xtype: 'label',
                                    cls: 'x-green-text',
                                    text: dSS._("Only in case you encounter issues with room, group or device states:"),
                                    flex: 3,
                                    autoHeight: true,
                                    ref: 'cacheResetLabel',
                                    listeners: {
                                        render: function (label) {
                                            setTimeout(function () {
                                                // immediate label parent fixed height needs to be removed after render probably because
                                                // the label is initially hidden and its height doesn't reflect contents at the time when parent's sizes are determined
                                                label.el.dom.parentNode.style.minHeight = label.getHeight() + 'px';
                                            }, 0);
                                        }
                                    }
                                },
                                this.resetCacheButton
                            ]
                        }
                    ]
                }
            ]
        });

        dSS.DeviceCacheInfo.superclass.initComponent.apply(this, arguments);

        var unmask = dSS.util.maskViewport.createDelegate(this, [ 'unmask' ]);
        var mask = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);

        dSS.MessageBus.on('reloadapartment', function() {
            if (!this.isVisible()) {
                return;
            }
            mask();
        }, this);

        dSS.MessageBus.on('structureloaded', function() {
            if (!this.isVisible()) {
                return;
            }
            unmask();
            this.getCacheStatus();
        }, this);

        dSS.MessageBus.on('refreshview', this.reloadData.createDelegate(this),
            this);
    },
    reloadData: function() {
        if (dSS.config.firstLoad === true) {
            var fn = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
            fn();
            dSS.data.Loader.load();
            dSS.config.firstLoad = false;
         } else {
            this.getCacheStatus();
        }
    },
    getCacheStatus : function() {
        if (!this.isVisible()) {
            return;
        }

        var mask = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
        var unmask = dSS.util.maskViewport.createDelegate(this, [ 'unmask' ]);

        mask();
        Ext.Ajax.request({
            url: '/api/v1/installation/parameterCache',
            disableCaching: true,
            timeout: 60000,
            method: 'GET',
            scope: this,
            success: function(response) {
                try {
                    if (response.status >= 300) {
                        Ext.Msg.alert(_("Error"),
                            dSS._("Could not determine device cache status"));
                        unmask();
                        return;
                    }


                    var json = Ext.util.JSON.decode(response.responseText);
                    readyMsg = dSS.gt.strargs(
                                    dSS._("Cache ready for <b>%1</b> of " +
                                          "<b>%2</b> devices."),
                                    [
                                        json.data.attributes.completed,
                                        json.data.attributes.total
                                    ]);

                    var updateStatus = dSS._("Readout status") + ": ";

                    updateStatus += dSS._(json.data.attributes.status);
                    if (json.data.attributes.status === "incomplete") {
                        this.updateButton.enable();
                    } else {
                        this.updateButton.disable();
                    }

                    this.mainSet.statusLabel.update(updateStatus);

                    var secs = parseInt(json.data.attributes.remainingTime, 10);
                    var hours = Math.floor(secs / 3600);
                    var minutes = Math.ceil(secs / 60) % 60;

                    var remaining = "";
                    if (hours > 0) {
                        remaining =
                            dSS11.config.gt.strargs(
                                dSS11.config.gt.ngettext("%1 hour", "%1 hours",
                                    hours), [ hours ]) + " ";
                    }
                    if (minutes > 0) {
                        remaining +=
                            dSS11.config.gt.strargs(
                                dSS11.config.gt.ngettext("%1 minute",
                                    "%1 minutes", minutes), [ minutes ]);
                    }

                    this.mainSet.deviceLabel.update(readyMsg);
                    this.mainSet.progress.updateProgress(
                            json.data.attributes.completion / 100.0,
                            String(Math.floor(json.data.attributes.completion, 10)) + "%");
                    if (remaining.length === 0) {
                        this.mainSet.estimationLabel.update("");
                        this.mainSet.estimationLabelSpacer.setHeight(0);
                    } else {
                        this.mainSet.estimationLabel.update(dSS._("Current estimation") + ":&nbsp;<b>" + remaining + "</b>");
                        this.mainSet.estimationLabelSpacer.setHeight(10);
                    }

                    var i;
                    var store = dSS.data.Loader.getDeviceStore();

                    if (json.data.included && json.data.included.functionBlocks) {
                        for (i = 0; i < json.data.included.functionBlocks.length; i++) {
                            var entry = json.data.included.functionBlocks[i];
                            var record = store.getById(entry.id);
                            if ((record !== undefined) &&
                            (entry.attributes.completion !== undefined)) {
                                record.set('cacheReadout', entry.attributes.completion);
                                record.commit();
                            }
                        }
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                            dSS._("Could not determine device cache status") +
                            ": " + dSS._(error.message));
                }
                unmask();
            },
            failure: function() {
                 Ext.Msg.alert(_("Error"),
                           dSS._("Could not determine device cache status"));
                 unmask();
            }
        });
    },
    updateCache: function() {
        var mask = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
        Ext.Ajax.request({
            url: '/api/v1/installation/parameterCache/update',
            disableCaching: true,
            jsonData: Ext.util.JSON.encode({ mode: 'slow' }),
            scope: this,
            success: function(response) {
                try {
                    if (response.status  >= 300) {
                        Ext.Msg.alert(_("Error"),
                            dSS._("Could not trigger device cache update"));
                        return;
                    }
                    mask();
                    this.reloadData.defer(2000, this);
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                            dSS._("Could not trigger device cache update") +
                            ": " + dSS._(error.message));
                }
            },
            failure: function() {
                Ext.Msg.alert(_("Error"),
                        dSS._("Could not trigger device cache update"));
            }
        });
    },
    showDetails: function() {
        var details = new Ext.Window({
            cls: 'dsClsDeviceCacheInfo_Details',

            title: dSS._("Device Cache Details"),
            width: 600,
            height: 400,
            modal: true,
            layout: 'fit',
            items: [ { xtype: 'dssdevicecachedetails' } ],
            buttons:
            [
                {
                    text: dSS._("Close"),
                    handler: function() { details.close(); }
                }
            ]
        });
        details.show();
    },
    confirmResetCache: function () {
        var me = this;
        Ext.Msg.show({
            title: dSS._("Reset device cache"),
            msg: dSS._("If you encounter issues like incorrect device or room states inside the dS smart home " +
                "smartphone app, there might be errors inside the device cache. Resetting the cache will delete " +
                "all cached data and start a new readout. You will not loose any settings, but the readout " +
                "can take several hours to complete."),
            buttons: {
                ok: dSS._("Reset device cache"),
                cancel: dSS._("Cancel")
            },
            cls: 'x-window-with-padded-buttons',
            fn: function (buttonClicked) {
                if (buttonClicked === 'ok') {
                    me.resetCache();
                }
            }
         });
    },
    resetCache: function () {
        Ext.Ajax.request({
            url: '/cgi-bin/rmdssdevcache.sh',
            method: 'GET',
            scope: this,
            failure: function () {
                Ext.Msg.show({
                    title: dSS._("Device cache reset could not be initiated"),
                    msg: dSS._("Please check your local network connection and try again after a while."),
                    buttons: Ext.Msg.OK
                 });
            }
        });
    }
});

Ext.reg('dssdevicecacheinfo', dSS.DeviceCacheInfo);
