//= require <dSS/util/Util>

Ext.namespace('dSS', 'dSS.grid');

dSS.EmptyClickPrevention = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        this.enableCheckbox = new Ext.form.Checkbox({
            cls: 'dsClsEmptyClickPrevention_EnableCheckbox',

            labelStyle: ' width:239px',
            x: 230,
            y: 85
        });
        Ext.apply(this, {
            cls: 'dsClsEmptyClickPrevention',

            layout: 'form',
            bodyBorder: false,
            items:
            [
                {
                    xtype: 'fieldset',
                    title: dSS._("Empty Click Prevention"),
                    layout: 'absolute',
                    scope: this,
                    ref: 'mainSet',
                    height: 145,
                    items:
                    [
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            html: dSS._("In rare cases, empty clicks may " +
                                "occur during button operation, i.e. " +
                                "the system may not seem to react. If the device" +
                                " data have been read out completely, " +
                                "the system can avoid the empty" +
                                " clicks if this function is switched on." +
                                " If you encounter problems during operation," +
                                " please switch off this function, " +
                                "which is currently in beta state."),
                            x: 0,
                            y: 0
                        },
                        {
                            xtype: 'label',
                            text: dSS._("Enable empty click prevention") + ':',
                            x: 0,
                            y: 88
                        },
                        this.enableCheckbox,
                        {
                            cls: 'dsClsEmptyClickPrevention_ApplyButton',

                            xtype: 'button',
                            text: dSS._("Apply"),
                            minWidth: 90,
                            handler: this.setEmptyClickPrevention.createDelegate(this),
                            x: 275,
                            y: 85
                        }
                    ]
                }
            ]
        });

        dSS.EmptyClickPrevention.superclass.initComponent.apply(this, arguments);
        dSS.MessageBus.on('refreshview', this.reloadData.createDelegate(this),
            this);
    },
    setEmptyClickPrevention:  function(enable) {
        dSS.util.genericRequest({
            url: '/json/apartment/setEmptyClickPreventionActive',
            method: 'GET',
            scope: this,
            params: {
                active: this.enableCheckbox.getValue() ? 1 : 0
            },
            maskEl: this.ownerCt.el,
            maskMsg: dSS._("Applying changes") + '...',
            failureMsg: dSS._("Could not set empty click prevention")
        });
    },
    getEmptyClickPrevention:  function() {
        dSS.util.genericRequest({
            url: '/json/apartment/getEmptyClickPreventionActive',
            method: 'GET',
            scope: this,
            maskEl: this.ownerCt.el,
            maskMsg: dSS._("Loading") + '...',
            failureMsg: dSS._("Could not get empty click prevention")
        }, (success, data) => {
            if ((success === true) &&
                (data.result !== undefined) &&
                (data.result.emptyClickPreventionActive !== undefined)) {
                this.enableCheckbox.setValue(data.result.emptyClickPreventionActive);
            }
        });
    },
    reloadData: function() {
        this.getEmptyClickPrevention()
    },
});

Ext.reg('dssemptyclickprevention', dSS.EmptyClickPrevention);
