//= require <external/IconCombo>

Ext.namespace("dSS", "dSS.config");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
dSS.BasicGroupForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {

        this.group = config.group;

        this.groupName = "";
        this.groupColor = dSS.config.groups.LIGHT;

        if (this.group) {
            if (this.group.get('name') !== undefined) {
                this.groupName = this.group.get('name');
            }

            this.groupID = this.group.get('id');

            if (this.group.get('color') !== undefined) {
                this.groupColor = this.group.get('color');
            }
        }

        this.uCss = 'UserGroup';

        var list =
        [
            [
                dSS.config.groups.LIGHT,
                dSS.config.GroupColorIconCls[dSS.config.deviceClass.GE] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.LIGHT])
            ],
            [
                dSS.config.groups.SHADOW,
                dSS.config.GroupColorIconCls[dSS.config.deviceClass.GR] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.SHADOW])
            ],
            [
                dSS.config.groups.HEATING,
                dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.HEATING])
            ],
            [
                dSS.config.groups.COOLING,
                dSS.config.GroupColorIconCls[dSS.config.deviceClass.BL] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.COOLING])
            ],
            [
                dSS.config.groups.AUDIO,
                dSS.config.GroupColorIconCls[dSS.config.deviceClass.TK] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.AUDIO])
            ],
            [
                dSS.config.groups.VIDEO,
                dSS.config.GroupColorIconCls[dSS.config.deviceClass.MG] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.VIDEO])
            ]
        ];

        this.addEvents('formsubmitted', 'formloaded', 'formchanged',
                       'formretry');

        this.colorSelection = new Ext.ux.IconCombo({
            cls: 'dsClsBasicGroupForm_ColorSelection',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            disabled: (config.fixedColor === true),
            width: 150,
            mode: 'local',
            fieldLabel: dSS._("Color group"),
            tpl: new Ext.XTemplate(
                '<tpl for="."><div class="x-combo-list-item">',
                '<table><tbody><tr>',
                '<td>',
                '<div class="{css} x-icon-combo-icon"></div></td>',
                '<td>{displayText}</td>',
                '</tr></tbody></table>',
                '</div></tpl>'),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'css',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText',
            iconClsField: 'css'
        });

        this.groupNameField = new Ext.form.TextField({
            cls: 'dsClsBasicGroupForm_GroupNameField',

            fieldLabel: dSS._("Group name"),
            emptyText: dSS._("Enter group name..."),
            allowBlank: false,
            width: 150
        });

        Ext.apply(this, {
            cls: 'dsClsBasicGroupForm',

            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'formchanged', 'formretry'
            ],
            padding: 5,
            scope: this,
            layout: 'fit',
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    ref: 'content',
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'box',
                            height: 5
                        },
                        this.groupNameField,
                        this.colorSelection
                    ]
                }
            ]
        });

        dSS.BasicGroupForm.superclass.constructor.apply(this, arguments);

        this.colorSelection.on('afterrender', function() {
                if ((this.closed !== true) &&
                   (((this.groupColor >= dSS.config.groups.LIGHT) &&
                     (this.groupColor < dSS.config.groups.JOKER)) ||
                    (this.groupColor == dSS.config.groups.COOLING) ||
                    (this.groupColor == dSS.config.groups.VENTILATION))) {
                    this.colorSelection.setValue(this.groupColor);
                }
                this.colorSelection.on('select', function() {
                    if (this.groupNameField.isValid()) {
                        this.fireEvent('formchanged');
                    }
                },
                this);
            },
            this);

         this.groupNameField.on('afterrender', function() {
                if (this.closed !== true) {
                    this.groupNameField.setValue(this.groupName);
                    this.groupNameField.on('valid', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);
    }
});

dSS.AddGroupForm = Ext.extend(dSS.BasicGroupForm, {
    initComponent: function() {
        dSS.AddGroupForm.superclass.initComponent.apply(this, arguments);

        this.on('submit', function() {
            if (this.groupNameField.isValid() !== true) {
                Ext.Msg.alert(
                    dSS._("Error"),
                    dSS._("Please enter a valid group name!"),
                    this.fireEvent('formretry'));
                return;
            }

            var color = this.colorSelection.getValue();
            var name = this.groupNameField.getValue();

            Ext.Ajax.request({
                url: '/json/structure/addCluster',
                method: "GET",
                params:
                {
                    applicationType: color,
                    name: name
                },
                scope: this,
                success: function(result, request) {
                    try {
                        var json = Ext.util.JSON.decode(result.responseText);
                        if (!json.ok) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not add new group") + ": " +
                                json.message);
                            this.fireEvent('formsubmitted');
                            return;
                        }
                        var groupStore = dSS.data.Loader.getGroupStore();
                        var record = {
                            name: name,
                            id: json.result.clusterID,
                            sortorder: 200,
                            color: json.result.color,
                            css: dSS.config.GroupColorIconCls[json.result.color] +
                                                              this.uCss
                        };
                        groupStore.loadData({ groups: [ record ] }, true);
                        dSS.MessageBus.fireEvent('statusmessage',
                                dSS.gt.strargs(dSS._("Group #%1 was created"),
                                        [json.result.groupID]));
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not add new group") + ": " + err);
                    }
                    this.fireEvent('formsubmitted');
                },
                failure: function(result, request) {
                    var message = '';
                    try {
                        var json = Ext.util.JSON.decode(result.responseText);
                        if ((!json.ok) && (json.message.length > 0)) {
                            message = ': ' + json.message;
                        }
                    } catch(err) {}
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not create new group") +
                                  message);

                    this.fireEvent('formsubmitted');
                }
            });
        },
        this);
    }
});


Ext.reg('dssaddgroupform', dSS.AddGroupForm);

dSS.EditGroupForm = Ext.extend(dSS.BasicGroupForm, {
    initComponent: function() {
        this.loadedLock = false;
        this.loadedWindLock = false;
        this.loadedRainLock = false;
        this.loadedFrostLock = false;
        this.loadedHailLock = false;

        dSS.EditGroupForm.superclass.initComponent.apply(this, arguments);

        if (dSS.util.isUserGroup(this.group.get('id'))) {
            if (this.group.get('isAutomatic')) {
                this.content.add(new Ext.BoxComponent({
                    height: 10
                }));
                this.content.add(new Ext.form.Label({
                    cls: 'dsClsEditGroupForm_OrientationLabel',
                    
                    fieldLabel: dSS._("Orientation"),
                    text: dSS._(dSS.util.getClusterLocationString(this.group.get('direction')))
                }));
                this.content.add(new Ext.form.Label({
                    cls: 'dsClsEditGroupForm_WindProtectionLabel',
                    
                    fieldLabel: dSS._("Wind protection"),
                    text:  dSS._(dSS.util.getWindProtectionString(this.group.get('protection')))
                }));
                this.content.add(new Ext.BoxComponent({ height: 10 }));
            }

            this.lock = new Ext.form.Checkbox({
                cls: 'dsClsEditGroupForm_ConfigurationLockCheckbox',

                fieldLabel: dSS._("Configuration lock"),
                labelStyle: ' width:239px'
            });

            this.lock.on('afterrender', function() {
                if (this.closed !== true) {
                    this.lock.setValue(this.group.get('locked'));
                    this.loadedLock = this.group.get('locked');
                    this.lock.on('check', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);

            this.content.add(this.lock);

            var spacer1 = new Ext.BoxComponent({ height: 10, width: 10 });
            this.content.add(spacer1);

            this.oplabel = new Ext.form.Label({
                cls: 'dsClsEditGroupForm_OpLocksLabel',
                
                text: dSS._("Operation locks"),
                cls: 'bold-text'
            });
            this.content.add(this.oplabel);

            var spacer2 = new Ext.BoxComponent({ height: 12, width: 10 });
            this.content.add(spacer2);

            this.windlock = new Ext.form.Checkbox({
                cls: 'dsClsEditGroupForm_WindLockCheckbox',
                
                fieldLabel: dSS._("Enabled for wind"),
                labelStyle: ' width:239px'
            });

            this.windlock.on('afterrender', function() {
                if (this.closed !== true) {
                    this.windlock.setValue(this.group.get('windlock'));
                    this.loadedWindLock = this.group.get('windlock');
                    this.windlock.on('check', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);
//            this.content.add(this.windlock);

            this.rainlock = new Ext.form.Checkbox({
                cls: 'dsClsEditGroupForm_RainLockCheckbox',
                
                fieldLabel: dSS._("Enabled for rain"),
                labelStyle: ' width:239px'
            });

            this.rainlock.on('afterrender', function() {
                if (this.closed !== true) {
                    this.rainlock.setValue(this.group.get('rainlock'));
                    this.loadedRainLock = this.group.get('rainlock');
                    this.rainlock.on('check', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);
            this.content.add(this.rainlock);

            this.frostlock = new Ext.form.Checkbox({
                cls: 'dsClsEditGroupForm_FrostLockLabel',
                
                fieldLabel: dSS._("Enabled for frost"),
                labelStyle: ' width:239px'
            });

            this.frostlock.on('afterrender', function() {
                if (this.closed !== true) {
                    this.frostlock.setValue(this.group.get('frostlock'));
                    this.loadedFrostLock = this.group.get('frostlock');
                    this.frostlock.on('check', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);
//            this.content.add(this.frostlock);

            this.haillock = new Ext.form.Checkbox({
                cls: 'dsClsEditGroupForm_HailLockCheckbox',
                
                fieldLabel: dSS._("Enabled for hail"),
                labelStyle: ' width:239px'
            });

            this.haillock.on('afterrender', function() {
                if (this.closed !== true) {
                    this.haillock.setValue(this.group.get('haillock'));
                    this.loadedHailLock = this.group.get('haillock');
                    this.haillock.on('check', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);
//            this.content.add(this.haillock);

            this.oplockpanel = new Ext.Panel({
                cls: 'dsClsGroupDialog_OpLockPanel',

                border: false,
                layout: 'form',
                items:
                [
                    this.windlock,
                    this.rainlock,
                    this.frostlock,
                    this.haillock
                ]
            });
            this.content.add(this.oplockpanel);
            this.content.doLayout();
            if (this.groupColor != dSS.config.groups.SHADOW) {
                this.oplockpanel.items.each(function(item) {
                    item.disable();
                });
            }
        }

        this.colorSelection.on('afterrender', function() {
            this.colorSelection.on('select', function(combo, record) {
                if (record.get('id') == dSS.config.groups.SHADOW) {
                    this.oplockpanel.items.each(function(item) {
                        item.enable();
                    });
                } else {
                    this.oplockpanel.items.each(function(item) {
                        item.disable();
                    });
                }
            }, this);
        }, this);

        this.on('submit', function() {
            if (this.groupNameField.isValid() !== true) {
                Ext.Msg.alert(
                    dSS._("Error"),
                    dSS._("Please enter a valid group name!"),
                    this.fireEvent('formretry'));
                return;
            }

            var showConfLockWarning = false;
            var showOpLockWarning = false;

            if ((this.lock && (this.loadedLock !== this.lock.getValue()) &&
                 (this.lock.getValue() == false))) {
                showConfLockWarning = true;
            }

            if (this.colorSelection.getValue() == dSS.config.groups.SHADOW) {
                if (((this.windlock && (this.loadedWindLock != this.windlock.getValue()) && this.windlock.getValue() == false)) ||
                    ((this.rainlock && (this.loadedRainLock != this.rainlock.getValue()) && this.rainlock.getValue() == false)) ||
                    ((this.frostlock && (this.loadedFrostLock != this.frostlock.getValue()) && this.frostlock.getValue() == false)) ||
                    ((this.haillock && (this.loadedHailLock != this.haillock.getValue()) && this.haillock.getValue() == false))) {
                    showOpLockWarning = true;
                }
            }

            var message;

            if ((showConfLockWarning === true) && (showOpLockWarning === true)) {
                message = "By disabling configuration or opeartion locks, changes of security relevant settings are possible. By continuing you acknowledge that you know what you are doing and take full responsibility for any resulting actions. Please remember to lock this group again once all modifications are completed.";
            } else {
                if (showConfLockWarning == true) {
                    message = "By disabling the configuration lock, changes of security relevant settings are possible. By continuing you acknowledge that you know what you are doing and take full responsibility for any resulting actions. Please remember to lock this group again once all modifications are completed.";
                }
                
                if (showOpLockWarning == true) {
                 message = "By disabling opeartion locks, changes of security relevant settings are possible. By continuing you acknowledge that you know what you are doing and take full responsibility for any resulting actions. Please remember to lock this group again once all modifications are completed.";
                }
            }

            if (message !== undefined) {
                Ext.Msg.show({
                    title: dSS._("Unlock configuration"),
                    msg: dSS._(message),
                    buttons: Ext.Msg.YESNO,
                    icon: Ext.MessageBox.QUESTION,
                    scope: this,
                    animEl: 'elId',
                    fn: function(button, text) {
                        if (button == 'yes') {
                            this.saveSettings();
                        } else {
                        	if (this.lock.getValue() == false) {
	                            this.lock.setValue(this.loadedLock);
                            }

                            if (this.colorSelection.getValue() == dSS.config.groups.SHADOW) {
                                if (this.windlock.getValue() == false) {
                                    this.windlock.setValue(this.loadedWindLock);
                                }
                                if (this.rainlock.getValue() == false) {
                                    this.rainlock.setValue(this.loadedRainLock);
                                }
                                if (this.frostlock.getValue() == false) {
                                    this.frostlock.setValue(this.loadedFrostLock);
                                }
                                if (this.haillock.getValue() == false) {
                                    this.haillock.setValue(this.loadedHailLock);
                                }
                            }
                            this.fireEvent('formretry')
                        }
                    }
                });

                return;
            } else {
                this.saveSettings();
            }
        },
        this);
    },
    saveSettings: function() {
        var color = this.colorSelection.getValue();
        var name = this.groupNameField.getValue();
        var delay = 1000;
        var sendEvent = true;
        var exit = true;
        if (name !== this.groupName) {
            exit = false;
            this.setGroupName(name);
            sendEvent = false;
        } else {
            delay = 0;
        }
        if (color !== this.groupColor) {
            exit = false;
            this.setGroupColor.defer(delay, this, [ color, sendEvent ]);
            delay = delay + 1000;
        }

        if (this.lock && (this.loadedLock !== this.lock.getValue())) {
            this.setClusterLock(this.groupID, this.lock.getValue());
        }

        if (color == dSS.config.groups.SHADOW) {
            var addWithComma = function(str, add) {
                if (str.length > 0) {
                    str = str + ',';
                }
                str = str + add;
                return str;
            }

            var statesToAdd = "";
            var statesToRemove = "";

            if (this.windlock && (this.loadedWindLock !== this.windlock.getValue())) {
                if (this.windlock.getValue() == false) {
                    statesToRemove = addWithComma(statesToRemove,
                            dSS.config.ClusterOperationLockState.WIND);
                } else {
                    statesToAdd = addWithComma(statesToAdd,
                            dSS.config.ClusterOperationLockState.WIND);
                }
            }

            if (this.rainlock && (this.loadedRainLock !== this.rainlock.getValue())) {
                if (this.rainlock.getValue() == false) {
                    statesToRemove = addWithComma(statesToRemove,
                            dSS.config.ClusterOperationLockState.RAIN);
                } else {
                    statesToAdd = addWithComma(statesToAdd,
                            dSS.config.ClusterOperationLockState.RAIN);
                }
            }

            if (this.frostlock && (this.loadedFrostLock !== this.frostlock.getValue())) {
                if (this.frostlock.getValue() == false) {
                    statesToRemove = addWithComma(statesToRemove,
                            dSS.config.ClusterOperationLockState.FROST);
                } else {
                    statesToAdd = addWithComma(statesToAdd,
                            dSS.config.ClusterOperationLockState.FROST);
                }
            }

            if (this.haillock && (this.loadedHailLock !== this.haillock.getValue())) {
                if (this.haillock.getValue() == false) {
                    statesToRemove = addWithComma(statesToRemove,
                            dSS.config.ClusterOperationLockState.HAIL);
                } else {
                    statesToAdd = addWithComma(statesToAdd,
                            dSS.config.ClusterOperationLockState.HAIL);
                }
            }

            if (statesToAdd.length > 0) {
                this.setOperationLock.defer(delay, this,
                        [ this.groupID, statesToAdd, 'add' ]);
                delay = delay + 1000;
            }

            if (statesToRemove.length > 0) {
                this.setOperationLock.defer(delay, this,
                        [ this.groupID, statesToRemove, 'remove' ]);
            }
        }

        if (exit === true) {
            this.fireEvent('formsubmitted');
        }
    },
    setClusterLock: function(id, lock) {
        Ext.Ajax.request({
            url: '/json/structure/clusterSetConfigLock',
            method: "GET",
            params:
            {
                clusterID: id,
                lock: (lock === true) ? 1 : 0
            },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set cluster lock") + ": " +
                            json.message);
                        return;
                    }
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var record = groupStore.getById(this.groupID);
                    if (record !== undefined) {
                        record.set('locked', lock);
                        record.set('locktip', dSS.util.GetLockString(record));
                        record.commit();
                        dSS.MessageBus.fireEvent('groupselection', record, !lock);
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not set cluster lock") + ": " + err);
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((!json.ok) && (json.message.length > 0)) {
                        message = ': ' + json.message;
                    }
                } catch(err) {}
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not set cluster lock") +
                              message);
            }
        });
    },
    setOperationLock: function(id, states, action) {
        var url = '/json/structure/clusterAddLockEvents';
        var lock = true;
        if (action == 'remove') {
          url = '/json/structure/clusterRemoveLockEvents';
          lock = false;
        }
        Ext.Ajax.request({
            url: url,
            method: "GET",
            params:
            {
                clusterID: id,
                names: states
            },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not modify operation lock") + ": " +
                            json.message);
                        return;
                    }
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var record = groupStore.getById(this.groupID);
                    if (record !== undefined) {
                        var arr = states.split(',');
                        var i;
                        for (i = 0; i < arr.length; i++) {
                          record.set(arr[i] + 'lock', lock);
                          record.commit();
                        }
                        record.set('locktip', dSS.util.GetLockString(record));
                        record.commit();
                    }
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not modify operation lock") + ": " + err);
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((!json.ok) && (json.message.length > 0)) {
                        message = ': ' + json.message;
                    }
                } catch(err) {}
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not modify operation lock") +
                              message);
            }
        });
    },

    setGroupName: function(name) {
        Ext.Ajax.request({
            url: '/json/structure/clusterSetName',
            method: "GET",
            params:
            {
                clusterID: this.groupID,
                newName: name
            },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set group name") + ": " +
                            json.message);
                        this.fireEvent('formsubmitted');
                        return;
                    }
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var record = groupStore.getById(this.groupID);
                    if (record !== undefined) {
                        record.set('name', name);
                        record.commit();
                        groupStore.doSort();
                    }
                    dSS.MessageBus.fireEvent('statusmessage',
                            dSS.gt.strargs(dSS._("Group #%1 was renamed"),
                                    [this.groupID]));
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not rename group") + ": " + err);
                }
                this.fireEvent('formsubmitted');
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((!json.ok) && (json.message.length > 0)) {
                        message = ': ' + json.message;
                    }
                } catch(err) {}
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not rename group") +
                              message);

                this.fireEvent('formsubmitted');
            }
        });
    },
    setGroupColor: function(color, sendEvent) {
        Ext.Ajax.request({
            url: '/json/structure/clusterSetColor',
            method: "GET",
            params:
            {
                clusterID: this.groupID,
                newColor: color
            },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set group color") + ": " +
                            json.message);
                        if (sendEvent === true) {
                            this.fireEvent('formsubmitted');
                        }
                        return;
                    }
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var record = groupStore.getById(this.groupID);
                    if (record !== undefined) {
                        record.set('color', color);
                        record.set('css', dSS.config.GroupColorIconCls[color] +
                                          this.uCss);
                        record.commit();
                        groupStore.doSort();
                    }
                    dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Group") + ' #' + this.groupID +
                            ' ' + dSS._("color has been changed"));
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not set group color") + ": " + err);
                }
                if (sendEvent === true) {
                    this.fireEvent('formsubmitted');
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((!json.ok) && (json.message.length > 0)) {
                        message = ': ' + json.message;
                    }
                } catch(err) {}
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not set group color") +
                              message);

                if (sendEvent === true) {
                    this.fireEvent('formsubmitted');
                }
            }
        });
    }
});


Ext.reg('dsseditgroupform', dSS.EditGroupForm);
