dSS.SaveSceneDialogWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        this.maskElement = config.maskElement;
        this.sceneList = [];
        if (config.sceneList !== undefined) {
            this.sceneList = config.sceneList;
        }
        this.caller = config.caller;

        this.selectionStore = new Ext.data.ArrayStore({
            id: 0,
            fields:
            [
                'id', // scene number
                'zoneId',
                'groupId',
                'css',
                'displayText'
            ],
            data: this.sceneList
        });

        this.sceneSelection = new Ext.ux.IconCombo({
            cls: 'dsClsSaveSceneDialogWindow_SceneSelectionCombo',

            x: 0,
            y: 0,
            width: 235,
            typeAhead: false,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: true,
            editable: false,
            mode: 'local',
            tpl: new Ext.XTemplate(
                '<tpl for="."><div class="x-combo-list-item">',
                '<table><tbody><tr>',
                '<td>',
                '<div class="{css} x-icon-combo-icon"></div></td>',
                '<td>{displayText}</td>',
                '</tr></tbody></table>',
                '</div></tpl>'),
            store: this.selectionStore,
            valueField: 'id',
            displayField: 'displayText',
            iconClsField: 'css'
        });

        this.saveButton = new Ext.Button({
            cls: 'dsClsSaveSceneDialogWindow_SaveButton',
            text: dSS._("Save"),
            disabled: true,
            scope: this,
            handler: this.saveScene
        });

        Ext.apply(this, {
            cls: 'dsClsSaveSceneDialogWindow',

            layout: 'fit',
            width: 300,
            resizable: false,
            title: dSS._("Save settings to another scene"),
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 150,
                    padding: 10,
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("You are about to save your current settings to a different scene. Please select the taget scene from the list") + ':'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'panel',
                            layout: 'absolute',
                            border: false,
                            height: 40,
                            items:
                            [
                                this.sceneSelection,
                                {
                                    cls: 'dsClsSaveSceneDialogWindow_DisplayAllScenesButton',

                                    xtype: 'button',
                                    iconCls: 'showAllScenesIcon',
                                    tooltip: dSS._("Display all available scenes"),
                                    pressed: config.sceneToggleState,
                                    enableToggle: true,
                                    handler: this.reloadScenes.createDelegate(this),
                                    x: 242,
                                    y: 0
                                }
                            ]
                        }
                    ]
                }
            ],
            buttons:
            [
                this.saveButton,
                {
                    text: dSS._("Close"),
                    scope: this,
                    handler: this.close.createDelegate(this)
                }
            ]
        });

        dSS.SaveSceneDialogWindow.superclass.constructor.apply(this, arguments);

        this.sceneSelection.on('select', function() {
                if (this.saveButton.disabled === true) {
                    this.saveButton.enable();
                }
            },
            this);
        this.on('afterrender', function() {
                if (this.maskElement !== undefined) {
                    this.maskElement.mask();
                }
            },
            this);

        this.on('close', function() {
                if (this.maskElement !== undefined) {
                    this.maskElement.unmask();
                }
            },
            this);

        this.caller.on('treeredrawcomplte', function() {
                var currentSelection = this.sceneSelection.getValue();
                this.selectionStore.loadData(this.caller.getSelectedSceneList());
                if ((currentSelection !== undefined) &&
                    (currentSelection != "")) {
                    var rec = this.selectionStore.getById(currentSelection);
                    if (rec === undefined) {
                        this.sceneSelection.clearValue();
                    }
                }
                this.el.unmask();
            },
            this);
    },
    warnSaveScene0: function() {
        var msg = dSS._("You are about to save scene 0 which represents " +
                        "the off scene. Note that you might not be able to " +
                        "switch off all devices in case some devices in the " +
                        "selected room are turned on right now.") + '<br/>' +
                  dSS._("Are you sure you want to save the scene?");
        Ext.Msg.confirm(
            dSS._("Warning"),
            msg,
            function(btn) {
                if (btn === 'yes') {
                    this.saveScene(false);
                }
            },
            this
        );
    },
    saveScene: function(warn0) {
        var sceneNumber = this.sceneSelection.getValue();
        if ((sceneNumber === 0) && (warn0 !== false)) {
            this.warnSaveScene0();
            return;
        }
        var rec = this.selectionStore.getById(sceneNumber);
        if (rec !== undefined) {
            var zoneId = rec.get('zoneId');
            if (isNaN(zoneId)) {
                zoneId = 0;
            }
            dSS.util.saveSceneOnZone(zoneId, sceneNumber,
                                     rec.get('groupId'));
        }
        this.saveButton.disable();
    },
    reloadScenes: function(button) {
        this.el.mask(dSS._("Loading scenes") + '...', 'x-mask-loading');
        this.caller.showAllScenes(button);
    }
});
