/* globals Ext, dSS */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

dSS.config.features = {};
dSS.config.settings = {};
dSS.config.dSSTimeDiff = 0;
dSS.config.firstLoad = true;

dSS.config.dSMSystemFirmwareImageVersion = {
    ARM: '',
    DSP: ''
};

dSS.config.language = 'en_US';
dSS.config.langPath = '/locale/' + dSS.config.language + '/LC_MESSAGES/dss-langpack.json';

dSS.config.dialogSettings = {
    de_DE: {
        shadePropertyWindowWidth: 500,
        heatingPropertyWindowWidth: 520,
        deviceSettingsWindowWidth: 460,
        finsCalibrationWindowWidth: 500
    },
    fr_CH: {
        shadePropertyWindowWidth: 685,
        heatingPropertyWindowWidth: 600,
        deviceSettingsWindowWidth: 465,
        finsCalibrationWindowWidth: 600
    },
    en_US: {
        shadePropertyWindowWidth: 510,
        heatingPropertyWindowWidth: 520,
        deviceSettingsWindowWidth: 450,
        finsCalibrationWindowWidth: 500
    },
    tr_TR: {
        shadePropertyWindowWidth: 500,
        heatingPropertyWindowWidth: 550,
        deviceSettingsWindowWidth: 465,
        finsCalibrationWindowWidth: 500
    },
    pt_PT: {
        heatingPropertyWindowWidth: 610
    },
    da_DK: {
        heatingPropertyWindowWidth: 530
    },
    it_IT: {
        heatingPropertyWindowWidth: 530
    }
};

dSS.config.zones = {
    BROADCAST: 0,
    NEW_DEVICES: 65534
};

dSS.config.floors = {
    NONE: -1,
    APARTMENT: 0
};


dSS.config.groups = {
    VARIABLE: 0,
    LIGHT: 1,
    SHADOW: 2,
    HEATING: 3, // new term: "Heating manual"
    AUDIO: 4,
    VIDEO: 5,
    SECURITY: 6,
    ACCESS: 7,
    JOKER: 8,
    COOLING: 9,
    VENTILATION: 10,
    WINDOW: 11,
    RECIRCULATION: 12,
    TEMPCONTROL: 48, // new term: "Heating/Cooling regulated"
    APT_VENTILATION: 64,
    SINGLEDEVICES: 309 // fake UI-only group for filtering white devices
};

dSS.config.groupNames = {
    0: 'Broadcast',
    1: 'Light',
    2: 'Shadow',
    3: 'Heating',
    4: 'Audio',
    5: 'Video',
    6: 'Reserved',
    7: 'Reserved',
    8: 'Joker',
    9: 'Cooling',
    10: 'Room ventilation',
    11: 'Window',
    12: 'Recirculation',
    13: 'Awnings',
    48: 'Temperature control',
    64: 'Apartment ventilation'
};


dSS.config.scenes = {
    DEC: 11,
    INC: 12,
    MIN: 13,
    MAX: 14,
    STOP: 15,
    T0_S0: 0,
    T1_S0: 1,  // off scene area 1
    T2_S0: 2,  // off scene area 2
    T3_S0: 3,  // off scene area 3
    T4_S0: 4,  // off scene area 4
    T0_S1: 5,
    T0_S2: 17,
    T0_S3: 18,
    T0_S4: 19,
    T1_S1: 6,  // mood area 1
    BOOST: 6,  // ventilation/recirculation: boost
    T2_S1: 7,  // mood area 2
    T3_S1: 8,  // mood area 3
    T4_S1: 9,  // mood area 4
    EX5: 32, // mood 5
    EX6: 33, // mood 6
    T1_S2: 20, // mood 7
    T1_S3: 21, // mood 8
    T1_S4: 22, // mood 9
    EX10: 34, // mood 10
    EX11: 35, // mood 11
    T2_S2: 23, // mood 12
    T2_S3: 24, // mood 13
    T2_S4: 25, // mood 14
    EX15: 36, // mood 15
    EX16: 37, // mood 16
    T3_S2: 26, // mood 17
    T3_S3: 27, // mood 18
    T3_S4: 28, // mood 19
    EX20: 38, // mood 20
    EX21: 39, // mood 21
    T4_S2: 29, // mood 22
    T4_S3: 30, // mood 23
    T4_S4: 31, // mood 24
    AUTO_OFF: 40,
    LOCAL_OFF: 50,
    LOCAL_ON: 51,
    T1_STOP: 52,
    T2_STOP: 53,
    T3_STOP: 54,
    T4_STOP: 55,
    SUNPROTECTION: 56,
    PANIC: 65,
    STANDBY: 67,
    DEEP_OFF: 68,
    SLEEPING: 69,
    WAKE_UP: 70,
    PRESENT: 71,
    ABSENT: 72,
    SIG_BELL: 73,
    ALARM1: 74,
    FIRE: 76,
    ALARM2: 83,
    ALARM3: 84,
    ALARM4: 85,
    WIND: 86,
    NO_WIND: 87,
    RAIN: 88,
    NO_RAIN: 89,
    HAIL: 90,
    NO_HAIL: 91
};

dSS.config.ApartmentScenes = [
    dSS.config.scenes.ABSENT,
    dSS.config.scenes.PRESENT,
    dSS.config.scenes.SIG_BELL,
    dSS.config.scenes.RAIN,
    dSS.config.scenes.NO_RAIN,
    dSS.config.scenes.WIND,
    dSS.config.scenes.NO_WIND,
    dSS.config.scenes.HAIL,
    dSS.config.scenes.NO_HAIL,
    dSS.config.scenes.PANIC,
    dSS.config.scenes.FIRE,
    dSS.config.scenes.ALARM1,
    dSS.config.scenes.ALARM2,
    dSS.config.scenes.ALARM3,
    dSS.config.scenes.ALARM4
];

dSS.config.sceneNamesAV = {
    0: 'Off',
    5: 'Level 1',
    17: 'Level 2',
    18: 'Level 3',
    19: 'Level 4',
    6: 'Boost'
};

dSS.config.sceneNamesShadow = {
    0: 'Down',
    1: 'Area 1 down',
    2: 'Area 2 down',
    3: 'Area 3 down',
    4: 'Area 4 down',
    5: 'Up',
    6: 'Area 1 up',
    7: 'Area 2 up',
    8: 'Area 3 up',
    9: 'Area 4 up'
};

dSS.config.sceneNames = {
    0: 'Off',
    1: 'Area 1 off',
    2: 'Area 2 off',
    3: 'Area 3 off',
    4: 'Area 4 off',
    5: 'Scene 1',
    17: 'Scene 2',
    18: 'Scene 3',
    19: 'Scene 4',
    40: 'Auto off',
    56: 'Sun protection',
    65: 'Panic',
    67: 'Standby',
    68: 'Deep off',
    69: 'Sleeping',
    70: 'Wake up',
    71: 'Coming home',
    72: 'Leave home',
    73: 'Door bell',
    74: 'Alarm 1',
    76: 'Fire',
    83: 'Alarm 2',
    84: 'Alarm 3',
    85: 'Alarm 4',
    86: 'Wind',
    87: 'No wind',
    88: 'Rain',
    89: 'No rain',
    90: 'Hail',
    91: 'No hail',
    6: 'Area 1 on',
    7: 'Area 2 on',
    8: 'Area 3 on',
    9: 'Area 4 on',
    32: 'Scene 10',
    33: 'Scene 11',
    20: 'Scene 12',
    21: 'Scene 13',
    22: 'Scene 14',
    34: 'Scene 20',
    35: 'Scene 21',
    23: 'Scene 22',
    24: 'Scene 23',
    25: 'Scene 24',
    36: 'Scene 30',
    37: 'Scene 31',
    26: 'Scene 32',
    27: 'Scene 33',
    28: 'Scene 34',
    38: 'Scene 40',
    39: 'Scene 41',
    29: 'Scene 42',
    30: 'Scene 43',
    31: 'Scene 44'
};

dSS.config.sceneNamesMsg = {
    0: 'Off',
    1: 'Area 1',
    2: 'Area 2',
    3: 'Area 3',
    4: 'Area 4',
    5: 'Scene 1',
    17: 'Scene 2',
    18: 'Scene 3',
    19: 'Scene 4',
    40: 'Auto off',
    56: 'Sun protection',
    65: 'Panic',
    67: 'Standby',
    68: 'Deep off',
    69: 'Sleeping',
    70: 'Wake up',
    71: 'Coming home',
    72: 'Leave home',
    73: 'Door bell',
    74: 'Alarm 1',
    76: 'Fire',
    83: 'Alarm 2',
    84: 'Alarm 3',
    85: 'alarm 4',
    86: 'Wind',
    87: 'No wind',
    88: 'Rain',
    89: 'No rain',
    90: 'Hail',
    91: 'No hail',
    6: 'Area 1',
    7: 'Area 2',
    8: 'Area 3',
    9: 'Area 4',
    32: 'Scene 10',
    33: 'Scene 11',
    20: 'Scene 12',
    21: 'Scene 13',
    22: 'Scene 14',
    34: 'Scene 20',
    35: 'Scene 21',
    23: 'Scene 22',
    24: 'Scene 23',
    25: 'Scene 24',
    36: 'Scene 30',
    37: 'Scene 31',
    26: 'Scene 32',
    27: 'Scene 33',
    28: 'Scene 34',
    38: 'Scene 40',
    39: 'Scene 41',
    29: 'Scene 42',
    30: 'Scene 43',
    31: 'Scene 44'
};

dSS.config.sceneTips = {
    0: 'Off = scene 0 (pushbutton scene 0-4)',
    1: 'Area 1 off',
    2: 'Area 2 off',
    3: 'Area 3 off',
    4: 'Area 4 off',
    5: 'Zone pushbutton scene 0-4',
    17: 'Zone pushbutton scene 0-4 / device pushbutton and area pushbutton scene 2-4',
    18: 'Zone pushbutton scene 0-4 / device pushbutton and area pushbutton scene 2-4',
    19: 'Zone pushbutton scene 0-4 / device pushbutton and area pushbutton scene 2-4',
    6: 'Area pushbutton 1',
    7: 'Area pushbutton 2',
    8: 'Area pushbutton 3',
    9: 'Area pushbutton 4',
    32: 'Zone pushbutton scene 10-14',
    33: 'Zone pushbutton scene 10-14',
    20: 'Zone pushbutton scene 10-14 / area pushbutton 1 scene 12-14',
    21: 'Zone pushbutton scene 10-14 / area pushbutton 1 scene 12-14',
    22: 'Zone pushbutton scene 10-14 / area pushbutton 1 scene 12-14',
    34: 'Zone pushbutton scene 20-24',
    35: 'Zone pushbutton scene 20-24',
    23: 'Zone pushbutton scene 20-24 / area pushbutton 2 scene 22-24',
    24: 'Zone pushbutton scene 20-24 / area pushbutton 2 scene 22-24',
    25: 'Zone pushbutton scene 20-24 / area pushbutton 2 scene 22-24',
    36: 'Zone pushbutton 30-34',
    37: 'Zone pushbutton 30-34',
    26: 'Zone pushbutton 30-34 / area pushbutton 3 scene 32-34',
    27: 'Zone pushbutton 30-34 / area pushbutton 3 scene 32-34',
    28: 'Zone pushbutton 30-34 / area pushbutton 3 scene 32-34',
    38: 'Zone pushbutton scene 40-44',
    39: 'Zone pushbutton scene 40-44',
    29: 'Zone pushbutton scene 40-44 / area pushbutton 4 scene 42-44',
    30: 'Zone pushbutton scene 40-44 / area pushbutton 4 scene 42-44',
    31: 'Zone pushbutton scene 40-44 / area pushbutton 4 scene 42-44'
};

dSS.config.sceneTipsShadowOverrides = {
    0: 'Down = scene 0 (pushbutton scene 0-4)',
    1: 'Area 1 down',
    2: 'Area 2 down',
    3: 'Area 3 down',
    4: 'Area 4 down',
    5: 'Up = scene 1 (pushbutton scene 0-4)'
};

dSS.config.sceneTipsVentilationOverrides = {
    6: 'Boost with automatic reset'
};

dSS.config.sceneSortOrder = {
    0: 0,
    5: 1,
    17: 2,
    18: 3,
    19: 4,
    1: 5,
    6: 6,
    2: 7,
    7: 8,
    3: 9,
    8: 10,
    4: 11,
    9: 12,
    32: 13,
    33: 14,
    20: 15,
    21: 16,
    22: 17,
    34: 18,
    35: 19,
    23: 20,
    24: 21,
    25: 22,
    36: 23,
    37: 24,
    26: 25,
    27: 26,
    28: 27,
    38: 28,
    39: 29,
    29: 30,
    30: 31,
    31: 32
};

dSS.config.deviceTypes = {
    0: 'KM',
    1: 'TKM',
    2: 'SDM',
    3: 'KL',
    4: 'TUP',
    5: 'ZWS',
    6: 'SDS',
    7: 'SK',
    8: 'AKM',
    10: 'TNY',
    11: 'UMV',
    12: 'UMR',
    14: 'SKM',
    15: 'ZWD'
};

dSS.config.deviceTypeNum = {
    KM: 0,
    TKM: 1,
    SDM: 2,
    KL: 3,
    TUP: 4,
    ZWS: 5,
    SDS: 6,
    SK: 7,
    AKM: 8,
    TNY: 10,
    UMV: 11,
    UMR: 12,
    SKM: 14,
    ZWD: 15
};

dSS.config.deviceColors = {
    1: 'GE',
    2: 'GR',
    3: 'BL',
    4: 'TK',
    5: 'MG',
    6: 'RT',
    7: 'GN',
    8: 'SW',
    9: 'WE'
};

dSS.config.deviceClass = {
    GE: 1,
    GR: 2,
    BL: 3,
    TK: 4,
    MG: 5,
    RT: 6,
    GN: 7,
    SW: 8,
    WE: 9
};


dSS.config.deviceParams = {
    OUTPUT_MODE_DISABLED: 0,
    OUTPUT_MODE_SWITCH: 16,
    OUTPUT_MODE_RMS_DIMMER: 17,
    OUTPUT_MODE_LEADING_EDGE_DIMMER: 20,
    OUTPUT_MODE_DIMMER: 22,
    OUTPUT_MODE_PULSE_PACKAGE_CTL_DIMMER: 28,

    OUTPUT_MODE_PWM: 30,
    OUTPUT_MODE_AUTO: 31,
    OUTPUT_MODE_TWO_STAGE_SWITCH: 34,
    OUTPUT_MODE_BIPOLAR_SWITCH: 43,
    OUTPUT_MODE_THREE_STAGE_SWITCH: 38,
    OUTPUT_MODE_SWITCH_2_POL: 35,
    OUTPUT_MODE_ALTSWITCH: 39,
    OUTPUT_MODE_WIPE: 40,
    OUTPUT_MODE_POWERSAVE: 41,
    OUTPUT_MODE_DIMMER_0_10V: 49,
    OUTPUT_MODE_DIMMER_1_10V: 51,
    OUTPUT_MODE_CONTROL_PWM: 64,
    OUTPUT_MODE_CONTROL_SWITCHED: 65,
    OUTPUT_MODE_SWITCH_2_OUT_IN_2_STEPS: 67,
    OUTPUT_MODE_SWITCH_2_OUT_IN_3_STEPS: 68,
    OUTPUT_MODE_SWITCH_2_OUT_PARALLEL: 69,

    OUTPUT_MODE_POSITION_CONTROL: 33,
    OUTPUT_MODE_POSITION_CONTROL_UNCALIBRATED: 42,

    OUTPUT_MODE_DEFAULT_ACTIVE: 254,
    OUTPUT_MODE_ERROR: 255,

    INPUT_MODE_STANDARD_BUTTON: 0,
    INPUT_MODE_TURBO_BUTTON: 1,
    INPUT_MODE_SWITCH: 3,

    LED_CONFIG_ON: 0,
    LED_CONFIG_AUTO_DARK: 1,
    LED_CONFIG_OFF: 2,

    DIM_CONFIG_SELECT_STANDARD: 1,
    DIM_CONFIG_SELECT_SLOW: 2,
    DIM_CONFIG_SELECT_CUSTOM: 3,

    PUSHBUTTON_DEVICE: 0,
    PUSHBUTTON_ALARM: 1,
    PUSHBUTTON_PANIC: 2,
    PUSHBUTTON_LEAVING_HOME: 3,
    PUSHBUTTON_DOOR_BELL: 5,
    PUSHBUTTON_GROUP1: 1,
    PUSHBUTTON_GROUP2: 2,
    PUSHBUTTON_GROUP3: 3,
    PUSHBUTTON_GROUP4: 4,
    PUSHBUTTON_ZONE0: 5,
    PUSHBUTTON_ZONE1: 6,
    PUSHBUTTON_ZONE2: 7,
    PUSHBUTTON_ZONE3: 8,
    PUSHBUTTON_ZONE4: 9,
    PUSHBUTTON_GROUP5: 10,
    PUSHBUTTON_GROUP6: 11,
    PUSHBUTTON_GROUP7: 12,
    PUSHBUTTON_GROUP8: 13,
    PUSHBUTTON_APARTMENT: 14,
    PUSHBUTTON_HIGHLEVEL: 15,

    BUTTONINPUT_STANDARD: 0,
    BUTTONINPUT_TURBO: 1,
    BUTTONINPUT_SWITCH_STANDARD: 2,
    BUTTONINPUT_SWITCH_EDGE: 3,
    BUTTONINPUT_SWITCH_TIP: 4,
    BUTTONINPUT_2WAY_DW_WITH_INPUT1: 5,
    BUTTONINPUT_2WAY_DW_WITH_INPUT2: 6,
    BUTTONINPUT_2WAY_DW_WITH_INPUT3: 7,
    BUTTONINPUT_2WAY_DW_WITH_INPUT4: 8,
    BUTTONINPUT_2WAY_UP_WITH_INPUT1: 9,
    BUTTONINPUT_2WAY_UP_WITH_INPUT2: 10,
    BUTTONINPUT_2WAY_UP_WITH_INPUT3: 11,
    BUTTONINPUT_2WAY_UP_WITH_INPUT4: 12,
    BUTTONINPUT_2WAY: 13,
    BUTTONINPUT_1WAY: 14,
    BUTTONINPUT_AKM_STANDARD: 16,
    BUTTONINPUT_AKM_INVERTED: 17,
    BUTTONINPUT_AKM_ON_RISINGEDGE: 18,
    BUTTONINPUT_AKM_ON_FALLINGEDGE: 19,
    BUTTONINPUT_AKM_OFF_RISINGEDGE: 20,
    BUTTONINPUT_AKM_OFF_FALLINGEDGE: 21,
    BUTTONINPUT_AKM_RISINGEDGE: 22,
    BUTTONINPUT_AKM_FALLINGEDGE: 23,
    BUTTONINPUT_DISABLED: 255
};

dSS.config.ButtonInputMode = {
    BUTTONINPUT_1WAY: '1way',
    BUTTONINPUT_2WAY_DOWN: '2way_down',
    BUTTONINPUT_2WAY_UP: '2way_up',
    BUTTONINPUT_2WAY: '2way',
    BUTTONINPUT_1WAY_COMBINED: '1way_combined',

    BUTTONINPUT_AKM_STANDARD: 'standard',
    BUTTONINPUT_AKM_INVERTED: 'inverted',
    BUTTONINPUT_AKM_ON_RISINGEDGE: 'on_rising_edge',
    BUTTONINPUT_AKM_ON_FALLINGEDGE: 'on_falling_edge',
    BUTTONINPUT_AKM_OFF_RISINGEDGE: 'off_rising_edge',
    BUTTONINPUT_AKM_OFF_FALLINGEDGE: 'off_falling_edge',
    BUTTONINPUT_AKM_RISINGEDGE: 'rising_edge',
    BUTTONINPUT_AKM_FALLINGEDGE: 'falling_edge'
};

dSS.config.ButtonInputModeLookup = {
    standard: dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD,
    inverted: dSS.config.deviceParams.BUTTONINPUT_AKM_INVERTED,
    on_rising_edge: dSS.config.deviceParams.BUTTONINPUT_AKM_ON_RISINGEDGE,
    on_falling_edge: dSS.config.deviceParams.BUTTONINPUT_AKM_ON_FALLINGEDGE,
    off_rising_edge: dSS.config.deviceParams.BUTTONINPUT_AKM_OFF_RISINGEDGE,
    off_falling_edge: dSS.config.deviceParams.BUTTONINPUT_AKM_OFF_FALLINGEDGE,
    rising_edge: dSS.config.deviceParams.BUTTONINPUT_AKM_RISINGEDGE,
    falling_edge: dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE
};

dSS.config.AttachedShadeTerminalDevice = {
    UNDEFINED: 'none',  // 0
    EXTERNAL_BLINDS: 'windowBlind', // 1
    ROLLER_SHUTTER: 'rollerShutter',  // 2
    AWNING: 'awningTerrace',  // 4
    FACADE_AWNING_VERTICAL: 'awningFacade', // 3
    FACADE_AWNING_HORIZONTAL: 'awningFacadeHorizontal', // 5
    // ELECTRIC_WINDOW: 'window',  // 6
    ELECTRIC_CURTAIN: 'curtain',   // 7
    INTERNAL_BLINDS: 'internalBlind'    // 8
};
dSS.config.AttachedShadeTerminalDeviceName = {
    [dSS.config.AttachedShadeTerminalDevice.UNDEFINED]: 'none',
    [dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS]: 'Blind',
    [dSS.config.AttachedShadeTerminalDevice.ROLLER_SHUTTER]: 'Roller shutter',
    [dSS.config.AttachedShadeTerminalDevice.AWNING]: 'Awning',
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_VERTICAL]: 'Façade awning (vertical)',
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_HORIZONTAL]: 'Façade awning (horizontal)',
    // [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW]: 'Electric window',
    [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_CURTAIN]: 'Electric curtain',
    [dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS]: 'Internal blinds'
};
dSS.config.defaultCalibrationOptionLabel = {
    UP_DOWN: { down: 'It is down or running down', up: 'It is up or running up', '${0%}': 'DOWN', '${100%}': 'UP' },
    IN_OUT: { down: 'It is out or running towards out', up: 'It is in or running towards in', '${0%}': 'OUT', '${100%}': 'IN' },
    OPEN_CLOSED: { down: 'It is closed or closing', up: 'It is open or opening', '${0%}': 'CLOSED', '${100%}': 'OPEN' },
    CLOSED_OPEN: { down: 'It is open or opening', up: 'It is closed or closing', '${0%}': 'OPEN', '${100%}': 'CLOSED' }
};
dSS.config.AttachedShadeTerminalDeviceCalibrationStep1Label = {
    [dSS.config.AttachedShadeTerminalDevice.UNDEFINED]: 'Please indicate where the shade currently is',
    [dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS]: 'Please indicate where the blind currently is',
    [dSS.config.AttachedShadeTerminalDevice.ROLLER_SHUTTER]: 'Please indicate where the roller shutter currently is',
    [dSS.config.AttachedShadeTerminalDevice.AWNING]: 'Please indicate where the awning currently is',
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_VERTICAL]: 'Please indicate where the vertical façade awning currently is',
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_HORIZONTAL]: 'Please indicate current horizontal façade awning position',
    // [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW]: 'Please indicate current electric window position',
    [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_CURTAIN]: 'Please indicate current electric curtain position',
    [dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS]: 'Please indicate current internal blinds position'
};
dSS.config.AttachedShadeTerminalDeviceCalibrationOptionLabels = {
    [dSS.config.AttachedShadeTerminalDevice.UNDEFINED]: dSS.config.defaultCalibrationOptionLabel.UP_DOWN,
    [dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS]: dSS.config.defaultCalibrationOptionLabel.UP_DOWN,
    [dSS.config.AttachedShadeTerminalDevice.ROLLER_SHUTTER]: dSS.config.defaultCalibrationOptionLabel.UP_DOWN,
    [dSS.config.AttachedShadeTerminalDevice.AWNING]: dSS.config.defaultCalibrationOptionLabel.IN_OUT,
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_VERTICAL]: dSS.config.defaultCalibrationOptionLabel.UP_DOWN,
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_HORIZONTAL]: dSS.config.defaultCalibrationOptionLabel.IN_OUT,
    // [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW]: dSS.config.defaultCalibrationOptionLabel.CLOSED_OPEN,
    [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_CURTAIN]: dSS.config.defaultCalibrationOptionLabel.OPEN_CLOSED,
    [dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS]: dSS.config.defaultCalibrationOptionLabel.OPEN_CLOSED
};
dSS.config.AttachedShadeTerminalDeviceCalibrationDirections = {
    [dSS.config.AttachedShadeTerminalDevice.UNDEFINED]: 'UP_DOWN',
    [dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS]: 'UP_DOWN',
    [dSS.config.AttachedShadeTerminalDevice.ROLLER_SHUTTER]: 'UP_DOWN',
    [dSS.config.AttachedShadeTerminalDevice.AWNING]: 'IN_OUT',
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_VERTICAL]: 'UP_DOWN',
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_HORIZONTAL]: 'IN_OUT',
    // [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW]: 'CLOSED_OPEN',
    [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_CURTAIN]: 'OPEN_CLOSED',
    [dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS]: 'OPEN_CLOSED'
};
dSS.config.AttachedShadeTerminalDeviceImages = {
    // kinda default image; no middle so it doesn't show up aside attached terminal device input:
    [dSS.config.AttachedShadeTerminalDevice.UNDEFINED]: {
        down: 'roller-shutter-max.svg',
        up: 'roller-shutter-min.svg'
    },
    [dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS]: {
        down: 'external-blind-max.svg',
        middle: 'external-blind-middle.svg',
        up: 'external-blind-min.svg'
    },
    [dSS.config.AttachedShadeTerminalDevice.ROLLER_SHUTTER]: {
        down: 'roller-shutter-max.svg',
        middle: 'roller-shutter-middle.svg',
        up: 'roller-shutter-min.svg'
    },
    [dSS.config.AttachedShadeTerminalDevice.AWNING]: {
        down: 'awning-max.svg',
        middle: 'awning-middle.svg',
        up: 'awning-min.svg'
    },
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_VERTICAL]: {
        down: 'facade-awning-vertical-max.svg',
        middle: 'facade-awning-vertical-middle.svg',
        up: 'facade-awning-vertical-min.svg'
    },
    [dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_HORIZONTAL]: {
        down: 'facade-awning-horizontal-max.svg',
        middle: 'facade-awning-horizontal-middle.svg',
        up: 'facade-awning-horizontal-min.svg'
    },
    // [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW]: {
    //     down: 'electric-window-max.svg',
    //     middle: 'electric-window-middle.svg',
    //     up: 'electric-window-min.svg'
    // },
    [dSS.config.AttachedShadeTerminalDevice.ELECTRIC_CURTAIN]: {
        down: 'electric-curtain-max.svg',
        middle: 'electric-curtain-middle.svg',
        up: 'electric-curtain-min.svg'
    },
    [dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS]: {
        down: 'internal-blinds-max.svg',
        middle: 'internal-blinds-middle.svg',
        up: 'internal-blinds-min.svg'
    }
};

dSS.config.OutModeStrings = {
    0: 'disabled',
    16: 'switched',
    22: 'dimmed',
    35: 'switched'
};

dSS.config.TitleIconCSSExt = 'Title';

dSS.config.UserGroupStartIndex = 16;
dSS.config.UserGroupMaxCount = 24;
dSS.config.UserGroupEndIndex = dSS.config.UserGroupStartIndex + dSS.config.UserGroupMaxCount - 1;
dSS.config.ControlGroupStartIndex = 48;
dSS.config.ControlGroupMaxCount = 8;
dSS.config.ControlGroupEndIndex = dSS.config.ControlGroupStartIndex + dSS.config.ControlGroupMaxCount - 1;
dSS.config.ResetActiveButtonGroup = 0xff;

dSS.config.GroupColorIconCls = {
    1: 'colorYellow',
    2: 'colorGrey',
    3: 'colorBlue',
    4: 'colorCyan',
    5: 'colorMagenta',
    6: 'colorRed',
    7: 'colorGreen',
    8: 'colorBlack',
    9: 'colorBlue',
    10: 'colorRoomVentilation',
    11: 'colorRoomVentilation',
    12: 'colorRoomVentilation',
    48: 'colorTemperatureControl',
    64: 'colorAptVentilation'
};

dSS.config.GroupColorLockCls = {
    1: 'lockYellow',
    2: 'lockGrey',
    3: 'lockBlue',
    4: 'lockCyan',
    5: 'lockMagenta',
    6: 'lockRed',
    7: 'lockGreen',
    8: 'lockBlack',
    9: 'lockBlue',
    10: 'lockBlue',
    11: 'lockBlue',
    12: 'lockBlue',
    48: 'lockBlue'
};


dSS.config.dSM_LED_CSS = {
    off: 'controllerStatusOff',
    ready: 'controllerStatusReady',
    error: 'controllerStatusError'
};

dSS.config.TransmissionQuality = {
    UPSTREAM_MAX: 62,
    DOWNSTREAM_MAX: 6
};

dSS.config.FeatureConfig = {
    SCENE: 0,
    ZONE: 1,
    CIRCUIT: 2,
    GROUP: 3,
    DEVICE: 4,
    SENSOR: 5
};

dSS.config.PushbuttonMinFW = 0x0314;

dSS.config.AKMInputTypeStrings = {
    0: 'Generic (App)',
    1: 'Presence detector',
    2: 'Brightness (Room)',
    3: 'Presence in darkness',
    4: 'Twilight',
    5: 'Movement',
    6: 'Movement in darkness',
    7: 'Smoke detector',
    8: 'Wind monitor',
    9: 'Rain monitor',
    10: 'Sunlight',
    11: 'Thermostat',
    12: 'Battery low',
    13: 'Window contact',
    14: 'Door contact',
    15: 'Window tilted',
    16: 'Garage door contact',
    17: 'Sun protection',
    18: 'Frost monitor',
    19: 'Heating operation',
    20: 'Change-over',
    21: 'Initialization',
    22: 'Malfunction',
    23: 'Service'
};

dSS.config.AKMInputTypes = {
    GENERIC: 0,
    PRESENCE: 1,
    BRIGHTNESS: 2,
    PRESENCE_DARK: 3,
    TWILIGHT: 4,
    MOVEMENT: 5,
    MOVEMENT_DARK: 6,
    SMOKE: 7,
    WIND: 8,
    RAIN: 9,
    SUNLIGHT: 10,
    THERMOSTAT: 11,
    BATTERY_LOW: 12,
    WINDOW_CONTACT: 13,
    DOOR_CONTACT: 14,
    WINDOW_TILTED: 15,
    GARAGE_DOOR: 16,
    SUNPROTECTION: 17,
    FROST: 18,
    HEATING: 19,
    HEATING_MODE: 20
};

dSS.config.AKMInputTypesValues = {
    0: { 1: 'contact closed', 2: 'contact open' },
    1: { 1: 'Presence', 2: 'No Presence' },
    2: { 1: 'Bright', 2: 'Dark' },
    3: { 1: 'Presence in darkness', 2: 'No Presence in darkness' },
    4: { 1: 'Twilight', 2: 'No Twilight' },
    5: { 1: 'Movement detected', 2: 'No Movement' },
    6: { 1: 'Movement in darkness', 2: 'No Movement in darkness' },
    7: { 1: 'Smoke', 2: 'No Smoke' },
    8: { 1: 'Wind', 2: 'No Wind' },
    9: { 1: 'Rain', 2: 'No Rain' },
    10: { 1: 'Sunlight', 2: 'No Sunlight' },
    11: { 1: 'Warm', 2: 'Cold' },
    12: { 1: 'Battery low', 2: 'Battery ok' },
    13: { 1: 'Window closed', 2: 'Window open' },
    14: { 1: 'Door closed', 2: 'Door open' },
    15: { 1: 'Window closed', 2: 'Window open', 3: 'Window tilted' },
    16: { 1: 'Garage door closed', 2: 'Garage door open' },
    17: { 1: 'Too much sunlight', 2: 'Not enough sunlight' },
    18: { 1: 'Frost', 2: 'No Frost' },
    19: { 1: 'Heating on', 2: 'Heating off' },
    20: { 1: 'Warm water', 2: 'Cold water' },
    21: { 1: 'Initialization active', 2: 'Initialization finished' },
    22: { 1: 'Malfunction', 2: 'No Malfunction' },
    23: { 1: 'Service required', 2: 'No Service required' }
};

dSS.config.AKMButtonInputStrings = {
    standard: 'Standard',
    inverted: 'Inverted',
    on_rising_edge: 'Rising edge on',
    on_falling_edge: 'Falling edge on',
    off_rising_edge: 'Rising edge off',
    off_falling_edge: 'Falling edge off',
    rising_edge: 'Rising edge',
    falling_edge: 'Falling edge'
};

dSS.config.dialogTypes = {
    DEVICE_PROPERTIES: 0,
    SHADE_PROPERTIES: 1,
    CONSUMPTION_SETTINGS: 2,
    RENAME_DIALOG: 3,
    EDIT_GROUP_DIALOG: 4,
    CREATE_GROUP_DIALOG: 5,
    HEATING_PROPERTIES: 6,
    SENSOR_VALUES: 7
};

dSS.config.SensorTypeStrings = {
    0: 'Reserved',
    4: 'Active power [W]',
    5: 'Output current [mA]',
    6: 'Electric meter [kWh]',
    9: 'Temperature indoors [°C]',
    10: 'Temperature outdoors [°C]',
    11: 'Brightness indoors [lux]',
    12: 'Brightness outdoors [lux]',
    13: 'Relative humidity indoors [%]',
    14: 'Relative humidity outdoors [%]',
    15: 'Air pressure [hPa]',
    16: 'Wind gust speed [m/s]',
    17: 'Wind gust direction [° degrees]',
    18: 'Wind speed average in 10 min [m/s]',
    19: 'Wind direction [° degrees]',
    20: 'Precipitation in last hour [mm/m2]',
    21: 'Carbon dioxide concentration [ppm]',
    22: 'Carbon monoxide concentration [ppm]',
    25: 'Sound pressure level [dB]',
    50: 'Room temperature setpoint [°C]',
    51: 'Room temperature control [%]',
    61: 'Reserved',
    62: 'Reserved',
    63: 'Reserved',
    64: 'Output current (H) [mA]',
    65: 'Power consumption [VA]',
    76: 'Sun azimuth [° degrees]',
    77: 'Sun elevation [° degrees]',
    253: 'Reserved',
    255: 'Unknown'
};

dSS.config.SensorTypeUnits = {
    0: '',
    4: 'W',
    5: 'mA',
    6: 'kWh',
    9: '°C',
    10: '°C',
    11: 'lux',
    12: 'lux',
    13: '%rH',
    14: '%rH',
    15: 'hPa',
    16: 'm/s',
    17: '° degrees',
    18: 'm/s',
    19: '° degrees',
    20: 'mm/m2',
    21: 'ppm',
    22: 'ppm',
    25: 'dB',
    50: '°C',
    51: '%',
    61: '',
    62: '',
    63: '',
    64: 'mA',
    65: 'VA',
    66: 'K',
    67: 'Lx',
    68: '%rH',
    69: 'W',
    70: 'kWh',
    71: 'l',
    72: 'l/s',
    73: 'm',
    74: 'g',
    75: 's',
    76: '° degrees',
    77: '° degrees',
    253: '',
    255: ''
};


dSS.config.SensorTypes = {
    RESERVED1: 0,
    TEMPERATURE_INDOORS: 9,
    TEMPERATURE_OUTDOORS: 10,
    BRIGHTNESS_INDOORS: 11,
    BRIGHTNESS_OUTDOORS: 12,
    HUMIDITY_INDOORS: 13,
    HUMIDITY_OUTDOORS: 14,
    AIR_PRESSURE: 15,
    GUST_SPEED: 16,
    GUST_DIRECTION: 17,
    WIND_SPEED: 18,
    WIND_DIRECTION: 19,
    PRECIPITATION: 20,
    CO2_CONCENTRATION: 21,
    CO_CONCENTRATION: 22,
    SOUND_PRESSURE: 25,
    ROOM_TEMP_SETPOINT: 50,
    ROOM_TEMP_CONTROL_VAR: 51,
    RESERVED2: 61,
    RESERVED3: 62,
    SUN_AZIMUTH: 76,
    SUN_ELEVATION: 77,
    RESERVED4: 253
};

dSS.config.StateValueStrings = {
    0: 'Invalid',
    1: 'Active',
    2: 'Inactive',
    3: 'Unknown'
};

dSS.config.HeatingOperationMode = {
    CONTROL: 0,
    EMERGENCY: 1,
    VALVE_PROTECTION: 2,
    OUTPUT_DEACTIVATED: 3,
    FORCED_MODE_ON: 4,
    FORCED_MODE_OFF: 5
};
dSS.config.HeatingOperationModeStrings = {
    [dSS.config.HeatingOperationMode.CONTROL]: 'control',
    [dSS.config.HeatingOperationMode.EMERGENCY]: 'emergency',
    [dSS.config.HeatingOperationMode.VALVE_PROTECTION]: 'valve protection',
    [dSS.config.HeatingOperationMode.OUTPUT_DEACTIVATED]: 'output deactivated',
    [dSS.config.HeatingOperationMode.FORCED_MODE_ON]: 'forced mode on',
    [dSS.config.HeatingOperationMode.FORCED_MODE_OFF]: 'forced mode off'
};

dSS.config.BusMemberTypes = {
    Unknown: 0,
    dSM11: 16,
    dSM12: 17,
    dSM20: 18,
    dSM25: 19,
    VDSM: 32,
    VDC: 33,
    backendVdc: 34
};

dSS.config.ValveTypes = {
    UNKNOWN: 'Unknown',
    FLOOR: 'Floor',
    WALL: 'Wall',
    RADIATOR: 'Radiator',
    CONVECTOR_PASSIVE: 'ConvectorPassive',
    CONVECTOR_ACTIVE: 'ConvectorActive',
    FLOOR_LOW_ENERGY: 'FloorLowEnergy'
};

dSS.config.UMVRelay = {
    FORCE_ON: 0,
    FOLLOW_1: 1,
    FOLLOW_N: 2,
    FOLLOW_L: 3
};

dSS.devprops.PBConfig = {
    PUSHBUTTON_AREA1: -1,
    PUSHBUTTON_AREA2: -2,
    PUSHBUTTON_AREA3: -3,
    PUSHBUTTON_AREA4: -4,
    PUSHBUTTON_DEVICE: -5,
    PUSHBUTTON_ZONE: -6,
    PUSHBUTTON_AREA: -7,
    PUSHBUTTON_HIGHLEVEL: -8,
    PUSHBUTTON_PANIC: -9,
    PUSHBUTTON_LEAVING_HOME: -10,
    PUSHBUTTON_DOOR_BELL: -11,
    PUSHBUTTON_ALARM: -12,
    PUSHBUTTON_SENSOR: -13,
    PUSHBUTTON_DISABLED: -14
};

dSS.config.Location = {
    NONE: 'none',
    NORTH: 'north',
    NORTH_EAST: 'north east',
    EAST: 'east',
    SOUTH_EAST: 'south east',
    SOUTH: 'south',
    SOUTH_WEST: 'south west',
    WEST: 'west',
    NORTH_WEST: 'north west'
};

dSS.config.WindProtectionClass = {
    NONE: 0,
    WPC1_AWNING: 1, // 7.8m/s
    WPC2_AWNING: 2, // 10.6m/s
    WPC3_AWNING: 3, // 13.3m/s
    WPC1_BLIND: 4, // 9.0m/s
    WPC2_BLIND: 5, // 10.7m/s
    WPC3_BLIND: 6, // 12.8m/s
    WPC4_BLIND: 7, // 16.7m/s
    WPC5_BLIND: 8, // 21.0m/s
    WPC6_BLIND: 9  // 25.6m/s
};

dSS.config.WindProtectionClassStrings = {
    0: 'Undefined',
    1: 'Class 1 - 7.8m/s',
    2: 'Class 2 - 10.6m/s',
    3: 'Class 3 - 13.3m/s',
    4: 'Class 1 - 9.0m/s',
    5: 'Class 2 - 10.7m/s',
    6: 'Class 3 - 12.8m/s',
    7: 'Class 4 - 16.7m/s',
    8: 'Class 5 - 21.0m/s',
    9: 'Class 6 - 25.6m/s'
};

dSS.config.ClusterLocationStrings = {
    [dSS.config.Location.NONE]: 'Undefined',
    [dSS.config.Location.NORTH]: 'North',
    [dSS.config.Location.NORTH_EAST]: 'North-East',
    [dSS.config.Location.EAST]: 'East',
    [dSS.config.Location.SOUTH_EAST]: 'South-East',
    [dSS.config.Location.SOUTH]: 'South',
    [dSS.config.Location.SOUTH_WEST]: 'South-West',
    [dSS.config.Location.WEST]: 'West',
    [dSS.config.Location.NORTH_WEST]: 'North-West'
};

dSS.config.TempControlSceneNames = {
    0: 'Off',
    1: 'Comfort',
    2: 'Economy',
    3: 'Not used',
    4: 'Night',
    5: 'Holiday',
    6: 'Cooling',
    7: 'Off'
};

dSS.config.SystemOverviewStateSortOder = {
    PRESENCE: 0,
    PRESENCE_SIM: 1,
    SECURITY: 2,
    SOLAR_ALTITUDE: 3,
    WEATHER: 4,
    HEATING: 5,
    APT_VENTILATION: 6,
    USER_DEFINED_STATES: 7
};

dSS.config.InputTypeHack = {
    SENSOR: 0,
    PUSHBUTTON: 1
};

dSS.config.SubmitDelayMS = 4000; // 4s

dSS.config.ButtonInputModeV2Map = {
    0: 'standard',
    '1way': 'mode1way',
    '2way_down': 'mode2wayDown',
    '2way_up': 'mode2wayUp',

    standard: 'standard',
    inverted: 'inverted',
    on_rising_edge: 'risingEdge1',
    on_falling_edge: 'fallingEdge1',
    off_rising_edge: 'risingEdge0',
    off_falling_edge: 'fallingEdge0',
    rising_edge: 'toggleModeOnRisingEdge',
    falling_edge: 'toggleModeOnFallingEdge'
};

dSS.config.GroupV2Map = {
    0: 'none',
    1: 'lights',
    2: 'blinds',
    3: 'heating',
    4: 'audio',
    5: 'video',
    6: 'security',
    7: 'access',
    8: 'joker',
    9: 'cooling',
    10: 'ventilation',
    11: 'window',
    12: 'recirculation',
    48: 'temperatureControl',
    64: 'apartmentVentilation'
};

dSS.config.AKMInputTypesV2Map =
{
    0: 'generic',
    1: 'presence',
    4: 'twilight',
    5: 'motion',
    7: 'smoke',
    8: 'windStrengthAboveLimit',
    9: 'rain',
    10: 'sunRadiation',
    11: 'temperatureBelowLimit',
    13: 'windowIsOpen',
    14: 'doorIsOpen',
    16: 'garageDoorIsOpen',
    17: 'sunProtection',
    18: 'frost',
    19: 'heatingSystemEnabled',
    20: 'changeOverSignal',
    22: 'malfunction',
    23: 'service'
};

dSS.config.PushButtonV2Map = {
    0: {
        target: 'device', // PUSHBUTTON_DEVICE / 2-4   / 2-4
        preset: 'devicePresets2To4'
    },
    1: {
        target: 'area',  // PUSHBUTTON_GROUP1 / 2-4   / 2-4    / A01
        area: 'area1',
        preset: 'area1Presets1To4'
    },
    10: {
        target: 'area',  // PUSHBUTTON_GROUP5 / 12-14 / 12-14  / A01
        area: 'area1',
        preset: 'area1Presets12To14'
    },
    2: {
        target: 'area',  // PUSHBUTTON_GROUP2 / 2-4   / 2-4    / A02
        area: 'area2',
        preset: 'area2Presets1To4'
    },
    11: {
        target: 'area',  // PUSHBUTTON_GROUP5 / 22-24 / 22-24  / A02
        area: 'area2',
        preset: 'area2Presets22To24'
    },
    3: {
        target: 'area',  // PUSHBUTTON_GROUP3 / 2-4   / 2-4    / A03
        area: 'area3',
        preset: 'area3Presets1To4'
    },
    12: {
        target: 'area',  // PUSHBUTTON_GROUP7 / 32-34 / 32-34  / A03
        area: 'area3',
        preset: 'area3Presets32To34'
    },
    4: {
        target: 'area',  // PUSHBUTTON_GROUP4 / 2-4   / 2-4    / A04
        area: 'area4',
        preset: 'area4Presets1To4'
    },
    13: {
        target: 'area',  // PUSHBUTTON_GROUP8 / 42-44 / 42-44  / A04
        area: 'area4',
        preset: 'area4Presets42To44'
    },
    5: {
        target: 'zone',  // PUSHBUTTON_ZONE0  / 0-4   / 2-4    / S01
        preset: 'zonePresets1To4'
    },
    6: {
        target: 'zone',  // PUSHBUTTON_ZONE1  / 10-14 / 12-14  / S11
        preset: 'zonePresets10To14'
    },
    7: {
        target: 'zone',  // PUSHBUTTON_ZONE2  / 20-24 / 22-24  / S21
        preset: 'zonePresets20To24'
    },
    8: {
        target: 'zone',  // PUSHBUTTON_ZONE3  / 30-34 / 32-34  / S31
        preset: 'zonePresets30To34'
    },
    9: {
        target: 'zone',   // PUSHBUTTON_ZONE4  / 40-44 / 42-44  / S41
        preset: 'zonePresets40To44'
    }
};

dSS.config.PushButtonV2JokerMap = {
    1: {
        target: 'alarm'       // PUSHBUTTON_ALARM
    },
    2: {
        target: 'panic'       // PUSHBUTTON_PANIC
    },
    3: {
        target: 'leavingHome' // PUSHBUTTON_HIGHLEVEL
    },
    5: {
        target: 'doorBell'    // PUSHBUTTON_DOOR_BELL
    },
    15: {
        target: 'app'   // PUSHBUTTON_HIGHLEVEL
    }
};

dSS.config.InputTypeV2Map = {
    0: 'state', // SENSOR
    1: 'pushButton'
};

dSS.config.DimTimeDefaults = {
    FAST: 100,
    SOFT_ON: 400
};

dSS.config.ClusterOperationLockState = {
    RAIN: 'rain',
    FROST: 'frost',
    HAIL: 'hail',
    WIND: 'wind'
};

dSS.config.GRKL387SliderIncrement = 6.6666666666;

dSS.config.ButtonUsage = {
    UNSUPPORTED: 'unsupported',
    USED: 'used',
    AUTO_UNUSED: 'auto_unused',
    MANUAL_UNUSED: 'manual_unused'
};

dSS.config.ButtonUsageStrings = {
    [dSS.config.ButtonUsage.UNSUPPORTED]: '',
    [dSS.config.ButtonUsage.USED]: 'used',
    [dSS.config.ButtonUsage.AUTO_UNUSED]: 'unused',
    [dSS.config.ButtonUsage.MANUAL_UNUSED]: 'unused (manual)'
};
