/* globals Ext, dSS */

/* Sensor function selection */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.AKMSensorConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function (config) {
        this.device = config.device;
        this.group = config.group;
        this.matrix = config.matrix;
        this.isJokerDevice = dSS.util.parseFunctionID(this.device.get('functionID'))[0] == dSS.config.deviceClass.SW;

        var deviceIsOfGrKl300Type = dSS.util.isGrKl300TypeDevice(config.device);

        var showGroup;
        var biArray = this.device.get('binaryInputs');
        if (biArray.length > 0) {
            showGroup = biArray[0].targetGroup;
        }
        switch (showGroup) {
            case dSS.config.groups.APT_VENTILATION:
            case dSS.config.groups.VENTILATION:
            case dSS.config.groups.RECIRCULATION:
                break;
            default:
                showGroup = dSS.config.groups.JOKER;
                break;
        }

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.forceSave = false;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.sensorFunctionInLight = [ // TODO magic numbers
            [
                0, // App-Mode
                dSS._(dSS.config.AKMInputTypeStrings[0])
            ],
            [
                1, // Presence
                dSS._(dSS.config.AKMInputTypeStrings[1])
            ],
            [
                4, // Twilight
                dSS._(dSS.config.AKMInputTypeStrings[4])
            ],
            [
                5, // Movement
                dSS._(dSS.config.AKMInputTypeStrings[5])
            ]
        ];
        this.sensorFunctionsInJoker = [
            [
                7, // Smoke
                dSS._(dSS.config.AKMInputTypeStrings[7])
            ],
            [
                8, // Wind
                dSS._(dSS.config.AKMInputTypeStrings[8])
            ],
            [
                9, // Rain
                dSS._(dSS.config.AKMInputTypeStrings[9])
            ],
            [
                10, // Sunlight
                dSS._(dSS.config.AKMInputTypeStrings[10])
            ],
            [
                17, // Sun Protection
                dSS._(dSS.config.AKMInputTypeStrings[17])
            ],
            [
                18, // Frost
                dSS._(dSS.config.AKMInputTypeStrings[18])
            ],
            [
                11, // Thermostat
                dSS._(dSS.config.AKMInputTypeStrings[11])
            ],
            [
                13, // Window contact
                dSS._(dSS.config.AKMInputTypeStrings[13])
            ],
            [
                14, // Door contact
                dSS._(dSS.config.AKMInputTypeStrings[14])
            ],
            [
                16, // Garage door contact
                dSS._(dSS.config.AKMInputTypeStrings[16])
            ],
            [
                19, // Heating system
                dSS._(dSS.config.AKMInputTypeStrings[19])
            ],
            [
                20, // Heating mode
                dSS._(dSS.config.AKMInputTypeStrings[20])
            ],
            [
                22, // Malfunction
                dSS._(dSS.config.AKMInputTypeStrings[22])
            ],
            [
                23, // Service
                dSS._(dSS.config.AKMInputTypeStrings[23])
            ]
        ];
        this.sensorFunctionsInGrKl300 = [
            [
                0, // App-Mode
                dSS._(dSS.config.AKMInputTypeStrings[0])
            ],
            [
                8, // Wind
                dSS._(dSS.config.AKMInputTypeStrings[8])
            ],
            [
                9, // Rain
                dSS._(dSS.config.AKMInputTypeStrings[9])
            ],
            [
                10, // Sunlight
                dSS._(dSS.config.AKMInputTypeStrings[10])
            ],
            [
                17, // Sun Protection
                dSS._(dSS.config.AKMInputTypeStrings[17])
            ],
            [
                13, // Window contact
                dSS._(dSS.config.AKMInputTypeStrings[13])
            ],
            [
                14, // Door contact
                dSS._(dSS.config.AKMInputTypeStrings[14])
            ]
        ];

        if (deviceIsOfGrKl300Type && deviceIsOfGrKl300Type.sensor) {
            this.sensorFunctions = this.sensorFunctionsInGrKl300;
        } else {
            this.sensorFunctions = this.sensorFunctionInLight;
            if (this.isJokerDevice) {
                this.sensorFunctions = this.sensorFunctionInLight.concat(this.sensorFunctionsInJoker);
            }
        }

        this.sensorSelection = new Ext.form.ComboBox({
            cls: 'dsClsAKMSensorConfigForm_SensorSelectionCombo',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            fieldLabel: dSS._('Sensor function'),
            labelStyle: 'margin-top: 5px',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: this.sensorFunctions
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        this.groupSelection = new Ext.form.ComboBox({
            cls: 'dsClsAKMSensorConfigForm_GroupSelectionCombo',

            x: 288,
            y: 5,
            typeAhead: false,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: true,
            editable: false,
            width: 122,
            mode: 'local',
            value: showGroup,
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: [
                    [
                        dSS.config.groups.JOKER,
                        dSS._('Whole apartment')
                    ],
                    [
                        dSS.config.groups.APT_VENTILATION,
                        dSS._(dSS.config.groupNames[dSS.config.groups.APT_VENTILATION])
                    ],
                    [
                        dSS.config.groups.VENTILATION,
                        dSS._(dSS.config.groupNames[dSS.config.groups.VENTILATION])
                    ],
                    [
                        dSS.config.groups.RECIRCULATION,
                        dSS._(dSS.config.groupNames[dSS.config.groups.RECIRCULATION])
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            hidden: Boolean(deviceIsOfGrKl300Type)
        });

        Ext.apply(this, {
            cls: 'dsClsAKMSensorConfigForm',

            bubbleEvents: [
                'formsubmitted', 'formloaded', 'formchanged', 'updateshadow',
                'formdelayothers', 'formsetdeviceconfig'
            ],
            padding: 5,
            height: 35,
            scope: this,
            layout: 'absolute',
            items: [
                {
                    x: 5,
                    y: 5,
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items: [
                        this.sensorSelection
                    ]
                },
                this.groupSelection
            ]
        });

        dSS.devprops.AKMSensorConfigForm.superclass.constructor.apply(this, arguments);

        this.on('afterrender', function () {
            if (this.deviceType === dSS.config.deviceTypeNum.UMR) {
                if (this.device.get('dSUIDIndex') == 0) {
                    if (this.group !== dSS.config.deviceClass.SW) {
                        this.hide.defer(1, this);
                    }
                }
            }
        }, this);

        this.on('loadcomplete', function () {
            this.sensorSelection.on('select', function (combo, record) {
                this.handleGroupVisibility(record.get('id'));
                this.fireEvent('formchanged');
                if (!this.matrix.setumr200config) {
                    this.fireEvent('formdelayothers', this.getId());
                }
            }, this);

            this.groupSelection.on('select', function () {
                this.fireEvent('formchanged');
                if (!this.matrix.setumr200config) {
                    this.fireEvent('formdelayothers', this.getId());
                }
            }, this);
        }, this);

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);

        this.on('beforedestroy', function () {
            this.closed = true;
            dSS.MessageBus.removeListener('inputtypechanged', this.updateVisibility, this);
        }, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            if (data.loader !== dSS.devprops.Loaders.AKM_BINARY_INPUTS) {
                return;
            }

            if (!data.json.ok) {
                this.sensorSelection.markInvalid(dSS._(data.json.message));
                this.fireEvent('formloaded', true);
                return;
            }

            var inputs = data.json.result.inputs;
            if (inputs.length < 1) {
                this.sensorSelection.markInvalid(dSS._(data.json.message));
                this.fireEvent('formloaded', true);
            }

            if (inputs.length) {
                this.loadedValue = inputs[0].inputType;
            } else {
                this.loadedValue = 0;
            }
            this.sensorSelection.setValue(this.loadedValue);
            this.handleGroupVisibility(this.loadedValue);
            this.fireEvent('formloaded', true);
        }, this);

        this.on('submit', function () {
            if (this.disabled) {
                this.fireEvent('formsubmitted');
                return;
            }

            var value = this.sensorSelection.getValue();
            var loaded = this.loadedValue;
            if (!this.forceSave && value === loaded) {
                this.fireEvent('formsubmitted');
                return;
            }

            var group = this.groupSelection.getValue();

            if (this.matrix.setumr200config) {
                var parameter = {};
                parameter.application = 'joker';
                parameter.stateType = dSS.config.AKMInputTypesV2Map[value];
                if (group == dSS.config.groups.JOKER) {
                    parameter.stateTarget = 'wholeApartment';
                } else if (group == dSS.config.groups.APT_VENTILATION) {
                    parameter.stateTarget = 'apartmentVentilation';
                } else if (group == dSS.config.groups.VENTILATION) {
                    parameter.stateTarget = 'ventilation';
                } else if (group == dSS.config.groups.RECIRCULATION) {
                    parameter.stateTarget = 'recirculation';
                }
                this.fireEvent('formsetdeviceconfig', parameter);
                return;
            }

            var bitcb = this.saveBinaryInputType.createDelegate(this, [ value ]);

            var nextcall = this.saveBinaryInputTarget.createDelegate(this, [ group, bitcb ]);
            if (this.isJokerDevice) {
                Ext.Ajax.request({
                    url: '/json/device/setJokerGroup',
                    disableCaching: true,
                    method: 'GET',
                    params: {
                        dsuid: this.device.get('dsuid'),
                        groupID: dSS.config.groups.JOKER
                    },
                    scope: this,
                    success: function (result) {
                        try {
                            var json = Ext.util.JSON.decode(result.responseText);
                            if (!json.ok) {
                                Ext.Msg.alert(
                                    dSS._('Error'),
                                    dSS._('Could not set joker device group') + ': (' + json.message + ')'
                                );
                                this.fireEvent('formsubmitted');
                                return;
                            }
                        } catch (err) {
                            Ext.Msg.alert(
                                dSS._('Error'),
                                dSS._('Could not set joker device group') + ': (' + err + ')'
                            );
                            this.fireEvent('formsubmitted');
                        }
                        // supress scenesmarkdirty event, will be fired with
                        // the last request
                        dSS.util.processDeviceActionResponse(json.result, true);
                        nextcall();
                    },
                    failure: function () {
                        Ext.Msg.alert(
                            dSS._('Error'),
                            dSS._('Could not set joker device group')
                        );
                        this.fireEvent('formsubmitted');
                    }
                });
            } else {
                nextcall();
            }
        }, this);
    },
    handleGroupVisibility: function (id) {
        // only malfunction and service are allowed to set the AV group
        if (id != 22 && id != 23) {
            this.groupSelection.setValue(dSS.config.groups.JOKER);
            this.groupSelection.disable();
        } else if (this.groupSelection.disabled) {
            this.groupSelection.enable();
        }
    },
    saveBinaryInputType: function (value) {
        var dSUID = this.device.get('dsuid');
        var me = this;

        dSS.util.genericRequest({
            url: '/json/device/setBinaryInputType',
            params:
            {
                dsuid: dSUID,
                index: 0,
                type: value
            },
            failureMsg: dSS._('Could not save sensor function'),
            scope: me
        },
        function (ok) {
            if (!ok) {
                me.fireEvent('formsubmitted');
                dSS.MessageBus.fireEvent('scenesmarkdirty');
                return;
            }
            var device = dSS.data.Loader.getDeviceStore().getById(dSUID);
            if (device == null) {
                Ext.Msg.alert(dSS._('Error'), dSS._('Device not found'));
                dSS.MessageBus.fireEvent('scenesmarkdirty');
                this.fireEvent('formsubmitted');
                return;
            }
            var inputs = device.get('binaryInputs');
            if (inputs.length) {
                inputs[0].inputType = value;
            }
            device.set('binaryInputs', inputs);
            device.commit();
            dSS.MessageBus.fireEvent('scenesmarkdirty');
            me.fireEvent('formsubmitted');
        });
    },
    saveBinaryInputTarget: function (group, nextcall) {
        var device = this.device;

        Ext.Ajax.request({
            url: '/json/device/setBinaryInputTarget',
            disableCaching: true,
            method: 'GET',
            params: {
                dsuid: device.get('dsuid'),
                index: 0,
                groupId: group
            },
            scope: this,
            success: function (result) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(
                            dSS._('Error'),
                            dSS._('Could not set binary input target group') + ': (' + json.message + ')'
                        );
                        dSS.MessageBus.fireEvent('scenesmarkdirty');
                        this.fireEvent('formsubmitted');
                        return;
                    }

                    var inputs = device.get('binaryInputs');
                    if (inputs.length) {
                        inputs[0].targetGroupId = group;
                    }
                    device.set('binaryInputs', inputs);
                    device.commit();
                    dSS.util.processDeviceActionResponse(json.result, true);
                } catch (err) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not set binary input target group') + ': (' + err + ')'
                    );
                    this.fireEvent('formsubmitted');
                }
                nextcall();
            },
            failure: function () {
                Ext.Msg.alert(
                    dSS._('Error'),
                    dSS._('Could not set binary input target group')
                );
                dSS.MessageBus.fireEvent('scenesmarkdirty');
                this.fireEvent('formsubmitted');
            }
        });
    },
    updateVisibility: function (input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.show();
            this.forceSave = true;
            if (!this.matrix.setumr200config) {
                this.fireEvent('formdelayothers', this.getId());
            }
            this.enable();
            // enabling the top panel does enable our disabled group
            // selection too
            this.handleGroupVisibility(this.sensorSelection.getValue());
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.hide();
            this.disable();
        }
        this.fireEvent('updateshadow');
    }
});

Ext.reg('dssakmsensorconfigform', dSS.devprops.AKMSensorConfigForm);
