Ext.namespace('dSS', 'dSS.config', 'dSS.devprops');

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured joker group)
dSS.devprops.AttachedShadeTerminalDeviceConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.device == null) {
            Ext.MessageBox.alert(dSS._('Error'), ('Missing dSUID'));
            return;
        }

        var me = this;
        this.dSUID = config.device.get('dsuid');
        this.matrix = config.matrix;

        this.forceSave = false;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var selectionItems = this.getSelectionItems();

        this.attachedTerminalDeviceSelection = new Ext.form.ComboBox({
            cls: 'dsClsAttachedShadeTerminalDeviceConfigForm_DeviceSelectionCombo',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender: true,
            forceSelection: true,
            editable: false,
            width: 185,
            mode: 'local',
            fieldLabel: dSS._('Attached native device'),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: selectionItems
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        this.changeWarningNotice = new Ext.BoxComponent({
            cls: 'dsClsAttachedShadeTerminalDeviceConfigForm_ChangeWarningBox',

            hidden: true,
            html: '<b><img src="/framework/images/warning.png" /> ' + dSS._('Note:') + '</b><br/>' +
                dSS._('Changing the attached terminal device resets the standard behavior of the device. This may take several minutes to complete.<br/>' +
                ' Moreover, the device has to be calibrated again.')
        });

        this.deviceImage = new (Ext.extend(Ext.BoxComponent, {
            xtype: 'box',
            width: 70,
            height: 55,
            style: {
                marginLeft: 20,
                marginBottom: 10
            },
            setIcon (deviceType) {
                const deviceImages = dSS.config.AttachedShadeTerminalDeviceImages[deviceType];
                if (!deviceImages.middle) {
                    this.update('');
                    this.hide();
                    return;
                }
                this.show();
                this.update('<img src="/images/shades/' + deviceImages.middle + '" width="50" alt="' + dSS._(dSS.config.AttachedShadeTerminalDeviceName[deviceType]) + '" />');
            }
        }));

        Ext.apply(this, {
            cls: 'dsClsAttachedShadeTerminalDeviceConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged', 'formsetdeviceconfig' ],
            padding: 5,
            scope: this,
            layout: 'hbox',
            items: [
                {
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    width: 290,
                    height: 110,
                    items: [
                        this.attachedTerminalDeviceSelection,
                        this.changeWarningNotice
                    ]
                },
                {
                    xtype: 'panel',
                    border: false,
                    style: {
                        marginLeft: 10
                    },
                    items: [
                        this.deviceImage
                    ]
                }
            ]
        });

        dSS.devprops.AttachedShadeTerminalDeviceConfigForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function (data) {
            if (data && data.loader === dSS.devprops.Loaders.ATTACHED_TERMINAL_DEVICE) {
                if (data.json && data.json.data) {
                    me.initialValue = data.json.data.attributes.shadeAttributes && data.json.data.attributes.shadeAttributes.shadeType ||
                        dSS.config.AttachedShadeTerminalDevice.UNDEFINED;
                    if (me.initialValue !== dSS.config.AttachedShadeTerminalDevice.UNDEFINED) {
                        this.attachedTerminalDeviceSelection.getStore().loadData(
                            this.getSelectionItems(dSS.config.AttachedShadeTerminalDevice.UNDEFINED)
                        );
                    }
                    this.attachedTerminalDeviceSelection.setValue(me.initialValue);
                    me.deviceImage.setIcon(me.initialValue);
                } else {
                    this.attachedTerminalDeviceSelection.markInvalid();
                }
            }
        });

        this.on('loadcomplete', function () {
            this.attachedTerminalDeviceSelection.on('select', function() {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        this.on('afterrender', function () {
            this.fireEvent('formloaded', true);
        }, this);

        this.on('formchanged', function () {
            var currentValue = this.attachedTerminalDeviceSelection.getValue();
            if (currentValue !== me.initialValue) {
                this.changeWarningNotice.show();
            } else {
                this.changeWarningNotice.hide();
            }
            this.deviceImage.setIcon(currentValue);
        }, this);

        this.on('submit', function () {
            var value = this.attachedTerminalDeviceSelection.getValue();
            var dSUID = this.dSUID;
            if (this.forceSave !== true && (value === '' || value === me.initialValue)) {
                this.fireEvent('formsubmitted');
                return;
            }

            Ext.Ajax.request({
                url: '/api/v1/apartment/dsDevices/' + dSUID,
                method: 'PATCH',
                disableCaching: true,
                scope: this,
                jsonData: Ext.util.JSON.encode([
                    {
                        op: 'replace',
                        path: '/shadeAttributes',
                        value: { shadeType: value }
                    }
                ]),
                success: function (result, request) {
                    try {
                        if (!result || result.status !== 204) {
                            throw 'error';
                        }

                        var requestsToDoCounter = 2;
                        function callback () {
                            requestsToDoCounter--;
                            if (!requestsToDoCounter) {
                                this.fireEvent('formsubmitted');
                            }
                        }
                        Ext.Ajax.request({          // reset shade calibration statuses
                            url: '/json/device/setConfig',
                            method: 'GET',
                            disableCaching: true,
                            scope: this,
                            params: {
                                dsuid: dSUID,
                                class: 1,
                                index: 92,
                                value: 0
                            },
                            success: callback,
                            failure: callback
                        });
                        Ext.Ajax.request({          // reset the mode to uncalibrated blades
                            url: '/json/device/setOutputMode',
                            method: 'GET',
                            disableCaching: true,
                            scope: this,
                            params: {
                                dsuid: dSUID,
                                modeID: 42
                            },
                            success: callback,
                            failure: callback
                        });

                        // when attached terminal device changes, there may be a need to update scene settings panel,
                        // e.g. disable blades angle slider, if the new device type has no fins:
                        var components = dSS.util.getComponentsByQuery('.dsClsDeviceSettingsDefaultView');
                        if (components) {
                            components.forEach(function (component) {
                                component.updateGrVisualConfig(value);
                            });
                        }
                    } catch (err) {
                        Ext.Msg.alert(
                            dSS._('Error'),
                            dSS._('Could not set attached terminal device type')
                        );
                        this.fireEvent('formsubmitted');
                    }
                },
                failure: function(result, request) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not set attached terminal device type')
                    );
                    this.fireEvent('formsubmitted');
                }
            });
        }, this);
    },

    getSelectionItems: function (filterOut) {
        return Object.values(dSS.config.AttachedShadeTerminalDevice)
            .filter(function (deviceType) {
                if (filterOut != null && deviceType === filterOut) {
                    return false;
                }
                return true;
            })
            .map(function (deviceType) {
                return [
                    deviceType,
                    dSS._(dSS.config.AttachedShadeTerminalDeviceName[deviceType])
                ];
            });
    }
});

Ext.reg('dssdevattachedshadeterminaldeviceconfigform', dSS.devprops.AttachedShadeTerminalDeviceConfigForm);
