Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.InputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.dSUID = config.dSUID;
        this.dialogType = config.type;
        this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.radio = undefined;

        var items = [];

        items =
        [
            {
                cls: 'dsClsInputModeForm_InputModeRadioGroup_ButtonRadio',

                boxLabel: dSS._("button"),
                name: 'rb_inmode_' + config.dSUID,
                id: dSS.config.deviceParams.INPUT_MODE_STANDARD_BUTTON.toString(),
                inputValue: dSS.config.deviceParams.INPUT_MODE_STANDARD_BUTTON
            },
            {
                cls: 'dsClsInputModeForm_InputModeRadioGroup_TurboButtonRadio',

                boxLabel: dSS._("turbo button"),
                name: 'rb_inmode_' + config.dSUID,
                id: dSS.config.deviceParams.INPUT_MODE_TURBO_BUTTON.toString(),
                inputValue: dSS.config.deviceParams.INPUT_MODE_TURBO_BUTTON
            },
            {
                cls: 'dsClsInputModeForm_InputModeRadioGroup_SwitchRadio',

                boxLabel: dSS._("switch"),
                name: 'rb_inmode_' + config.dSUID,
                id: dSS.config.deviceParams.INPUT_MODE_SWITCH.toString(),
                inputValue: dSS.config.deviceParams.INTPUT_MODE_SWITCH
            }
        ];

        this.radio = new Ext.form.RadioGroup(
        {
            cls: 'dsClsInputModeForm_InputModeRadioGroup',

            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._("Input mode"),
            columns: 1,
            items: items
        });


        Ext.apply(this, {
            cls: 'dsClsInputModeForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items: this.radio
        });

        dSS.devprops.InputModeForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.radio.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var val = data.json.result[this.dSUID][0].outputMode;
                this.setRadioGroup(val);

                this.el.unmask();
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            var value;
            // this happens if the value was not loaded correctly
            if (this.radio.getValue() != null) {
                value = this.radio.getValue().getRawValue();
            }

            // do not submit anything if value was not loaded and also has
            // not been changed by the user
            if ((value === undefined) || (value == this.loadedValue)) {
                this.fireEvent('formsubmitted');
                return;
            }

            var dSUID = this.dSUID;

            value = parseInt(value);
            if (value === NaN) {
                value = this.loadedValue;
            }

            // this should never happen, but lets double check so we
            // do not attempt to write some random crap into the klemme
            var validValue = true;
            if ((value !== dSS.config.deviceParams.INPUT_MODE_SWITCH) &&
                (value !== dSS.config.deviceParams.INPUT_MODE_TURBO_BUTTON) &&
                (value !== dSS.config.deviceParams.INPUT_MODE_STANDARD_BUTTON)) {
                validValue = false;
            }

            if (!validValue) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Attempted to set invalid value") +
                              ": " + value);
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest({
                    url: '/json/device/setConfig',
                    params:
                    {
                        dsuid : this.dSUID,
                        'class': 3,
                        index: 0x1e, // ltmode
                        value : value
                    },
                    failureMsg: dSS._("Could not save input mode settings"),
                    scope: this,
                    evt: 'formsubmitted'
                },
                function(success) {
                    if (success === true) {
                        var devStore = dSS.data.Loader.getDeviceStore();
                        var record = devStore.getById(dSUID);
                        if (record !== undefined) {
                            record.set('buttonInputMode', parseInt(value, 10));
                            record.commit();
                        }
                    }
                }
            );
        },
        this);
    },
    updateGroup: function(group) {
        this.group = group;
    },
    setRadioGroup: function(value) {
        this.radio.setValue(value, value);
    }
});

Ext.reg('dssdevinputmodeform', dSS.devprops.InputModeForm);

