Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OperationTypeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Missing dSUID"));
            return;
        }

        this.dSUID = config.dSUID;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.opTypeSelection = new Ext.form.ComboBox({
            cls: 'dsClsOperationTypeForm_OpTypeSelectionCombo',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            fieldLabel: dSS._("Operation type"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [
                        dSS.config.deviceParams.OUTPUT_MODE_ALTSWITCH,
                        dSS._("Switch")
                    ],
                    [
                        dSS.config.deviceParams.OUTPUT_MODE_POWERSAVE,
                        dSS._("Powersave")
                    ],
                    [
                        dSS.config.deviceParams.OUTPUT_MODE_WIPE,
                        dSS._("Wipe")
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            cls: 'dsClsOperationTypeForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            height: 35,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.opTypeSelection
                    ]
                }
            ]
        });

        dSS.devprops.OperationTypeForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.opTypeSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.opTypeSelection.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.loadedValue = data.json.result[this.dSUID][0].outputMode;
                this.opTypeSelection.setValue(this.loadedValue);
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var value = this.opTypeSelection.getValue();
            var dSUID = this.dSUID;
            if (value === this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest({
                    url: '/json/device/setConfig',
                    params:
                    {
                        'class' : 3,
                        index: 0,
                        dsuid: this.dSUID,
                        value: value
                    },
                    failureMsg: dSS._("Could not save operation type"),
                    scope: this,
                    evt: 'formsubmitted'
                },
                function(success) {
                    if (success === true) {
                        var devStore = dSS.data.Loader.getDeviceStore();
                        var record = devStore.getById(dSUID);
                        if (record !== undefined) {
                            record.set('outputMode', parseInt(value, 10));
                            record.commit();
                        }
                    }
                }
            );
        },
        this);
    }
});

Ext.reg('dssdevoptypeform', dSS.devprops.OperationTypeForm);

