Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputBlinkConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];
        this.deviceNumber = devInfo[1];

        var fid = dSS.util.parseFunctionID(this.device.get('functionID'));
        var color = dSS.config.deviceColors[fid[0]];

        var mainLabel = dSS._("Blink behaviour for output on scene calls");
        var foreverText = dSS._("forever");
        var repetitionsText = dSS._("Repetitions");
        var turnOnText = dSS._("Turn-on duration");
        var turnOffText = dSS._("Turn-off duration");
        var turnOnMinValue = 0;
        var turnOffMinValue = 0;

        if ((this.deviceType == dSS.config.deviceTypeNum.UMR) ||
            (color == 'SW')) {
            mainLabel = dSS._("Impulse behaviour for output on scene calls");
            foreverText = dSS._("endless");
            repetitionsText = dSS._("Number of pulses");
            turnOnText = dSS._("Pulse duration");
            turnOffText = dSS._("Pulse pause");
            if ((this.deviceType == dSS.config.deviceTypeNum.ZWS) &&
                (this.deviceNumber == 205)) {
                turnOnMinValue = 0.5;
                turnOffMinValue = 0.5;
            }
        }

        this.repetitions = new Ext.Panel(
        {
            cls: 'dsClsOutputBlinkConfigForm_Repetitions',

            border: false,
            height: 35,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: repetitionsText + ":",
                    width: 200,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsOutputBlinkConfigForm_Repetitions_CountSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'count',
                    minValue: 0,
                    maxValue: 254,
                    allowDecimals: false,
                    defaultValue: 1, // default value as defined by spec
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 20
                },
                {
                    cls: 'dsClsOutputBlinkConfigForm_Repetitions_ForeverCheckbox',

                    xtype: 'checkbox',
                    ref: 'forever'
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: foreverText, 
                    style: { marginTop: '5px' }
                }
            ]
        });

        this.onDelay = new Ext.Panel(
        {
            cls: 'dsClsOutputBlinkConfigForm_OnDelay',

            border: false,
            height: 35,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: turnOnText + ":",
                    width: 200,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsOutputBlinkConfigForm_OnDelay_SecondsSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: turnOnMinValue,
                    maxValue: 8.4,
                    incrementValue: 0.1,
                    allowDecimals: true,
                    defaultValue: turnOnMinValue,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    style: { marginTop: '5px' }
                }
            ]
        });

        this.offDelay = new Ext.Panel(
        {
            cls: 'dsClsOutputBlinkConfigForm_OffDelay',

            border: false,
            height: 25,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: turnOffText + ":",
                    width: 200,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsOutputBlinkConfigForm_OffDelay_SecondsSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: turnOffMinValue,
                    maxValue: 8.4,
                    incrementValue: 0.1,
                    allowDecimals: true,
                    defaultValue: turnOffMinValue,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    style: { marginTop: '5px' }
                }
            ]
        });

        Ext.apply(this, {
            cls: 'dsClsOutputBlinkConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'formretry' ],
            padding: 5,
            scope: this,
            border: false,
            items:
            [
                {
                    xtype: 'label',
                    text: mainLabel 
                },
                {
                    xtype: 'box',
                    height: 10
                },
                this.repetitions,
                this.onDelay,
                this.offDelay
            ]
        });

        dSS.devprops.OutputBlinkConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.onDelay.seconds.on('valid', function() {
                        this.fireEvent('formchanged');
                    },
                    this);

                this.offDelay.seconds.on('valid', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.repetitions.count.on('valid', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.onDelay.seconds.on('spin', function() {
                        this.fireEvent('formchanged');
                    },
                    this);

                this.offDelay.seconds.on('spin', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.repetitions.count.on('spin', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.repetitions.forever.on('check', function(obj, value) {
                         this.fireEvent('formchanged');
                         if (value === true) {
                            this.repetitions.count.disable();
                         } else {
                            this.repetitions.count.enable();
                         }
                    },
                    this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.BLINK_CONFIG) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.repetitions.count.markInvalid(dSS._(data.json.message));

                    this.offDelay.seconds.markInvalid(dSS._(data.json.message));
                    this.onDelay.seconds.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.loadedCount = data.json.result.count;
                this.loadedOnDelay = data.json.result.ondelay.toPrecision(4);
                this.loadedOffDelay = data.json.result.offdelay.toPrecision(4);

                this.onDelay.seconds.setValue(this.loadedOnDelay);
                this.offDelay.seconds.setValue(this.loadedOffDelay);
                if (this.loadedCount < 255) {
                    this.repetitions.count.setValue(this.loadedCount);
                } else {
                    this.repetitions.forever.setValue(true);
                    this.repetitions.count.setValue(1); // default value (spec)
                    this.repetitions.count.disable();
                }

                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            if (this.getForm().isValid() !== true) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Please correct values in the red " +
                                    "marked fields and try again"));
                this.fireEvent('formretry');
                return;
            }

            var on = this.onDelay.seconds.getValue();
            var off = this.offDelay.seconds.getValue();
            var count;
            if (this.repetitions.forever.getValue() === true) {
                count = 255;
            } else {
                count = this.repetitions.count.getValue();
            }

            if ((this.loadedOnDelay == on) &&
                (this.loadedOffDelay == off) &&
                (this.loadedCount == count)) {
                this.fireEvent('formsubmitted');
                return;
            }

            // do not submit values that have not changed
            if (this.loadedCount === count) {
                count = '';
            }

            if (this.loadedOnDelay === on) {
                on = '';
            }

            if (this.loadedOffDelay === off) {
                off = '';
            }

            dSS.util.genericRequest.defer(2000, this,
            [
                {
                    url: '/json/device/setBlinkConfig',
                    params:
                    {
                        dsuid: this.dSUID,
                        count: count,
                        ondelay: on,
                        offdelay: off
                    },
                    failureMsg: dSS._("Could not save blink configuration"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        },
        this);
    }
});

Ext.reg('outputblinkconfigform', dSS.devprops.OutputBlinkConfigForm);

