Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputSwitchTresholdConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];
        this.deviceNumber = devInfo[1];

        this.hideInTC = false;
        if (((this.deviceType === dSS.config.deviceTypeNum.ZWS) &&
            (this.deviceNumber === 205)) ||
            ((this.deviceType == dSS.config.deviceTypeNum.UMR) &&
             (this.deviceNumber == 200))) {
            this.hideInTC = true;
        }

        this.threshold = new Ext.Panel(
        {
            cls: 'dsClsOutputSwitchTresholdConfigForm_Threshold',
            border: false,
            height: 40,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    height: 40,
                    text: dSS._("Switch output off below") + ":",
                    width: 200,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsOutputSwitchTresholdConfigForm_Threshold_BelowSpinnerField',

                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'threshold',
                    minValue: 1,
                    maxValue: 99,
                    allowDecimals: false,
                    defaultValue: 128, // default value as defined by spec
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: "%",
                    style: { marginTop: '5px' }
                }
            ]
        });

        Ext.apply(this, {
            cls: 'dsClsOutputSwitchTresholdConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'formretry' ],
            padding: 5,
            height: 45,
            scope: this,
            border: false,
            disabled: (dSS.util.getJokerGroup(this.device.get('groups')) === dSS.config.groups.TEMPCONTROL),
            items:
            [
                this.threshold
            ]
        });

        dSS.devprops.OutputSwitchTresholdConfigForm.superclass.constructor.apply(this, arguments);

        if (this.hideInTC === true) {
            dSS.MessageBus.on('jokerconfigformchanged', this.updateVisibility, this);
        }

        this.on('loadcomplete', function() {
                this.threshold.threshold.on('valid', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.threshold.threshold.on('spin', function() {
                         this.fireEvent('formchanged');
                    },
                    this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
                if (this.hideInTC === true) {
                    dSS.MessageBus.removeListener('jokerconfigformchanged',
                                                  this.updateVisibility, this);
                }
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.OUTPUT_SWITCH_THRESHOLD) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.threshold.threshold.markInvalid(dSS._(data.json.message));

                    this.fireEvent('formloaded', true);
                    return;
                }

                this.loadedThreshold = data.json.result.threshold;
                this.threshold.threshold.setValue(Math.ceil(this.loadedThreshold * 100 / 256));

                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            if (this.disabled) {
                this.fireEvent('formsubmitted');
                return;
            }

            if (this.getForm().isValid() !== true) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Please correct values in the red " +
                                    "marked fields and try again"));
                this.fireEvent('formretry');
                return;
            }

            var threshold = Math.floor(this.threshold.threshold.getValue() * 256 / 100);

            if (this.loadedThreshold == threshold) {
                this.fireEvent('formsubmitted');
                return;
            }

            // do not submit values that have not changed
            if (this.loadedThreshold !== threshold) {
                dSS.util.genericRequest.defer(2000, this,
                [
                    {
                        url: '/json/device/setSwitchThreshold',
                        params:
                        {
                            dsuid: this.dSUID,
                            threshold: threshold
                        },
                        failureMsg: dSS._("Could not save switch threshold configuration"),
                        scope: this,
                        evt: 'formsubmitted'
                    }
                ]);
            }
        },
        this);
    },
    updateVisibility: function(group) {
        if (group == dSS.config.groups.TEMPCONTROL) {
            this.disable();
        } else {
            this.enable();
        }
    }
});

Ext.reg('outputswitchtresholdconfigform', dSS.devprops.OutputSwitchTresholdConfigForm);

