Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.TemperatureOffsetConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.offsetLoaded = false;
        this.sensorLoaded = false;
        this.loadedOffset = 0;
        this.forceReset = false;

        this.offset = new Ext.Panel(
        {
            cls: 'dsClsTemperatureOffsetConfigForm_Offset',

            border: false,
            bubbleEvents: [ 'change' ],
            height: 35,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Current temperature") + ":",
                    width: 150,
                    style: { marginTop: '5px' }
                },
                {
                    cls: 'dsClsTemperatureOffsetConfigForm_Offset_TemperatureSpinnerField',
                    
                    xtype: 'spinnerfield',
                    width: 60,
                    ref: 'temperature',
                    allowDecimals: true,
                    allowNegaive: true,
                    decimalPrecision: 1,
                    allowEmpty: false,
                    incrementValue: 0.1
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("°C"),
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'box',
                    width: 10
                },
                {
                    cls: 'dsClsTemperatureOffsetConfigForm_Offset_ResetOffsetButton',

                    xtype: 'button',
                    text: dSS._("Reset offset"),
                    ref: 'resetbutton',
                    width: 115,
                    disabled: true,
                    handler: this.resetOffset.createDelegate(this)
                }
            ]
        });

        Ext.apply(this, {
            cls: 'dsClsTemperatureOffsetConfigForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("To set a temperature offset, please enter " +
                                "the current valid temperature.")
                },
                {
                    xtype: 'box',
                    height: 10
                },
                this.offset
            ]
        });

        dSS.devprops.TemperatureOffsetConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {

                this.offset.temperature.on('change', function(field, n, o) {
                        this.fireEvent('formchanged');
                        this.offset.resetbutton.enable();
                        this.forceReset = false;
                    },
                    this);

               this.offset.temperature.on('spin', function() {
                         this.fireEvent('formchanged');
                         this.offset.resetbutton.enable();
                         this.forceReset = false;
                    },
                    this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader == dSS.devprops.Loaders.SENSOR_VALUE) {

                    if (data.json.ok !== true) {
                        this.offset.temperature.markInvalid(
                                dSS._(data.json.message));
                        return;
                    }

                    this.loadedTemperature =
                        dSS.util.Sensor2Temperature(
                                data.json.result.sensorValue);
                    this.offset.temperature.setValue(this.loadedTemperature);
                    this.sensorLoaded = true;
                } else if (data.loader == dSS.devprops.Loaders.TEMPERATURE_OFFSET) {
                    if (data.json.ok !== true) {
                        this.offset.temperature.markInvalid(
                                dSS._(data.json.message));
                        return;
                    }
                    this.loadedOffset = data.json.result.offset;
                    if (this.loadedOffset != 0) {
                        this.offset.resetbutton.enable();
                    }
                    this.offsetLoaded = true;
                }

                if ((this.sensorLoaded == true) &&
                    (this.offsetLoaded == true)) {

                    var real = this.loadedTemperature - (this.loadedOffset / 10);
                    this.offset.temperature.setMinValue(Math.ceil((real - 12.7) * 10) / 10);
                    this.offset.temperature.setMaxValue(Math.floor((real + 12.7) * 10) / 10);
                    this.fireEvent('formloaded', true);
                }
            },
            this);

        this.on('submit', function() {
            if (this.getForm().isValid() !== true) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Please correct values in the red " +
                                    "marked fields and try again"));
                this.fireEvent('formretry');
                return;
            }

            var temperature = this.offset.temperature.getValue();

            var setOffset = 0;
            if (this.forceReset != true) {
                setOffset = this.loadedOffset + Math.round((temperature - this.loadedTemperature) * 10);
            }

            if ((this.forceReset != true) && (this.loadedTemperature == temperature)) {
                this.fireEvent('formsubmitted');
            } else {
                dSS.util.genericRequest.defer(2000, this,
                [
                    {
                        url: '/json/device/setTemperatureOffset',
                        params:
                        {
                            dsuid: this.dSUID,
                            offset: setOffset
                        },
                        failureMsg: dSS._("Could not set temperature offset"),
                        scope: this,
                        evt: 'formsubmitted'
                    }
                ]);
            }
        },
        this);
    },
    resetOffset: function() {
        this.offset.temperature.setValue(this.loadedTemperature - (this.loadedOffset / 10));
        this.forceReset = true;
        this.fireEvent('formchanged');
        this.offset.resetbutton.disable();
    }
});

Ext.reg('temperatureoffsetconfigform', dSS.devprops.TemperatureOffsetConfigForm);

