Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured group)
dSS.devprops.WindProtectionClassForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.device === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing device"));
            return;
        }

        var list =
        [
            [
                dSS.config.WindProtectionClass.NONE,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.NONE))
            ]
        ]
        if (config.matrix.windprotectionconfigawning) {
            list.push([
                          dSS.config.WindProtectionClass.WPC1_AWNING,
                          dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC1_AWNING))
                      ],
                      [
                          dSS.config.WindProtectionClass.WPC2_AWNING,
                          dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC2_AWNING))
                      ],
                      [
                          dSS.config.WindProtectionClass.WPC3_AWNING,
                          dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC3_AWNING))
                      ]);
        } else if (config.matrix.windprotectionconfigblind) {
            list.push(
            [
                dSS.config.WindProtectionClass.WPC1_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC1_BLIND))
            ],
            [
                dSS.config.WindProtectionClass.WPC2_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC2_BLIND))
            ],
            [
                dSS.config.WindProtectionClass.WPC3_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC3_BLIND))
            ],
            [
                dSS.config.WindProtectionClass.WPC4_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC4_BLIND))
            ],
            [
                dSS.config.WindProtectionClass.WPC5_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC5_BLIND))
            ],
            [
                dSS.config.WindProtectionClass.WPC5_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC5_BLIND))
            ],
            [
                dSS.config.WindProtectionClass.WPC6_BLIND,
                dSS._(dSS.util.getWindProtectionString(dSS.config.WindProtectionClass.WPC6_BLIND))
            ]);
        }

        this.device = config.device;
        this.dSUID = this.device.get('dsuid');
        this.protectionClass = dSS.config.WindProtectionClass.NONE;

        this.addEvents('formsubmitted', 'formloaded');

        this.protectionSelection = new Ext.form.ComboBox({
            cls: 'dsClsWindProtectionClassForm_ProtectionSelectionCombo',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            fieldLabel: dSS._("Wind protection"),
            labelStyle: 'margin-top: 5px',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            cls: 'dsClsWindProtectionClassForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            disabled: this.device.get('clusterLock'),
            padding: 5,
            height: 35,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.protectionSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.WindProtectionClassForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.protectionSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.WIND_PROTECTION) {
                    return;
                }

                if (data.json.ok === true) {
                    this.protectionClass = data.json.result["class"];
                    this.protectionSelection.setValue(this.protectionClass);
                } else {
                    this.protectionSelection.markInvalid(data.json.message);
                }


                this.fireEvent('formloaded', true);
            },
            this);


        this.on('submit', function() {
                var value = this.protectionSelection.getValue();
                var dSUID = this.dSUID;

                dSS.util.genericRequest({
                    url: '/json/device/setWindProtectionClass',
                    scope: this,
                    params:
                    {
                        dsuid: this.dSUID,
                        "class": value
                    },
                    failureMsg: dSS._("Could not set wind protection"),
                    evt: 'formsubmitted'
                });
            },
            this);
    }
});

Ext.reg('dssdevwindprotectionclassform', dSS.devprops.WindProtectionClassForm);

