/* globals Ext, dSS */

Ext.namespace('dSS', 'dSS.devices');

// *STANDARD
dSS.devices.DeviceSettingsDefaultView = Ext.extend(Ext.Panel, {
    constructor: function (config) {
        var xPos = config.xPos;
        var xPosOffset = config.xPosOffset;

        this.maxValue = config.maxValue;
        this.closed = false;
        this.dSUID = config.dSUID;
        this.device = config.device;

        this.valuesConfig = {
            inputMin: 0,
            inputMax: 100,
            inputStep: 5,
            grKl300InputStep: 10,
            bladeDelay: 300,
            valueDelay: 500,
            flowRateDelay: 800,
            maxOutputValue: 0xff
        };
        // GR-KL300, GR-HKL300:
        this.attachedTerminalDevice = null;

        this.apartmentMode = false;
        if (config.apartmentMode != null) {
            this.apartmentMode = config.apartmentMode;
        }

        this.dirtyValue = false;
        this.dirtyAngle = false;

        this.previousValue = 0;
        this.previousBladeAngle = 0;
        this.decPrecision = config.decPrecision === 0
            ? 1
            : 0.001;

        var increment = this.decPrecision;
        if (this.apartmentMode && config.matrix.grkl387workaround) {
            increment = dSS.config.GRKL387SliderIncrement;
        }

        var radioItems = [
            {
                cls: 'dsClsDeviceSettingsDefaultView_FlowRateRadioGroup_NormalModeRadio',

                boxLabel: dSS._('Normal mode'),
                name: 'rb_flow_' + config.dSUID,
                inputValue: dSS.config.scenes.STOP
            },
            {
                cls: 'dsClsDeviceSettingsDefaultView_FlowRateRadioGroup_Force100Radio',

                boxLabel: dSS._('Force to 100%'),
                name: 'rb_flow_' + config.dSUID,
                inputValue: dSS.config.scenes.LOCAL_ON
            },
            {
                cls: 'dsClsDeviceSettingsDefaultView_FlowRateRadioGroup_Force0Radio',

                boxLabel: dSS._('Force to 0%'),
                name: 'rb_flow_' + config.dSUID,
                inputValue: dSS.config.scenes.LOCAL_OFF
            }

        ];

        var bl361Hack = false;
        var isBL = config.devClass === dSS.config.deviceClass.BL;
        if (isBL) {
            var device = dSS.util.getDeviceInfoFromRecord(config.device);
            if (
                device.type === 'KM' && device.number === 200 &&
                config.matrix.pwmvalue &&
                config.device.get('productRevision') == 0x361 &&
                config.device.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED
            ) {
                bl361Hack = true;
            }
        }

        Ext.apply(this, {
            cls: 'dsClsDeviceSettingsDefaultView',

            layout: 'absolute',
            border: false,
            padding: 5,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged', 'enableme' ],
            items: [
                {
                    xtype: 'label',
                    ref: 'valueLabel',
                    x: 10,
                    y: 16,
                    disabled: this.dummy,
                    hidden: bl361Hack,
                    text: isBL
                        ? dSS._('Flow rate') + ':'
                        : dSS._('Value') + ':'
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_' + (isBL ? 'FlowRate' : 'Value') + 'NumberField',

                    xtype: 'numberfield',
                    ref: 'valueField',
                    x: xPos - xPosOffset,
                    y: 12,
                    width: 50,
                    allowBlank: false,
                    hidden: bl361Hack,
                    allowNegative: false,
                    allowDecimals: config.decPrecision !== 0,
                    decimalPrecision: config.decPrecision,
                    forceDecimalPrecision: true,
                    disabled: isBL,
                    value: this.valuesConfig.inputMin,
                    minValue: this.valuesConfig.inputMin,
                    maxValue: this.valuesConfig.inputMax,
                    listeners: {
                        change: this.handleValueChanged.createDelegate(this)
                    }
                },
                {
                    x: xPos - (xPosOffset - 55),
                    y: 16,
                    ref: 'valuePostLabel',
                    hidden: bl361Hack,
                    xtype: 'label',
                    text: '%'
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_ValueSlider',

                    xtype: 'slider',
                    ref: 'valueSlider',
                    x: xPos - (xPosOffset - 70),
                    y: 12,
                    height: 150,
                    value: this.valuesConfig.inputMin,
                    minValue: this.valuesConfig.inputMin,
                    maxValue: this.valuesConfig.inputMax,
                    vertical: true,
                    increment: increment,
                    decimalPrecision: config.decPrecision,
                    hidden: isBL,
                    plugins: new Ext.slider.Tip({
                        cls: 'dsClsDeviceSettingsDefaultView_ValueSlider_Tip',

                        getText: function (thumb) {
                            return String.format('{0}%', thumb.value);
                        }
                    }),
                    listeners: {
                        changeComplete: this.handleValueChanged.createDelegate(this)
                    }
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_StopButton',

                    xtype: 'button',
                    ref: 'stopButton',
                    x: xPos - xPosOffset,
                    y: 140,
                    text: dSS._('Stop'),
                    width: 50,
                    handler: this.handleStop.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_MaxButton',

                    xtype: 'button',
                    ref: 'maxButton',
                    x: xPos - (xPosOffset - 102),
                    y: 12,
                    text: dSS._('Max'),
                    tooltip: dSS._('Set maximum value'),
                    width: 130,
                    handler: this.handleMax.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_MinButton',

                    xtype: 'button',
                    ref: 'minButton',
                    x: xPos - (xPosOffset - 102),
                    y: 140,
                    text: dSS._('Min'),
                    tooltip: dSS._('Set minimum value'),
                    width: 130,
                    handler: this.handleMin.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_IncButton',

                    xtype: 'button',
                    ref: 'incButton',
                    x: xPos - (xPosOffset - 102),
                    y: 54,
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._('Increase value'),
                    width: 130,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_DecButton',

                    xtype: 'button',
                    ref: 'decButton',
                    x: xPos - (xPosOffset - 102),
                    y: 98,
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._('Decrease value'),
                    width: 130,
                    handler: this.handleDec.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'flowRateLabel',
                    x: 10,
                    y: 43,
                    text: dSS._('Control of flow rate') + ':',
                    hidden: !isBL
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_FlowRateRadioGroup',

                    xtype: 'radiogroup',
                    ref: 'flowRateSelection',
                    columns: 1,
                    items: radioItems,
                    hidden: !isBL,
                    x: xPos - xPosOffset - 5,
                    y: 41
                },
                {
                    xtype: 'label',
                    ref: 'forceModeHelp',
                    hidden: !isBL,
                    text: dSS._('Forced flow rate resets to normal mode after 6 hours.'),
                    x: xPos - xPosOffset,
                    y: 115
                },
                {
                    xtype: 'label',
                    ref: 'opModeLabel',
                    x: 10,
                    y: 152,
                    text: dSS._('Current operation mode') + ':',
                    hidden: !isBL
                },
                {
                    xtype: 'label',
                    ref: 'opModeText',
                    x: xPos - xPosOffset,
                    y: 152,
                    hidden: !isBL
                },
                {
                    xtype: 'label',
                    ref: 'bladeAngleLabel',
                    x: 10,
                    y: 178,
                    text: dSS._('Set blade angle') + ':'
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_BladeAngleNumberField',

                    xtype: 'numberfield',
                    ref: 'bladeAngleField',
                    x: xPos - xPosOffset,
                    y: 174,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: this.valuesConfig.inputMin,
                    minValue: this.valuesConfig.inputMin,
                    maxValue: this.valuesConfig.inputMax,
                    listeners: {
                        change: this.handleBladeAngleChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'label',
                    ref: 'bladeAngleUnit',
                    x: xPos - (xPosOffset - 55),
                    y: 178,
                    text: '%'
                },
                {
                    cls: 'dsClsDeviceSettingsDefaultView_BladeAngleSlider',

                    xtype: 'slider',
                    ref: 'bladeAngleSlider',
                    x: xPos - (xPosOffset - 74),
                    y: 174,
                    width: 160,
                    value: this.valuesConfig.inputMin,
                    increment: 1,
                    minValue: this.valuesConfig.inputMin,
                    maxValue: this.valuesConfig.inputMax,
                    vertical: false,
                    decimalPrecision: 1,
                    plugins: new Ext.slider.Tip({
                        cls: 'dsClsDeviceSettingsDefaultView_BladeAngleSlider_Tip',

                        getText: function (thumb) {
                            return String.format('{0}%', thumb.value);
                        }
                    }),
                    listeners: {
                        changeComplete: this.handleBladeAngleChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'label',
                    ref: 'grKLHack',
                    x: 10,
                    y: 205,
                    text: dSS._('This device only supports output values in 6.66% steps.'),
                    hidden: !this.apartmentMode || !config.matrix.grkl387workaround
                }
            ]
        });

        dSS.devices.DeviceSettingsDefaultView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function () {
            this.killRequests();
            this.closed = true;
        }, this);

        this.flowRateSelection.on('change', function (radiogroup, radio) {
            if (this.apartmentMode) {
                return;
            }
            dSS.util.callScene(this.dSUID, radio.getRawValue());
            this.readDeviceValue.defer(this.valuesConfig.valueDelay, this, [ this.dSUID ]);
        }, this);

        this.valueField.on('specialkey', function (field, e) {
            if (e.getKey() === e.ENTER) {
                var nv = parseFloat(field.getValue());
                var ov = parseFloat(this.valueSlider.getValue());
                if (this.apartmentMode && this.matrix.grkl387workaround) {
                    nv = (Math.round(nv * 15 / 100) / 15) * 100;
                    field.setValue(nv);
                }
                if (nv !== ov) {
                    this.handleValueChanged(field, nv);
                }
            }
        }, this);

        this.bladeAngleField.on('specialkey', function (field, e) {
            if (e.getKey() === e.ENTER) {
                var nv = field.getValue();
                this.handleBladeAngleChanged(field, nv);
            }
        }, this);
    },

    setMatrix: function (matrix) {
        this.matrix = matrix;
    },

    handleValueChanged: function (container, newVal, oldVal) {
        if (newVal === '') {
            this.valueField.setValue(this.valueSlider.getValue());
            return;
        }
        var value = dSS.devices.helpers.ensureValueWithinRange({
            value: newVal,
            min: this.valuesConfig.inputMin,
            max: this.valuesConfig.inputMax
        });
        if (value === oldVal) {
            if (value !== newVal) {
                this.valueField.setValue(value);
            }
            return;
        }

        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.valueSlider.setValue(value);
        this.valueField.setValue(value);
        this.dirtyValue = true;
        if (!this.apartmentMode) {
            this.setValueOnDevice(value);
        }
        if (this.matrix.shadebladeang) {
            if (value >= 0.9 * this.valuesConfig.inputMax) {
                this.handleBladeAngleChanged.defer(this.valuesConfig.bladeDelay, this, [ this.bladeAngleField, this.valuesConfig.inputMax ]);
            }
        }
    },

    setValueOnDevice: function (value) {
        if (value === '') {
            return;
        }
        this.previousValue = value;

        if (this.maxValue <= this.valuesConfig.maxOutputValue) {
            dSS.util.setValueOnDevice(this.dSUID, dSS.util.percentToValue(value, this.maxValue));
        } else {
            dSS.util.setOutputValueOnDevice(this.dSUID, 0x02, dSS.util.percentToValue(value, this.maxValue));
        }
    },

    handleBladeAngleChanged: function (container, newVal) {
        if (newVal === '') {
            this.bladeAngleField.setValue(this.bladeAngleSlider.getValue());
            return;
        }
        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.dirtyAngle = true;
        this.bladeAngleField.setValue(newVal);
        this.bladeAngleSlider.setValue(newVal);

        if (!this.apartmentMode && newVal !== this.previousBladeAngle) {
            this.setAngleOnDevice(newVal);
        }
    },

    setAngleOnDevice: function (angle) {
        dSS.util.genericRequest({
            url: '/json/device/setConfig',
            scope: this,
            params: {
                class: 64,
                index: 0x04,
                dsuid: this.dSUID,
                value: Math.round(angle / this.valuesConfig.inputMax * this.valuesConfig.maxOutputValue)
            },
            failureMsg: dSS._('Could not set device blade angle')
        });
        this.previousBladeAngle = angle;
    },

    handleMax: function () {
        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            var sceneToCall = dSS.config.scenes.MAX;
            dSS.util.callScene(this.dSUID, sceneToCall);
        } else {
            this.fireEvent('formchanged');
        }
        this.valueField.setValue(this.valuesConfig.inputMax);
        this.valueSlider.setValue(this.valuesConfig.inputMax);
        this.dirtyValue = true;
        if (this.matrix.shadebladeang && (!this.attachedTerminalDevice || this.deviceHasFins())) {
            this.bladeAngleField.setValue(this.valuesConfig.inputMax);
            this.bladeAngleSlider.setValue(this.valuesConfig.inputMax);
            this.dirtyAngle = false;
            if (!this.apartmentMode) {
                this.handleBladeAngleChanged.defer(this.valuesConfig.bladeDelay, this, [ this.bladeAngleField, this.valuesConfig.inputMax ]);
            }
        }
    },

    handleMin: function () {
        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            var sceneToCall = dSS.config.scenes.MIN;
            dSS.util.callScene(this.dSUID, sceneToCall);
        } else {
            this.fireEvent('formchanged');
        }
        this.valueField.setValue(this.valuesConfig.inputMin);
        this.valueSlider.setValue(this.valuesConfig.inputMin);
        this.dirtyValue = true;
        if (this.matrix.shadebladeang && (!this.attachedTerminalDevice || this.deviceHasFins())) {
            this.bladeAngleField.setValue(this.valuesConfig.inputMin);
            this.bladeAngleSlider.setValue(this.valuesConfig.inputMin);
            this.dirtyAngle = false;
            if (!this.apartmentMode) {
                this.handleBladeAngleChanged.defer(this.valuesConfig.bladeDelay, this, [ this.bladeAngleField, this.valuesConfig.inputMin ]);
            }
        }
    },

    handleStop: function () {
        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
        } else {
            this.fireEvent('formchanged');
        }
        if (!this.apartmentMode && this.matrix.shadeposition) {
            this.dirtyValue = true;
            this.readDeviceValue.defer(this.valuesConfig.valueDelay, this, [ this.dSUID ]);
        }
    },

    handleInc: function () {
        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
        } else {
            this.fireEvent('formchanged');
        }

        this.dirtyValue = true;

        var setShadeValueInsteadOfScene = this.setValueInsteadOfSceneOnIncDecForShade();
        if (this.matrix.shadeposition && !this.apartmentMode) {
            if (!setShadeValueInsteadOfScene) {
                var sceneToCall = this.swapMinMax()
                    ? dSS.config.scenes.DEC
                    : dSS.config.scenes.INC;
                dSS.util.callScene(this.dSUID, sceneToCall);
                this.readDeviceValue.defer(this.valuesConfig.valueDelay, this, [ this.dSUID ]);
                return;
            }
        }

        var v = this.valueSlider.getValue() + (setShadeValueInsteadOfScene ? this.valuesConfig.grKl300InputStep : this.valuesConfig.inputStep);
        if (v > this.valuesConfig.inputMax) {
            v = this.valuesConfig.inputMax;
        }

        if (v != this.valueSlider.getValue()) {
            this.valueSlider.setValue(v);
            this.handleValueChanged(this.valueSlider, v);
        }
    },

    handleDec: function () {
        if (!this.apartmentMode) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
        } else {
            this.fireEvent('formchanged');
        }

        this.dirtyValue = true;

        var setShadeValueInsteadOfScene = this.setValueInsteadOfSceneOnIncDecForShade();
        if (this.matrix.shadeposition && !this.apartmentMode) {
            if (!setShadeValueInsteadOfScene) {
                var sceneToCall = this.swapMinMax()
                    ? dSS.config.scenes.INC
                    : dSS.config.scenes.DEC;
                dSS.util.callScene(this.dSUID, sceneToCall);
                this.readDeviceValue.defer(this.valuesConfig.valueDelay, this, [ this.dSUID ]);
                return;
            }
        }

        var v = this.valueSlider.getValue() - (setShadeValueInsteadOfScene ? this.valuesConfig.grKl300InputStep : this.valuesConfig.inputStep);
        if (v < this.valuesConfig.inputMin) {
            v = this.valuesConfig.inputMin;
        }

        this.handleValueChanged(this.valueSlider, v);
    },

    clearData: function () {
        this.killRequests();
        this.valueField.setValue(this.valuesConfig.inputMin);
        this.valueSlider.setValue(this.valuesConfig.inputMin);
        this.bladeAngleField.setValue(this.valuesConfig.inputMin);
        this.bladeAngleSlider.setValue(this.valuesConfig.inputMin);
        this.dirtyValue = false;
        this.dirtyAngle = false;
        this.previousValue = this.valuesConfig.inputMin;
        this.previousBladeAngle = this.valuesConfig.inputMin;
    },

    killRequests: function () {
        this.fireEvent('loading', false);

        if (this.AjaxRequestAttachedTerminalDevice != null) {
            Ext.Ajax.abort(this.AjaxRequestAttachedTerminalDevice);
            this.AjaxRequestAttachedTerminalDevice = null;
        }

        if (this.AjaxRequestValue != null) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = null;
        }

        if (this.AjaxRequestAngle != null) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = null;
        }

        if (this.AjaxRequestOperationMode != null) {
            Ext.Ajax.abort(this.AjaxRequestOperationMode);
            this.AjaxRequestOperationMode = null;
        }
    },

    readBladeAngleValue: function (dSUID) {
        this.killRequests();

        if (dSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        var request = 'getConfig';
        var params = { class: 64, index: 0x04, dsuid: dSUID };

        this.dirtyAngle = false;
        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params: params,
            scope: this,
            success: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = null;
                if (this.closed || this.dirtyAngle || dSUID !== this.dSUID) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (!jsonData.ok) {
                        dSS.MessageBus.fireEvent(
                            'statusmessage',
                            dSS._('Could not load current device blade angle position') + ': ' + jsonData.message
                        );
                        this.bladeAngleField.markInvalid(
                            dSS._('Could not load current device blade angle position') + ': ' + jsonData.message
                        );

                    } else if (!isNaN(jsonData.result.value)) {
                        this.previousBladeAngle = Math.round(jsonData.result.value * this.valuesConfig.inputMax / this.valuesConfig.maxOutputValue);
                        this.bladeAngleField.setValue(this.previousBladeAngle);
                        this.bladeAngleSlider.setValue(this.previousBladeAngle);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device blade angle position') + ': ' + err
                    );
                    this.bladeAngleField.markInvalid(
                        dSS._('Could not load current device blade angle position') + ': ' + err
                    );
                }
            },
            failure: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = null;
                if (this.closed || this.dirtyAngle || dSUID !== this.dSUID) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device blade angle position') + ': ' + jsonData.message
                    );
                    this.bladeAngleField.markInvalid(
                        dSS._('Could not load current device blade angle position') + ': ' + jsonData.message
                    );

                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device blade angle position') + ': ' + err
                    );
                    this.bladeAngleField.markInvalid(
                        dSS._('Could not load current device blade angle position') + ': ' + err
                    );
                }
            }
        });
    },

    readOperationMode: function (dSUID) {
        this.killRequests();

        if (dSUID !== this.dSUID) {
            return;
        }

        var bl361Hack = false;
        // yet another BL-KM200 hack for firmware 3.6.1
        var device = dSS.util.getDeviceInfoFromRecord(this.device);
        if (device.classId === dSS.config.deviceClass.BL) {
            if (
                device.type === 'KM' && device.number === 200 &&
                this.matrix.pwmvalue &&
                this.device.get('productRevision') == 0x361
            ) {
                bl361Hack = true;
            }
        }

        this.fireEvent('loading', true);
        this.opModeText.setText(dSS._('loading') + '...');
        var request = 'getConfig';
        var offset = 0x05;

        this.AjaxRequestOperationMode = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params: {
                class: 64,
                index: offset,
                dsuid: dSUID
            },
            scope: this,
            success: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = null;
                if (this.closed || dSUID !== this.dSUID) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (!jsonData.ok) {
                        dSS.MessageBus.fireEvent(
                            'statusmessage',
                            dSS._('Could not load current device operation mode') + ': ' + jsonData.message
                        );
                        this.opModeText.setText('');
                        this.flowRateSelection.suspendEvents(false);
                        this.flowRateSelection.setValue('');
                        this.resumeFlowRateEvents.defer(this.valuesConfig.flowRateDelay, this, [ this ]);

                    } else if (!isNaN(jsonData.result.value)) {
                        var text = dSS.config.HeatingOperationModeStrings[jsonData.result.value];
                        this.opModeText.setText(dSS._(text));
                        // #dsd-484:
                        // modes 0-3 => normal
                        // mode 4    => forced on
                        // mode 5    => forced off
                        this.flowRateSelection.suspendEvents(false);
                        switch (jsonData.result.value) {
                            case dSS.config.HeatingOperationMode.CONTROL:
                            case dSS.config.HeatingOperationMode.EMERGENCY:
                            case dSS.config.HeatingOperationMode.VALVE_PROTECTION:
                            case dSS.config.HeatingOperationMode.OUTPUT_DEACTIVATED:
                                this.flowRateSelection.setValue(dSS.config.scenes.STOP);
                                break;
                            case dSS.config.HeatingOperationMode.FORCED_MODE_ON:
                                this.flowRateSelection.setValue(dSS.config.scenes.LOCAL_ON);
                                if (bl361Hack) {
                                    this.valueSlider.setValue(this.valuesConfig.inputMax);
                                    this.valueField.setValue(this.valuesConfig.inputMax);
                                    this.previousValue = this.valuesConfig.inputMax;
                                }
                                break;
                            case dSS.config.HeatingOperationMode.FORCED_MODE_OFF:
                                this.flowRateSelection.setValue(dSS.config.scenes.LOCAL_OFF);
                                if (bl361Hack) {
                                    this.valueSlider.setValue(this.valuesConfig.inputMin);
                                    this.valueField.setValue(this.valuesConfig.inputMin);
                                    this.previousValue = this.valuesConfig.inputMin;
                                }
                                break;
                            default:
                                break;
                        }
                        // hack: deferring flowRateSelection.resumeEvents()
                        // directly did not work
                        this.resumeFlowRateEvents.defer(this.valuesConfig.flowRateDelay, this, [ this ]);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device operation mode') + ': ' + err
                    );
                    this.opModeText.setText('');
                    this.flowRateSelection.suspendEvents(false);
                    this.flowRateSelection.setValue('');
                    this.resumeFlowRateEvents.defer(this.valuesConfig.flowRateDelay, this, [ this ]);
                }
            },
            failure: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = null;
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device operation mode') + ': ' + jsonData.message
                    );
                    this.opModeText.setText('');
                    this.flowRateSelection.suspendEvents(false);
                    this.flowRateSelection.setValue('');
                    this.resumeFlowRateEvents.defer(this.valuesConfig.flowRateDelay, this, [ this ]);
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device operation mode') + ': ' + err
                    );
                    this.opModeText.setText('');
                    this.flowRateSelection.suspendEvents(false);
                    this.flowRateSelection.setValue('');
                    this.resumeFlowRateEvents.defer(this.valuesConfig.flowRateDelay, this, [ this ]);
                }
            }
        });
    },

    resumeFlowRateEvents: function (thiz) {
        if (thiz.flowRateSelection != null) {
            thiz.flowRateSelection.resumeEvents();
        }
    },

    swapMinMax: function () {
        return [
            dSS.config.AttachedShadeTerminalDevice.AWNING,
            dSS.config.AttachedShadeTerminalDevice.FACADE_AWNING_HORIZONTAL,
            // dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW
        ].indexOf(this.attachedTerminalDevice) !== -1;
    },
    setValueInsteadOfSceneOnIncDecForShade () {
        return [
            dSS.config.AttachedShadeTerminalDevice.AWNING,
            // dSS.config.AttachedShadeTerminalDevice.ELECTRIC_WINDOW,
            dSS.config.AttachedShadeTerminalDevice.ELECTRIC_CURTAIN
        ].indexOf(this.attachedTerminalDevice) !== -1;
    },
    deviceHasFins: function () {
        return [
            dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS,
            dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS
        ].indexOf(this.attachedTerminalDevice) !== -1;
    },

    loadData: function (dSUID, devClass, matrix, virtualDevice, bl361Hack, record) {
        this.killRequests();

        this.dSUID = dSUID;
        this.device = record;
        this.matrix = matrix;

        this.fireEvent('loading', true);

        this.valueField.setValue(this.valuesConfig.inputMin);
        this.valueSlider.setValue(this.valuesConfig.inputMin);
        this.bladeAngleField.setValue(this.valuesConfig.inputMin);
        this.bladeAngleSlider.setValue(this.valuesConfig.inputMin);

        if (matrix.shadeposition && matrix.outvalue8) {
            Ext.Msg.alert(dSS._('Error'), dSS._('Unsupported configuration'));
            return;
        }

        // #dsd-484: broken 3.6.1 firmware hack for BL-KM
        if (bl361Hack) {
            this.fireEvent('loading', false);
            return; // dialog won't get enabled
        }

        var device = dSS.util.getDeviceInfoFromRecord(record);
        // output mode dependency, hack for BL-KM
        if (devClass === dSS.config.deviceClass.BL) {
            if (
                device.type === 'KM' &&
                (device.number === 200 || device.number === 300)
            ) {
                var hack = dSS.util.getHeatingMods(matrix, record.get('outputMode'));
                matrix = hack.matrix;
                devClass = hack.deviceClass;
            }
        }

        this.fireEvent('enableme');
        this.readDeviceInfo(matrix, record);
    },

    readDeviceInfo (matrix, record) {
        var deviceIsOfGrKl300Type = dSS.util.isGrKl300TypeDevice(record);
        if (!deviceIsOfGrKl300Type) {
            this.attachedTerminalDevice = null;
            this.visualConfig(matrix, record, false);
            this.readDeviceValue(this.dSUID);
            return;
        }

        Ext.Ajax.request({
            url: '/api/v1/apartment/dsDevices/' + record.get('dsuid'),
            method: 'GET',
            scope: this,
            success: function (response) {
                if (this.closed) {
                    return;
                }
                try {
                    var data = JSON.parse(response.responseText).data;
                    if (data) {
                        this.attachedTerminalDevice = data.attributes.shadeAttributes.shadeType;
                    } else {
                        this.attachedTerminalDevice = dSS.config.AttachedShadeTerminalDevice.UNDEFINED;
                    }
                } catch (e) {
                    this.attachedTerminalDevice = dSS.config.AttachedShadeTerminalDevice.UNDEFINED;
                }
            },
            callback: function () {
                this.visualConfig(matrix, record, false);
                this.readDeviceValue(this.dSUID);
            }
        });
    },

    readDeviceValue: function (dSUID) {
        this.dirtyValue = false;
        this.dirtyAngle = false;

        if (dSS.util.isGrKl300TypeDevice(this.device)) {
            this.readDeviceValueUsingGetOutputValue(dSUID);
        } else {
            this.readDeviceValueUsingGetConfig(dSUID);
        }
    },

    readDeviceValueUsingGetOutputValue: function (dSUID) {
        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/getOutputValue',
            method: 'GET',
            params: {
                dsuid: dSUID,
                offset: 0
            },
            scope: this,
            success: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = null;
                if (this.closed || this.dirtyValue || dSUID !== this.dSUID) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (!jsonData.ok) {
                        dSS.MessageBus.fireEvent(
                            'statusmessage',
                            dSS._('Could not load current device value') + ': ' + jsonData.message
                        );
                        this.valueField.markInvalid(dSS._('Could not load current device value') + ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var percent = dSS.util.valueToPercent(jsonData.result.value, this.valuesConfig.maxOutputValue);
                        this.valueSlider.setValue(percent);
                        this.valueField.setValue(percent);
                        this.previousValue = percent;
                    }

                    if (this.matrix.shadebladeang) {
                        this.readBladeAngleValue(dSUID);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device value') + ': ' + err
                    );
                    this.valueField.markInvalid(
                        dSS._('Could not load current device value') + ': ' + err
                    );
                }
            },
            failure: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = null;
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device value') + ': ' + jsonData.message
                    );

                    if (this.matrix.shadebladeang) {
                        this.readBladeAngleValue(dSUID);
                    }
                    this.valueField.markInvalid(
                        dSS._('Could not load current device value') + ': ' + jsonData.message
                    );
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device value') + ': ' + err
                    );
                    this.valueField.markInvalid(
                        dSS._('Could not load current device value') + ': ' + err
                    );
                }
            }
        });
    },

    readDeviceValueUsingGetConfig: function (dSUID) {
        var request = 'getConfigWord';
        var offset = 0x02;

        if (this.maxValue <= this.valuesConfig.maxOutputValue) {
            request = 'getConfig';
            offset = 0;
        }

        // when in modes 64 and 65 read from bank 64, offset 4
        if (this.matrix.pwmvalue) {
            if (
                this.device.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM ||
                this.device.get('outputMode') == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED
            ) {
                request = 'getConfig';
                offset = 4;
            }
        }

        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params: {
                class: 64,
                index: offset,
                dsuid: dSUID
            },
            scope: this,
            success: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = null;
                if (this.closed || this.dirtyValue || dSUID !== this.dSUID) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (!jsonData.ok) {
                        dSS.MessageBus.fireEvent(
                            'statusmessage',
                            dSS._('Could not load current device value') + ': ' + jsonData.message
                        );
                        this.valueField.markInvalid(dSS._('Could not load current device value') + ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var percent;
                        if (this.matrix.pwmvalue) {
                            percent = jsonData.result.value;
                        } else {
                            percent = dSS.util.valueToPercent(jsonData.result.value, this.maxValue);
                        }
                        this.valueSlider.setValue(percent);
                        this.valueField.setValue(percent);
                        this.previousValue = percent;
                    }

                    if (this.matrix.shadebladeang) {
                        this.readBladeAngleValue(dSUID);
                    }
                    if (this.matrix.pwmvalue) {
                        this.readOperationMode(dSUID);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device value') + ': ' + err
                    );
                    this.valueField.markInvalid(
                        dSS._('Could not load current device value') + ': ' + err
                    );
                }
            },
            failure: function (response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = null;
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device value') + ': ' + jsonData.message
                    );

                    if (this.matrix.shadebladeang) {
                        this.readBladeAngleValue(dSUID);
                    }
                    if (this.matrix.pwmvalue) {
                        this.readOperationMode(dSUID);
                    }
                    this.valueField.markInvalid(
                        dSS._('Could not load current device value') + ': ' + jsonData.message
                    );
                } catch (err) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS._('Could not load current device value') + ': ' + err
                    );
                    this.valueField.markInvalid(
                        dSS._('Could not load current device value') + ': ' + err
                    );
                }
            }
        });
    },

    setDeviceValue: function (value) {
        if (isNaN(value)) {
            return;
        }

        var percent = dSS.util.valueToPercent(value, this.maxValue);
        this.valueSlider.setValue(percent);
        this.valueField.setValue(percent);
    },

    setBladeAngle: function (angle) {
        if (isNaN(angle)) {
            return;
        }

        this.previousBladeAngle = angle;

        var percent = Math.round(dSS.util.valueToPercent(angle, this.valuesConfig.maxOutputValue));
        this.bladeAngleField.setValue(percent);
        this.bladeAngleSlider.setValue(percent);
    },

    saveSceneValues: function (scene) {
        if (!this.isVisible()) {
            return;
        }
        if (this.valueField.getValue() === '' || this.matrix.shadebladeang && this.bladeAngleField.getValue() === '') {
            return;
        }
        if (!this.apartmentMode) {
            return;
        }

        var value = dSS.util.percentToValue(this.valueField.getValue(), this.maxValue);
        var angle = this.matrix.shadebladeang && !this.bladeAngleField.disabled
            ? dSS.util.percentToValue(this.bladeAngleField.getValue(), this.valuesConfig.maxOutputValue)
            : undefined;

        dSS.util.setSceneValue.defer(500, this, [ this.dSUID, scene, value, angle ]);
        this.resetDefaults();
    },

    resetDefaults: function () {
        if (!this.isVisible()) {
            return;
        }
        this.previousValue = this.valueField.getValue();
        if (this.matrix.shadebladeang) {
            this.previousBladeAngle = this.bladeAngleField.getValue();
        }
    },

    resetValues: function () {
        if (!this.isVisible()) {
            return;
        }
        var previousValue = this.previousValue;
        this.valueField.setValue(previousValue);
        this.valueSlider.setValue(previousValue);

        if (this.matrix.shadebladeang) {
            previousValue = this.previousBladeAngle;
            this.bladeAngleField.setValue(previousValue);
            this.bladeAngleSlider.setValue(previousValue);
        }
    },

    doPreview: function () {
        if (!this.isVisible()) {
            return;
        }

        var value = this.valueField.getValue();
        console.log(this.valueField.getValue(), this.valueSlider.getValue());
        if (value === '') {
            this.valueField.setValue(this.valueSlider.getValue());
            return;
        }

        this.setValueOnDevice(Number(value));

        if (this.matrix.shadebladeang) {
            var angle = this.bladeAngleField.getValue();
            this.setAngleOnDevice(angle);
        }
    },

    enableBlindInputs () {
        this.bladeAngleLabel.enable();
        this.bladeAngleField.enable();
        this.bladeAngleUnit.enable();
        this.bladeAngleSlider.enable();
    },
    disableBladeInputs () {
        this.bladeAngleLabel.disable();
        this.bladeAngleField.disable();
        this.bladeAngleUnit.disable();
        this.bladeAngleSlider.disable();
    },

    updateGrVisualConfig: function (attachedTerminalDevice) {
        this.attachedTerminalDevice = attachedTerminalDevice;
        this.maxButton.setText(dSS._('Up'));
        this.minButton.setText(dSS._('Down'));
        if (this.attachedTerminalDevice != null) {
            var deviceDirections = dSS.config.AttachedShadeTerminalDeviceCalibrationDirections[this.attachedTerminalDevice];
            switch (deviceDirections) {
                case 'IN_OUT':
                    this.maxButton.setText(dSS._('Out'));
                    this.minButton.setText(dSS._('In'));
                    break;
                case 'OPEN_CLOSED':
                    this.maxButton.setText(dSS._('Open'));
                    this.minButton.setText(dSS._('Close'));
                    break;
                case 'CLOSED_OPEN':
                    this.maxButton.setText(dSS._('Close'));
                    this.minButton.setText(dSS._('Open'));
                    break;
                // for other cases - leave default up/down labels
            }
            if (this.deviceHasFins()) {
                this.enableBlindInputs();
            } else {
                this.disableBladeInputs();
            }
        } else {
            this.enableBlindInputs();
        }
    },

    visualConfig: function (matrix, record, bl361Hack) {
        this.matrix = matrix;

        if (matrix.shadeposition) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (matrix.shadeposition) {
            this.valueLabel.setText(dSS._('Position') + ':');
        } else {
            this.valueLabel.setText(dSS._('Value') + ':');
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);
        this.bladeAngleSlider.setVisible(matrix.shadebladeang);
        this.grKLHack.setVisible(Boolean(this.apartmentMode && matrix.grkl387workaround));

        var device = dSS.util.getDeviceInfoFromRecord(record);
        if (
            device.classId === dSS.config.deviceClass.GR ||
            device.classId === dSS.config.deviceClass.GN ||
            device.classId === dSS.config.deviceClass.RT
        ) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }

        this.valueField.show();
        this.valueLabel.show();
        this.valuePostLabel.show();

        if (device.classId === dSS.config.deviceClass.GR) {
            this.updateGrVisualConfig(this.attachedTerminalDevice);
            this.flowRateLabel.hide();
            this.flowRateSelection.hide();
            this.forceModeHelp.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.show();
            if (this.apartmentMode && matrix.grkl387workaround) {
                this.valueSlider.increment = dSS.config.GRKL387SliderIncrement;
                this.valueSlider.keyIncrement = dSS.config.GRKL387SliderIncrement;
            } else {
                this.valueSlider.increment = this.decPrecision;
                this.valueSlider.keyIncrement = this.decPrecision;
            }
            this.valueSlider.enable();
        } else if (device.classId === dSS.config.deviceClass.BL && device.isVdcDevice) {
            this.maxButton.hide(); this.minButton.hide();
            this.decButton.hide(); this.incButton.hide();
            this.flowRateLabel.hide(); this.flowRateSelection.hide();
            this.forceModeHelp.hide();
            this.opModeLabel.hide(); this.opModeText.hide();
            this.valueField.disable(); this.valueSlider.hide();

        } else if (device.classId === dSS.config.deviceClass.BL && matrix.valvetype) {
            this.maxButton.hide();
            this.minButton.hide();
            this.decButton.hide();
            this.incButton.hide();
            this.flowRateLabel.show();
            this.flowRateSelection.show();
            this.forceModeHelp.show();
            this.opModeLabel.show();
            this.opModeText.show();
            this.opModeText.setText(dSS._('loading') + '...');
            this.valueField.disable();
            this.valueSlider.hide();
            this.valueLabel.setText(dSS._('Flow rate') + ':');
            if (bl361Hack) {
                this.valueField.hide();
                this.valueLabel.hide();
                this.valuePostLabel.hide();
            }

        } else if (device.classId === dSS.config.deviceClass.BL) {
            this.minButton.setText(dSS._('Min'));
            this.maxButton.setText(dSS._('Max'));
            this.flowRateLabel.hide();
            this.flowRateSelection.hide();
            this.forceModeHelp.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.show();

        } else {
            this.minButton.setText(dSS._('Min'));
            this.maxButton.setText(dSS._('Max'));
            this.flowRateLabel.hide();
            this.flowRateSelection.hide();
            this.forceModeHelp.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.show();
        }
    }
});
