Ext.namespace("dSS", "dSS.devices", "dSS.config");

dSS.devices.DeviceSettingsFCUView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var xPos = config.xPos;
        var xPosOffset = config.xPosOffset;

        this.closed = false;

        this.dSUID = config.dSUID;

        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.AjaxRequestFCUOutputChannels = undefined;

        this.dirtyFanSpeed = false;
        this.dirtyLouverPos = false;

        this.previousairflowintensity = 0;
        this.previousLouverPosition = 0;
        this.previousautomatic = false;
        this.previousswing = false;
        this.previousfanretain = false;
        this.previouslouverretain = false;

        this.fanspeedslider = new Ext.slider.SingleSlider({
            cls: 'dsClsDeviceSettingsFCUView_FanSpeedSlider',

            height: 70,
            value: 0,
            minValue: 0,
            maxValue: 100,
            vertical: true,
            disabled: true,
            increment: 1,
            allowDecimals: false,
            decimalPrecision: 0,
            forceDecimalPrecision: true,
            plugins: new Ext.slider.Tip({
                cls: 'dsClsDeviceSettingsFCUView_FanSpeedSlider_Tip',

                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleFanSpeedChanged.createDelegate(this)
            }
        });

        this.louverpositionslider = new Ext.slider.SingleSlider({
            cls: 'dsClsDeviceSettingsFCUView_LouverPositionSlider',

            height: 70,
            value: 0,
            minValue: 0,
            maxValue: 100,
            vertical: true,
            disabled: true,
            increment: 1,
            allowDecimals: false,
            decimalPrecision: 0,
            forceDecimalPrecision: true,
            plugins: new Ext.slider.Tip({
                cls: 'dsClsDeviceSettingsFCUView_LouverPositionSlider_Tip',

                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleLouverPositionChanged.createDelegate(this)
            }
        });

        var directionitems = [];

        var group;
        if (config.device != null) {
            group = dSS.util.getJokerGroup(config.device.get('groups'));
        }

        if (group === dSS.config.groups.RECIRCULATION) {
            directionitems =
            [
                [
                    0,
                    dSS._("no direction defined")
                ],
                [
                    1,
                    dSS._("down")
                ],
                [
                    2,
                    dSS._("up")
                ]
            ];
        } else { // ventilation
            directionitems =
            [
                [
                    0,
                    dSS._("supply and exhaust air")
                ],
                [
                    1,
                    dSS._("supply air")
                ],
                [
                    2,
                    dSS._("exhaust air")
                ]
            ];
        }

        this.airflowdirection = new Ext.form.ComboBox({
            cls: 'dsClsDeviceSettingsFCUView_AirflowDirectionCombo',

            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            disabled: true,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: directionitems
            }),
            valueField: 'id',
            displayField: 'displayText',
            valueNotFoundText: " ",
            value: 0
        });

        Ext.apply(this, {
            cls: 'dsClsDeviceSettingsFCUView',

            layout: 'absolute',
            border: false,
            padding: 5,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme'  ],
            items:
            [
                // ventilation part
                {
                    xtype: 'label',
                    ref: 'fanspeedlabel',
                    text: dSS._("Ventilation speed") + ':',
                    disabled: true,
                    x: 10,
                    y: 16
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_VentilationSpeedNumberField',

                    xtype: 'numberfield',
                    ref: 'fanspeedvalue',
                    disabled: true,
                    x: xPos - xPosOffset,
                    y: 12,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleFanSpeedChanged.createDelegate(this)
                    }
                },
                {
                    xtype:'label',
                    ref: 'fanspeedpercent',
                    disabled: true,
                    x: xPos - (xPosOffset - 56),
                    y: 16,
                    text: '%'
                },
                // this is a workaround for a bug when the slider thumb was
                // rendered at a wrong position; I could not figure out why
                // this is happening, but it is related to who is rendered
                // when/hidden rendering, so the workaround is to render only a
                // container here and attach the slider upon view activation
                {
                    xtype: 'container',
                    ref: 'fanspeedslidercontainer',
                    disabled: true,
                    x: xPos - (xPosOffset - 70),
                    y: 12,
                    height: 70
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_VentilationMaxButton',

                    xtype: 'button',
                    x: xPos - (xPosOffset - 102),
                    y: 12,
                    width: 130,
                    ref: 'ventmax',
                    disabled: true,
                    text: dSS._("Max"),
                    handler: this.handleFanSpeedMaxButton.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_VentilationMinButton',

                    xtype: 'button',
                    x: xPos - (xPosOffset - 102),
                    y: 60,
                    width: 130,
                    ref: 'ventmin',
                    disabled: true,
                    text: dSS._("Min"),
                    handler: this.handleFanSpeedMinButton.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_FCUAutomaticCheckbox',

                    xtype: 'checkbox',
                    ref: 'fcuautomatic',
                    disabled: true,
                    x: 50,
                    y: 35,
                    handler: this.handleAutomaticSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'automaticlabel',
                    text: dSS._("Automatic"),
                    disabled: true,
                    x: 70,
                    y: 40
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_FCURetainOutputCheckbox',

                    xtype: 'checkbox',
                    ref: 'fcufanretainoutput',
                    disabled: true,
                    x: 50,
                    y: 65,
                    hidden: (this.apartmentMode != true),
                    handler: this.handleFanRetainOutputSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'fcufanretainoutputlabel',
                    text: dSS._("Retain output"),
                    x: 70,
                    y: 70,
                    disabled: true,
                    hidden: (this.apartmentMode != true)
                },
                // airflow direction part
                {
                    xtype: 'label',
                    ref: 'directionlabel',
                    disabled: true,
                    text: dSS._("Airflow direction") + ':',
                    x: 10,
                    y: 108
                },
                {
                    xtype: 'panel',
                    x: xPos - xPosOffset,
                    y: 105,
                    width: 232,
                    layout: 'fit',
                    border: false,
                    items: [ this.airflowdirection ]
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_RetainAirflowDirectionOutputCheckbox',

                    xtype: 'checkbox',
                    ref: 'airflowdirectionretainoutput',
                    disabled: true,
                    x: 50,
                    y: 130,
                    hidden: (this.apartmentMode != true),
                    handler: this.handleAirflowDirectionRetainOutputSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    disabled: true,
                    ref: 'airflowdirectionretainoutputlabel',
                    text: dSS._("Retain output"),
                    x: 70,
                    y: 133,
                    hidden: (this.apartmentMode != true)
                },

                // louver part
                {
                    xtype: 'label',
                    ref: 'louverlabel',
                    disabled: true,
                    text: dSS._("Louver position") + ':',
                    x: 10,
                    y: 166
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_LouverPositionNumberField',

                    xtype: 'numberfield',
                    ref: 'louverpositionvalue',
                    disabled: true,
                    x: xPos - xPosOffset,
                    y: 162,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleLouverPositionChanged.createDelegate(this)
                    }
                },
                {
                    xtype:'label',
                    ref: 'louverpercent',
                    disabled: true,
                    x: xPos - (xPosOffset - 56),
                    y: 166,
                    text: '%'
                },
                {
                    xtype: 'container',
                    ref: 'louverpositionslidercontainer',
                    disabled: true,
                    x: xPos - (xPosOffset - 70),
                    y: 162,
                    height: 70
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_LouverPositionMaxButton',

                    xtype: 'button',
                    disabled: true,
                    x: xPos - (xPosOffset - 102),
                    y: 162,
                    width: 130,
                    ref: 'louvermax',
                    text: dSS._("Max"),
                    handler: this.handleLouverPositionMaxButton.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_LouverPositionMinButton',

                    xtype: 'button',
                    disabled: true,
                    x: xPos - (xPosOffset - 102),
                    y: 206,
                    width: 130,
                    ref: 'louvermin',
                    text: dSS._("Min"),
                    handler: this.handleLouverPositionMinButton.createDelegate(this)
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_FCUSwingCheckbox',

                    xtype: 'checkbox',
                    disabled: true,
                    ref: 'fcuswing',
                    x: 50,
                    y: 185,
                    handler: this.handleSwingSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'swinglabel',
                    disabled: true,
                    text: dSS._("Swing"),
                    x: 70,
                    y: 190
                },
                {
                    cls: 'dsClsDeviceSettingsFCUView_FCURetainLouverOutputCheckbox',

                    xtype: 'checkbox',
                    disabled: true,
                    ref: 'fculouverretainoutput',
                    x: 50,
                    y: 215,
                    hidden: (this.apartmentMode != true),
                    handler: this.handleLouverRetainOutputSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'fculouverretainoutputlabel',
                    text: dSS._("Retain output"),
                    disabled: true,
                    x: 70,
                    y: 220,
                    hidden: (this.apartmentMode != true)
                }
            ]
        });

        dSS.devices.DeviceSettingsFCUView.superclass.constructor.apply(this, arguments);

        this.fanspeedvalue.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var nv = parseFloat(field.getValue());
                var ov = parseFloat(this.fanspeedslider.getValue());
                if (nv !== ov) {
                    this.handleFanSpeedChanged(field, nv);
                }
            }
        },
        this);

        this.louverpositionvalue.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var nv = parseFloat(field.getValue());
                var ov = parseFloat(this.louverpositionslider.getValue());
                if (nv !== ov) {
                    this.handleLouverPositionChanged(field, nv);
                }
            }
        },
        this);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.on('activate', function() {
            this.fanspeedslidercontainer.add(this.fanspeedslider);
            this.louverpositionslidercontainer.add(this.louverpositionslider);
        },
        this);

        this.airflowdirection.on('select',
                this.handleAirflowDirectionChanged.createDelegate(this), this);
    },
    handleFanSpeedMaxButton: function() {
        this.fanspeedvalue.setValue(100);
        this.handleFanSpeedChanged(this.fanspeedvalue, 100);
    },
    handleFanSpeedMinButton: function() {
        this.fanspeedvalue.setValue(0);
        this.handleFanSpeedChanged(this.fanspeedvalue, 0);
    },
    handleLouverPositionMaxButton: function() {
        this.louverpositionvalue.setValue(100);
        this.handleLouverPositionChanged(this.louverpositionvalue, 100);
    },
    handleLouverPositionMinButton: function() {
        this.louverpositionvalue.setValue(0);
        this.handleLouverPositionChanged(this.louverpositionvalue, 0);
    },
    handleFanSpeedChanged: function(container, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.fanspeedvalue.setValue(newVal);
        this.fanspeedslider.setValue(newVal);

        if ((this.apartmentMode !== true) &&
            (this.previousFanSpeed != newVal)) {
            this.dirtyFanSpeed = true;
            this.setFanSpeedOnDevice(newVal);
            this.previousFanSpeed = newVal;
        }
    },
    setFanSpeedOnDevice: function(value) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
		channelvalues: 'airFlowIntensity=' + value,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set airflow intensity")
        });
    },
    handleAutomaticSelection: function(cb, checked) {
        if (checked) {
            this.disableFanSpeed();
        } else {
            this.enableFanSpeed();
        }
        if (this.apartmentMode !== true) {
            this.setAutomaticModeOnDevice('airFlowAuto', checked);
            this.dirtyAutomatic = true;
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleSwingSelection: function(cb, checked) {
        if (checked) {
            this.disableLouver();
        } else {
            this.enableLouver();
        }
        if (this.apartmentMode !== true) {
            this.setAutomaticModeOnDevice('airLouverAuto', checked);
            this.dirtySwing = true;
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleFanRetainOutputSelection: function(cb, checked) {
        if (this.apartmentMode !== true) {
            this.setRetainModeOnDevice('airFlowIntensity', checked);
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleLouverRetainOutputSelection: function(cb, checked) {
        if (this.apartmentMode !== true) {
            this.setRetainModeOnDevice('airLouverPosition', checked);
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleAirflowDirectionRetainOutputSelection: function(cb, checked) {
        if (this.apartmentMode !== true) {
            this.setRetainModeOnDevice('airFlowDirection', checked);
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleLouverPositionChanged: function(field, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }

        this.killRequests();
        this.louverpositionvalue.setValue(newVal);
        this.louverpositionslider.setValue(newVal);

        if ((this.apartmentMode !== true) &&
            (this.previousLouverPosition !== newVal)) {
            this.dirtyLouverPosition = true;
            this.setLouverPositionOnDevice(newVal);
            this.previousLouverPosition = newVal;
        }
    },
    handleAirflowDirectionChanged: function(combo, record) {
        if (this.apartmentMode !== true) {
            this.setAirflowDirectionOnDevice(record.get('id'));
            this.dirtyDirection = true;
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    setLouverPositionOnDevice: function(value) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
		channelvalues: 'airLouverPosition=' + value,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set louver position")
        });
    },
    setAutomaticModeOnDevice: function(channel, checked) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: channel + '=' + (checked ? "1" : "0"),
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set selected mode on device")
        });
    },
    setRetainModeOnDevice: function(channel, checked) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelScene',
            scope: this,
            params:
            {
                channels: JSON.stringify(parameter),
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set dontcare")
        });
    },
    setAirflowDirectionOnDevice: function(direction) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: 'airFlowDirection' + '=' + direction,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set selected mode on device")
        });
    },
    enableFanSpeed: function() {
        this.fanspeedvalue.enable();
        this.fanspeedslider.enable();
        this.fanspeedlabel.enable();
        this.fanspeedpercent.enable();
        this.ventmax.enable();
        this.ventmin.enable();
    },
    disableFanSpeed: function() {
        this.fanspeedvalue.disable();
        this.fanspeedslider.disable();
        this.fanspeedlabel.disable();
        this.fanspeedpercent.disable();
        this.ventmax.disable();
        this.ventmin.disable();
    },
    enableLouver: function() {
        this.louverpositionvalue.enable();
        this.louverpositionslider.enable();
        this.louverlabel.enable();
        this.louverpercent.enable();
        this.louvermax.enable();
        this.louvermin.enable();
    },
    disableLouver: function() {
        this.louverpositionvalue.disable();
        this.louverpositionslider.disable();
        this.louverlabel.disable();
        this.louverpercent.disable();
        this.louvermax.disable();
        this.louvermin.disable();
    },
    enableDirection: function() {
        this.airflowdirection.enable();
        this.directionlabel.enable();
    },
    enableAuto: function() {
        this.fcuautomatic.enable();
        this.automaticlabel.enable();
    },
    enableSwing: function() {
        this.fcuswing.enable();
        this.swinglabel.enable();
    },
    enableFanSpeedRetain: function() {
        this.fcufanretainoutput.enable();
        this.fcufanretainoutputlabel.enable();
    },
    enableDirectionRetain: function() {
        this.airflowdirectionretainoutput.enable();
        this.airflowdirectionretainoutputlabel.enable();
    },
    enableLouverRetain: function() {
        this.fculouverretainoutput.enable();
        this.fculouverretainoutputlabel.enable();
    },
    loadData: function(reqdSUID) {
        this.killRequests();

        this.fireEvent('loading', true);

        this.fireEvent('enableme');

        this.dSUID = reqdSUID;

        this.dirtyFanSpeed = false;
        this.dirtyLouverPosition = false;
        this.dirtyAutomatic = false;
        this.dirtySwing = false;
        this.dirtyDirection = false;

        this.AjaxRequestFCUOutputChannels = Ext.Ajax.request({
            url: '/json/device/getOutputChannelValue2',
            method: 'GET',
            params:
            {
                dsuid: reqdSUID,
                channels: 'airFlowIntensity;airFlowAuto;airLouverPosition;airLouverAuto;airFlowDirection'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestFCUOutputChannels = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current output channel " +
                                  "values") + ': ' + jsonData.message);
                        if (this.dirtyFanSpeed != true) {
                            this.fanspeedvalue.markInvalid();
                        }
                        if (this.dirtyairLouverPosition != true) {
                            this.louverpositionvalue.markInvalid();
                        }
                        return;
                    }

                    var airflow;
                    var louver;
                    var direction;
                    var swing = false;
                    var automatic = false;

                    if (jsonData.result.channels.airFlowIntensity !== undefined) {
                        var chan = jsonData.result.channels.airFlowIntensity;
                        airflow = Math.round(chan.value);
                        if (jsonData.result.channels.airFlowAuto !== undefined) {
                            var chan = jsonData.result.channels.airFlowAuto;
                            if (chan.value !== undefined) {
                                automatic = chan.value > 0;
                            }
                            this.enableAuto();
                        }
                        if (automatic != true) {
                            this.enableFanSpeed();
                        }
                    }

                    if (jsonData.result.channels.airLouverPosition !== undefined) {
                        var chan = jsonData.result.channels.airLouverPosition;
                        louver = Math.round(chan.value);
                        if (jsonData.result.channels.airLouverAuto !== undefined) {
                            var chan = jsonData.result.channels.airLouverAuto;
                            if (chan.value !== undefined) {
                                swing = chan.value > 0;
                            }
                            this.enableSwing();
                        }
                        if (swing != true) {
                            this.enableLouver();
                        }
                    }
                    if (jsonData.result.channels.airFlowDirection !== undefined) {
                        var chan = jsonData.result.channels.airFlowDirection;
                        if (chan.value !== undefined) {
                            direction = chan.value;
                        }
                        this.enableDirection();
                    }

                    if (this.dirtyFanSpeed != true) {
                        if (!isNaN(airflow)) {
                            this.previousairflowintensity = airflow;
                            this.fanspeedvalue.setValue(airflow);
                            this.fanspeedslider.setValue(airflow);
                        } else {
                            this.fanspeedvalue.markInvalid();
                        }
                    }

                    if (this.dirtyAutomatic != true) {
                        this.previousautomatic = automatic;
                        this.fcuautomatic.setValue(automatic);
                    }

                    if (this.dirtyLouverPosition != true) {
                        if (!isNaN(louver)) {
                            this.previousLouverPosition = louver;
                            this.louverpositionvalue.setValue(louver);
                            this.louverpositionslider.setValue(louver);
                        } else {
                            this.louverpositionvalue.markInvalid();
                        }
                    }

                    if (this.dirtySwing != true) {
                        this.prevousswing = swing;
                        this.fcuswing.setValue(swing);
                    }

                    if (this.dirtyDirection != true) {
                        if (!isNaN(direction)) {
                            this.previousdirection = direction;
                            this.airflowdirection.setValue(direction);
                        }
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestFCUOutputChannels = undefined;
                if ((this.closed === true) ||
                    ((this.dirtyFanSpeed === true) &&
                     (this.dirtyLouverPosition === true))) {
                    return;
                }

                this.fireEvent('enableme');
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            }
        });
    },
    killRequests: function() {
        if (this.AjaxRequestFCUOutputChannels !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestFCUOutputChannels);
            this.AjaxRequestFCUOutputChannels = undefined;
        }
    },
    clearData: function() {
        this.killRequests();
        this.fanspeedvalue.setValue(0);
        this.fanspeedslider.setValue(0);
        this.louverpositionvalue.setValue(0);
        this.louverpositionslider.setValue(0);
        this.dirtyFanSpeed = false;
        this.dirtyLouverPos = false;
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }

        if (this.apartmentMode !== true) {
            return;
        }

        this.killRequests();

        var parameter = {};

        if (this.airflowdirection.disabled != true) {
            parameter["airFlowDirection"] = {};
            parameter["airFlowDirection"]["value"] = this.airflowdirection.getValue();
            parameter["airFlowDirection"]["dontCare"] = this.airflowdirectionretainoutput.getValue();
        }

        if (this.fcuautomatic.disabled != true) {
            var fs = this.fanspeedvalue.getValue();
            parameter["airFlowAuto"] = {};
            parameter["airFlowAuto"]["value"] = this.fcuautomatic.getValue() ? 1 : 0;
            parameter["airFlowIntensity"] = {};
            if (this.fcuautomatic.getValue() != true) {
                parameter["airFlowIntensity"]["value"] = fs;
            }
            parameter["airFlowIntensity"]["dontCare"] = this.fcufanretainoutput.getValue();
        }

        if (this.fcuswing.disabled != true) {
            var lp = this.louverpositionvalue.getValue();
            parameter["airLouverAuto"] = {};
            parameter["airLouverAuto"]["value"] = this.fcuswing.getValue() ? 1 : 0;
            parameter["airLouverPosition"] = {};
            if (this.fcuswing.getValue() != true) {
                parameter["airLouverPosition"]["value"] = lp;
            }
            parameter["airLouverPosition"]["dontCare"] = this.fculouverretainoutput.getValue();
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelSceneValue2',
            scope: this,
            params:
            {
                channels: JSON.stringify(parameter),
                dsuid: this.dSUID,
                sceneNumber: scene
            },
            failureMsg: dSS._("Could not set louver position")
        });
        this.resetDefaults();
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }

        this.previousairflowintensity = this.fanspeedvalue.getValue();
        this.previousLouverPosition = this.louverpositionvalue.getValue();
        this.previousautomatic = this.fcuautomatic.getValue();
        this.previousswing = this.fcuswing.getValue();
        this.previousdirection = this.airflowdirection.getValue();
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }

        this.fanspeedvalue.setValue(this.previousairflowintensity);
        this.fanspeedslider.setValue(this.previousairflowintensity);

        this.louverpositionvalue.setValue(this.previousLouverPosition);
        this.louverpositionslider.setValue(this.previousLouverPosition);

        this.fcuautomatic.setValue(this.previousautomatic);
        this.fcuswing.setValue(this.previousswing);
        this.airflowdirection.setValue(this.previousdirection);
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }

        var parameter = {};
        if (this.fanspeedvalue.disabled != true) {
            if ((this.fcuautomatic.disabled != true) &&
                (this.fcuautomatic.getValue()) == true) {
                parameter["airFlowAuto"] = true;
            } else {
                var fs = this.fanspeedvalue.getValue();
                parameter["airFlowIntensity"] = fs;
                if (this.fcuautomatic.disabled != true) {
                    parameter["airFlowAuto"] = false;
                }
            }
        }

        if (this.louverpositionvalue.disabled != true) {
            if ((this.fcuswing.disabled != true) &&
                (this.fcuswing.getValue() == true)) {
                parameter["airLouverAuto"] = true;
            } else {
                var lp = this.louverpositionvalue.getValue();
                parameter["airLouverPosition"] = lp;
                if (this.fcuswing.disabled != true) {
                    parameter["airLouverAuto"] = false;
                }
            }
        }

        if (this.airflowdirection.disabled != true) {
            parameter["airFlowDirection"] = this.airflowdirection.getValue();
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue2',
            scope: this,
            params:
            {
                channels: JSON.stringify(parameter),
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set selected mode on device")
        });
    },
    setLouverPosition: function(position) {
        this.enableLouver();
        if (isNaN(position)) {
            return;
        }

        this.louverpositionvalue.setValue(position);
        this.louverpositionslider.setValue(position);
    },
    setAirflowIntensity: function(speed) {
        this.enableFanSpeed();

        if (isNaN(speed)) {
            return;
        }
        this.fanspeedvalue.setValue(speed);
        this.fanspeedslider.setValue(speed);
    },
    setFanAutomatic: function(automatic) {
        this.enableAuto();
        this.fcuautomatic.setValue(automatic);
    },
    setLouverSwing: function(swing) {
        this.enableSwing();
        this.fcuswing.setValue(swing);
    },
    setFanRetainOutput: function(retain) {
        this.enableFanSpeedRetain();
        this.fcufanretainoutput.setValue(retain);
    },
    setLouverRetainOutput: function(retain) {
        this.enableLouverRetain();
        this.fculouverretainoutput.setValue(retain);
    },
    setAirFlowDirection: function(direction) {
        this.enableDirection();
        this.airflowdirection.setValue(direction);
    },
    setAirFlowDirectionRetainOutput: function(retain) {
        this.enableDirectionRetain();
        this.airflowdirectionretainoutput.setValue(retain);
    }
});
