//= require <dSS/devices/device-settings/device-type-panels/color-view/DeviceSettingsColorTemperatureViewApartmentMode>
//= require <dSS/devices/device-settings/device-type-panels/color-view/DeviceSettingsColorViewApartmentMode>

Ext.namespace("dSS", "dSS.devices");

// *COLOR
dSS.devices.DeviceSettingsHSVViewApartmentMode = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.dSUID = config.dSUID;

        this.defaultColor = {
            colorHSVPicker: { h: 0, s: 0, v: 0 },
            colorTemperaturePicker: { brightness: 0, temperature: 3500 }
        };

        var me = this;

        var viewConfig = {
            x: 0,
            y: 0,
            xPos: 450,
            xPosOffset: 280,
            height: 330,
            dSUID: this.dSUID,
            hidden: true
        };
        var colorViewConfig = Object.assign({}, viewConfig, {
            ref: 'colorHSVPicker',
            toggleView: function () {
                me.colorTemperaturePicker.show();
                me.colorHSVPicker.hide();
                me.loadData(me.dSUID, me.record);
            }
        });
        var temperatureViewConfig = Object.assign({}, viewConfig, {
            ref: 'colorTemperaturePicker',
            toggleView: function () {
                me.colorHSVPicker.show();
                me.colorTemperaturePicker.hide();
                me.loadData(me.dSUID, me.record);
            }
        });
        var panelItems = [
                new dSS.devices.DeviceSettingsColorViewApartmentMode(colorViewConfig),
                new dSS.devices.DeviceSettingsColorTemperatureViewApartmentMode(temperatureViewConfig)
            ];
        Ext.apply(this, {
            cls: 'dsClsDeviceSettingsHSVViewApartmentMode',

            layout: 'absolute',
            padding: 5,
            border: false,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged', 'enableme' ],
            items: panelItems
        });

        dSS.devices.DeviceSettingsHSVViewApartmentMode.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.closed = true;
        },
        this);
    },
    visiblePicker: function () {
        if (!this.colorHSVPicker.hidden) {
            return this.colorHSVPicker;
        } else if (!this.colorTemperaturePicker.hidden) {
            return this.colorTemperaturePicker;
        }
        return {};
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }
        this.visiblePicker().doPreview();
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }
        this.visiblePicker().saveSceneValues(scene);
        this.resetDefaults();
    },
    clearData: function() {
        this.colorHSVPicker.setColor(this.defaultColor.colorHSVPicker);
        this.colorTemperaturePicker.setColor(this.defaultColor.colorTemperaturePicker);
    },
    killRequests: function() {},    // for sake of the same interface as DeviceSettingsHSVView
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }
        this.visiblePicker().resetDefaults();
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }
        this.visiblePicker().resetValues();
    },
    loadData: function(reqdSUID, record) {
        if (this.colorHSVPicker.hidden && this.colorTemperaturePicker.hidden || this.dSUID !== reqdSUID) {
            if (record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'hue' || channel.channelId === 'saturation';
            })) {
                this.colorHSVPicker.show();
                this.colorTemperaturePicker.hide();
            } else if (record.json.outputChannels.find(function (channel) {
                return channel.channelId === 'colortemp';
            })) {
                this.colorTemperaturePicker.show();
                this.colorHSVPicker.hide();
            }
        }
        this.record = record;
        this.dSUID = reqdSUID;

        this.visiblePicker().loadData(reqdSUID, record.json.outputChannels);
    },
    setColor: function(color) {
        // this.previousColor = color;
        if (color.hue != null && color.saturation != null) {
            this.colorHSVPicker.setColor({ h: color.hue, s: color.saturation, v: color.brightness });
        }
        if (color.colortemp != null) {
            this.colorTemperaturePicker.setColor({ brightness: color.brightness, temperature: color.colortemp });
        }
        if (color.colortemp != null && color.hue != null && color.saturation != null) {
            this.colorHSVPicker.switchToTheOtherColorPicker.show();
            this.colorTemperaturePicker.switchToTheOtherColorPicker.show();
        } else {
            this.colorHSVPicker.switchToTheOtherColorPicker.hide();
            this.colorTemperaturePicker.switchToTheOtherColorPicker.hide();
        }
    }
});
