Ext.namespace("dSS", "dSS.devices", "dSS.gthelper");

dSS.devices.DeviceSettingsVdcConfigLinkView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.dSUID = config.dSUID;
        this.deviceRecord = config.device;

        Ext.apply(this, {
            cls: 'dsClsDeviceSettingsVdcConfigLinkView',

            padding: 5,
            layout: 'absolute',
            border: false,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged', 
                            'enableme' ],
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    padding: 5,
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Clicking the button will open the vDC " +
                                        "configuration page in a new tab or window."),
                            x: 0,
                            y: 30
                        }
                    ]
                },
                {
                    cls: 'dsClsDeviceSettingsVdcConfigLinkView_OpenVDCConfigurationButton',

                    xtype: 'button',
                    x: ((dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 34) / 2) - (260 / 2),
                    y: (config.initialHeight / 2) - (25 / 2),
                    minWidth: 260,
                    height: 25,
                    handler: this.openVdcConfigPage.createDelegate(this),
                text: dSS._("Open vDC configuration")
                }
            ]
        });

        dSS.devices.DeviceSettingsVdcConfigLinkView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);
    },
    openVdcConfigPage: function(record) {
        if (this.deviceRecord) {
            dSS.util.openVdcLink(this.deviceRecord, 'standardaction');
        }
    },
    killRequests: function() {
        if (this.AjaxRequestConfigLink !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestConfigLink);
            this.AjaxRequestConfigLink = undefined;
        }
    },
    loadData: function(reqdSUID, record, shownow) {
        this.killRequests();

        this.dSUID = reqdSUID;

        this.deviceRecord = record;

        this.fireEvent('loading', true);

        this.AjaxRequestConfigLink = Ext.Ajax.request({
            url: '/json/property/query',
            method: 'GET',
            params:
            {
                query: '/apartment/zones/zone0/devices/' + this.dSUID +
                       '/productInfo(ConfigLink,State,ProductState)'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestConfigLink = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if ((jsonData.ok !== true) ||
                        (jsonData.result == undefined) ||
                        (jsonData.result.productInfo == undefined) ||
                        (jsonData.result.productInfo.length == 0) ||
                        (jsonData.result.productInfo[0].State !== "Valid") ||
                        (jsonData.result.productInfo[0].ProductState !== "Valid")) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("The configuration page location is " +
                                  "unknown and cannot be loaded."));
                        return;
                    } else {
                        // this dialog is an exception in relation to
                        // the enableme event; the user can not change anything
                        // here, only clicking the button is allowed, so
                        // the dialog will stay disabled while loading
                        this.fireEvent('enableme');
                        if (shownow === true) {
                            this.openVdcConfigPage();
                        }
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("The configuration page location is " +
                              "unknown and cannot be loaded."));
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestConfigLink = undefined;
                if (this.closed === true) {
                    return;
                }
            }
        });
    }
});