//= require <dSS/devices/device-settings/device-type-panels/color-view/Components>
//= require <dSS/devices/device-settings/device-type-panels/color-view/Helpers>

Ext.namespace("dSS", "dSS.devices");

// *TEMPERATURE
dSS.devices.DeviceSettingsColorTemperatureViewApartmentMode = Ext.extend(Ext.Panel, {

    constructor: function(config) {
        var xPos = config.xPos;
        var xPosOffset = config.xPosOffset;

        this.brightness = {
            min: config.minBrightness || 0,
            max: config.maxBrightness || 100,    // in %
            previous: null,
            current: config.minBrightness || 0
        };

        var defaults = dSS.devices.helpers.colorTemperatureDefaults;
        this.temperature = { // actual channel range is 100-1000
            min: config.minTemperature || defaults.min,
            max: config.maxTemperature || defaults.max,
            whiteAmbiance: defaults.philipsHueWhiteAmbiance,
            whiteAndColorAmbiance: defaults.philipsHueWhiteAndColorAmbiance,
            previous: null,
            current: config.minTemperature || defaults.min
        };

        this.closed = false;

        this.dSUID = config.dSUID;

        var me = this;
        var items = [
            new dSS.devices.components.BrightnessLabel(),
            new dSS.devices.components.BrightnessField({    // this.brightnessField
                x: xPos - xPosOffset,
                value: this.brightness.current,
                minValue: this.brightness.min,
                maxValue: this.brightness.max,
                listeners: {
                    change: this.setBrightness.createDelegate(this)
                }
            }),
            new dSS.devices.components.BrightnessUnitLabel({
                x: xPos - (xPosOffset - 55)
            }),
            new dSS.devices.components.BrightnessSlider({   // this.brightnessSlider
                x: xPos - (xPosOffset - 70),
                value: this.brightness.current,
                minValue: this.brightness.min,
                maxValue: this.brightness.max,
                listeners: {
                    changeComplete: this.setBrightness.createDelegate(this)
                }
            }),
            new dSS.devices.components.MaxButton({
                x: xPos - (xPosOffset - 102),
                handler: this.setMaxBrightness.createDelegate(this)
            }),
            new dSS.devices.components.IncButton({
                x: xPos - (xPosOffset - 102),
                handler: this.incBrightness.createDelegate(this)
            }),
            new dSS.devices.components.DecButton({
                x: xPos - (xPosOffset - 102),
                handler: this.decBrightness.createDelegate(this)
            }),
            new dSS.devices.components.MinButton({
                x: xPos - (xPosOffset - 102),
                handler: this.setMinBrightness.createDelegate(this)
            }),
            new dSS.devices.components.TemperatureHint(),
            new dSS.devices.components.TemperatureLabel(),
            new dSS.devices.components.TemperatureField({   // this.temperatureField
                x: xPos - xPosOffset,
                value: this.temperature.current,
                minValue: this.temperature.min,
                maxValue: this.temperature.max,
                listeners: {
                    change: this.setTemperature.createDelegate(this)
                }
            }),
            new dSS.devices.components.TemperatureSlider({  // this.temperatureSLider
                x: xPos - (xPosOffset - 74),
                value: this.temperature.current,
                minValue: this.temperature.min,
                maxValue: this.temperature.max,
                listeners: {
                    changeComplete: this.setTemperature.createDelegate(this)
                }
            }),
            new dSS.devices.components.KelvinMinLabel({
                x: xPos - (xPosOffset - 74),
                text: this.temperature.min
            }),
            new dSS.devices.components.KelvinLabel({
                x: xPos - (xPosOffset - 140)
            }),
            new dSS.devices.components.KelvinMaxLabel({
                x: xPos - (xPosOffset - 207),
                text: this.temperature.max
            }),
            new dSS.devices.components.SwitchToColorModeButton({
                listeners: {
                    render: function (me) {
                        me.getEl().on('click', config.toggleView, me);
                    }
                },
                hidden: true
            })
        ];
        Ext.apply(this, {
            cls: 'dsClsDeviceSettingsColorTemperatureViewApartmentMode',

            layout: 'absolute',
            border: false,
            padding: 5,
            bubbleEvents: [ 'formchanged', 'enableme' ],
            items: items,
            listeners: {
                show: function () {
                    // hack for hidden slider behaviour:
                    // slider handle positions need to be updated to match value, as setColor run when sliders
                    // were not visible yet and setting value on them didn't reposition handles:
                    me.brightnessSlider.syncThumb();
                    me.temperatureSlider.syncThumb();
                }
            }
        });

        dSS.devices.DeviceSettingsColorTemperatureViewApartmentMode.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.closed = true;
        }, this);

        this.brightnessField.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var newValue = parseFloat(field.getValue());
                var oldValue = parseFloat(this.brightnessSlider.getValue());
                if (newValue !== oldValue) {
                    this.setBrightness(field, newValue);
                }
            }
        }, this);

        this.temperatureField.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var newValue = parseFloat(field.getValue());
                var oldValue = parseFloat(this.temperatureSlider.getValue());
                if (newValue !== oldValue) {
                    this.setTemperature(field, newValue);
                }
            }
        }, this);
    },
    updateBrightnessInForm: function (newValue, notifyAncestors) {
        if (notifyAncestors) {
            this.fireEvent('formchanged');
        }
        this.brightnessSlider.setValue(newValue);
        this.brightnessField.setValue(newValue);
        this.brightness.current = newValue;
    },
    updateTemperatureInForm: function (newValue, notifyAncestors) {
        if (notifyAncestors) {
            this.fireEvent('formchanged');
        }
        this.temperatureSlider.setValue(newValue);
        this.temperatureField.setValue(newValue);
        this.temperature.current = newValue;
    },
    setBrightness: function(container, newValue) {
        if (newValue === '') {
            this.brightnessField.setValue(this.brightnessSlider.getValue());
            return;
        }
        var value = dSS.devices.helpers.ensureValueWithinRange({
            value: newValue,
            min: this.brightness.min,
            max: this.brightness.max
        });
        this.updateBrightnessInForm(value, true);
    },
    setTemperature: function(container, newValue) {
        if (newValue === '') {
            this.temperatureField.setValue(this.temperatureSlider.getValue());
            return;
        }
        var value = dSS.devices.helpers.ensureValueWithinRange({
            value: newValue,
            min: this.temperature.min,
            max: this.temperature.max
        });
        this.updateTemperatureInForm(value, true);
    },
    setChannelValueOnDevice: function(channelId, value) {
        var channelValue = channelId === 'colortemp'
            ? dSS.devices.helpers.temperatureToChannelValue(value)
            : value;
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: channelId + '=' + channelValue,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set device color")
        });
    },
    setMaxBrightness: function() {
        this.updateBrightnessInForm(this.brightness.max, true);
    },
    setMinBrightness: function() {
        this.updateBrightnessInForm(this.brightness.min, true);
    },
    incBrightness: function() {
        this.setBrightness(this.brightnessSlider, dSS.devices.helpers.ensureValueWithinRange({
            value: this.brightness.current + 5,
            max: this.brightness.max
        }));
    },
    decBrightness: function() {
        this.setBrightness(this.brightnessSlider, dSS.devices.helpers.ensureValueWithinRange({
            value: this.brightness.current - 5,
            min: this.brightness.min
        }));
    },
    updateMinMaxTemperature (lightbulbType) {
        if (this.temperature[lightbulbType]) {
            this.temperature.min = this.temperature[lightbulbType].min;
            this.temperature.max = this.temperature[lightbulbType].max;
            this.temperatureField.setMinValue(this.temperature.min);
            this.temperatureField.setMaxValue(this.temperature.max);
            this.temperatureSlider.setMinValue(this.temperature.min);
            this.temperatureSlider.setMaxValue(this.temperature.max);
            this.kelvinMinLabel.setText(this.temperature.min);
            this.kelvinMaxLabel.setText(this.temperature.max);
        }
    },
    clearData: function() {
        this.brightnessSlider.setValue(this.brightness.min);
        this.brightnessField.setValue(this.brightness.min);
        this.brightness.current = this.brightness.min;
        this.brightness.previous = null;
        this.temperatureSlider.setValue(this.temperature.min);
        this.temperatureField.setValue(this.temperature.min);
        this.temperature.current = this.temperature.min;
        this.temperature.previous = null;
    },
    loadData: function(dSUID, deviceChannels) {
        this.dSUID = dSUID;
        this.fireEvent('enableme');

        if (deviceChannels.find(function (channel) {
            return channel.channelId === 'hue' || channel.channelId === 'saturation';
        })) {
            this.switchToTheOtherColorPicker.show();
            this.updateMinMaxTemperature('whiteAndColorAmbiance');
        } else {
            this.switchToTheOtherColorPicker.hide();
            this.updateMinMaxTemperature('whiteAmbiance');
        }
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }
        var brightnessFieldValue = this.brightnessField.getValue();
        var temperatureFieldValue = this.temperatureField.getValue();
        if (brightnessFieldValue === '' || temperatureFieldValue === '') {
            return;
        }

        var colorWithinRange = {
            brightness: dSS.devices.helpers.ensureValueWithinRange({
                value: brightnessFieldValue,
                min: this.brightness.min,
                max: this.brightness.max
            }),
            temperature: dSS.devices.helpers.ensureValueWithinRange({
                value: temperatureFieldValue,
                min: this.temperature.min,
                max: this.temperature.max
            })
        };
        if (brightnessFieldValue !== colorWithinRange.brightness) {
            this.updateBrightnessInForm(colorWithinRange.brightness);
        }
        if (temperatureFieldValue !== colorWithinRange.temperature) {
            this.updateTemperatureInForm(colorWithinRange.temperature);
        }
        colorWithinRange.temperature = dSS.devices.helpers.temperatureToChannelValue(colorWithinRange.temperature);
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelSceneValue2',
            scope: this,
            params: {
                channels: JSON.stringify({
                    colortemp: { value: colorWithinRange.temperature, dontCare: false },
                    brightness: { value: colorWithinRange.brightness, dontCare: false }
                }),
                dsuid: this.dSUID,
                sceneNumber: scene
            },
            failureMsg: dSS._("Could not set device color")
        });
        this.resetDefaults();
    },
    resetDefaults: function() {     // store current values to be able to return to them if 'resetValues' happens
        if (!this.isVisible()) {
            return;
        }
        this.brightness.previous = this.brightnessField.getValue();
        this.temperature.previous = this.temperatureField.getValue();
    },
    resetValues: function() {       // user clicked 'Reset' - return to previous values
        if (!this.isVisible()) {
            return;
        }
        this.brightnessField.setValue(this.brightness.previous);
        this.brightnessSlider.setValue(this.brightness.previous);
        this.temperatureField.setValue(this.temperature.previous);
        this.temperatureSlider.setValue(this.temperature.previous);
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }
        var brightness = Math.min(Math.max(this.brightnessField.getValue(), this.brightness.min), this.brightness.max);
        var temperature = Math.min(Math.max(this.temperatureField.getValue(), this.temperature.min), this.temperature.max);
        this.setChannelValueOnDevice('brightness', brightness);
        this.setChannelValueOnDevice('colortemp', temperature);
    },
    setColor: function (newColor) {
        var newTemperature = newColor.temperature;
        if (newTemperature < this.temperatureField.minValue) {
            newTemperature = this.temperatureField.minValue;
        } else if (newTemperature > this.temperatureField.maxValue) {
            newTemperature = this.temperatureField.maxValue;
        }
        this.brightness.previous = newColor.brightness;
        this.temperature.previous = newTemperature;
        this.updateBrightnessInForm(newColor.brightness);
        this.updateTemperatureInForm(newTemperature);
    }
});
