//= require <dSS/devices/device-settings/device-type-panels/color-view/Components>
//= require <dSS/devices/device-settings/device-type-panels/color-view/Helpers>

Ext.namespace("dSS", "dSS.devices");

// *COLOR
dSS.devices.DeviceSettingsColorView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;
        this.AjaxRequestOutputChannels = undefined;

        this.previousColor = { h: -1, s: -1, v: -1 };
        this.defaultColor = { h: 0, s: 0, v: 0 };

        this.dirtyColor = false;

        this.dSUID = config.dSUID;

        var panelItems = [
            {
                xtype: 'jpickerpanel',
                ref: 'colorPicker',
                border: false,
                localization: dSS.devices.helpers.localizePicker,
                hidden: false
            },
            new dSS.devices.components.SwitchToTemperatureModeButton({
                listeners: {
                    render: function (me) {
                        me.getEl().on('click', config.toggleView, me);
                    } 
                },
                hidden: true
            })
        ];
        Ext.apply(this, {
            cls: 'DeviceSettingsColorView',

            layout: 'absolute',
            padding: 5,
            border: false,
            bubbleEvents: [ 'loading', 'valuechanged', 'enableme' ],
            items: panelItems
        });

        dSS.devices.DeviceSettingsColorView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.colorPicker.on('jpickercolorchanged', this.handleColorChanged.createDelegate(this), this);

    },
    setColorOnDevice: function(color) {
        // hue usually goes from 0 to 360°
        if (color.h > 360) {
            color.h = 360;
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: 'hue=' + color.h +
                               ';saturation=' + color.s +
                               ';brightness=' + color.v,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set device color")
        });
    },
    handleColorChanged: function(panel, color) {
        this.fireEvent('valuechanged', this.dSUID);
        if (this.colorPicker.equalHSVColors(this.previousColor, color)) {
            return;
        }
        this.setColorOnDevice(color);
    },
    clearData: function() {
        this.killRequests();
        this.colorPicker.setColor(this.defaultColor);
    },
    killRequests: function() {
        this.dirtyColor = true;
        this.fireEvent('loading', false);
        if (this.AjaxRequestOutputChannels !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestOutputChannels);
            this.AjaxRequestOutputChannels = undefined;
        }
    },
    loadData: function(reqdSUID, deviceChannels) {
        this.killRequests();

        this.dSUID = reqdSUID;

        this.dirtyColor = false;
        this.fireEvent('loading', true);
        this.fireEvent('enableme');

        var DEFAULT_COLOR_CHANNELS = 'brightness;hue;saturation';

        this.AjaxRequestOutputChannels = Ext.Ajax.request({
            url: '/json/device/getOutputChannelValue',
            method: 'GET',
            params:
            {
                dsuid: reqdSUID,
                channels: deviceChannels.map(function (channel) {
                    return channel.channelId;
                }).join(';') || DEFAULT_COLOR_CHANNELS
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOutputChannels = undefined;
                if ((this.closed === true) ||
                    (this.dirtyColor === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }
                        
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current output channel " +
                                  "values") + ': ' + jsonData.message);
                        return;
                    }

                    var color = jsonData.result.channels && jsonData.result.channels.reduce(function (allChannels, channel) {
                        allChannels[channel.channelId] = Math.round(channel.value);
                        return allChannels;
                    }, {});

                    if (color &&
                        color.hue != null && !isNaN(color.hue) &&
                        color.saturation != null && !isNaN(color.saturation) &&
                        color.brightness != null && !isNaN(color.brightness)) {
                        this.previousColor.h = color.hue;
                        this.previousColor.s = color.saturation;
                        this.previousColor.v = color.brightness;

                        this.colorPicker.setColor(this.previousColor);
                    }

                    if (deviceChannels.find(function (channel) {
                        return channel.channelId === 'colortemp';
                    })) {
                        this.switchToTheOtherColorPicker.show();
                    } else {
                        this.switchToTheOtherColorPicker.hide();
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOutputChannels = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            }
        });
    },
    setColor: function(color) {
        this.previousColor = color;
        this.colorPicker.setColor(color);
    }
});