/* globals Ext, dSS */

dSS.devprops.CreatePanelForDeviceProperties = function (record, featureConfig, scope) {
    var dSUID = record.get('dsuid');
    var groups = record.get('groups');
    var devInfo = dSS.util.parseProductID(record.get('productID'));
    var type = devInfo[0];
    var deviceNumber = devInfo[1];
    var functionId = record.get('functionID');
    var cls = dSS.util.parseFunctionID(functionId);
    var stdForms = [];
    var advancedForms = [];
    var loaders = [];
    var blinkForms = [];
    var transitionForms = [];
    var matrix;
    var width = 400;

    var deviceIsOfGrKl300Type = dSS.util.isGrKl300TypeDevice(record);
    var dsid = record.get('dsid');

    try {
        matrix = dSS.devprops.GetVisibleProperties(record);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
        var addPBOutModeLoader = false;
        var addLocalPrioLoader = false;
        var color = cls[0];

        if (matrix.enabletypehack || deviceIsOfGrKl300Type && deviceIsOfGrKl300Type.sensor) {
            stdForms.push({
                xtype: 'dssdevinputtypeform',
                device: record,
                matrix: matrix,
                color: color
            });
            addPBOutModeLoader = true;
        }

        if (matrix.jokerconfig && color === dSS.config.deviceClass.SW) {
            color = dSS.util.getJokerGroup(groups);
            // assume it's not yet configured
            if (color == null) {
                color = dSS.config.deviceClass.SW;
            }
            stdForms.push({
                xtype: 'dssdevjokerconfigform',
                dSUID: dSUID,
                device: record,
                product: type,
                group: color,
                matrix: matrix,
                hidden: matrix.enabletypehack
            });
        }

        if (matrix.heatinggroup) {
            // in reality the length of this array should always be 1
            var group = groups.length
                ? groups[0]
                : dSS.config.groups.HEATING;
            stdForms.push({
                xtype: 'dssdevheatingconfigform',
                dSUID: dSUID,
                device: record,
                product: type,
                group: group
            });
        }

        if (matrix.valvetype) {
            stdForms.push({
                xtype: 'dssdevvalvetypeform',
                device: record,
                matrix: matrix
            });

            loaders.push(new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.VALVE_TYPE,
                url: '/json/device/getValveType',
                params: {
                    dsuid: dSUID
                }
            }));
        }

        if (matrix.optypeconfig) {
            stdForms.push({
                xtype: 'dssdevoptypeform',
                dSUID: dSUID
            });
            addPBOutModeLoader = true;
        }

        if (deviceIsOfGrKl300Type && !deviceIsOfGrKl300Type.sensor) {
            stdForms.push(
                new dSS.devprops.MultiButtonConfigForm({    // Button Function - 1 way, 2-way...
                    device: record,
                    group: color,
                    matrix: matrix
                })
            );
        }

        var override = color;
        if (matrix.heatinggroup) {
            // in reality the length of this array should always be 1
            if (groups.length) {
                override = groups[0];
            }
        }

        if (
            matrix.pushbutton &&
            (!deviceIsOfGrKl300Type || !deviceIsOfGrKl300Type.sensor) &&
            record.get('buttonInputCount') > 0 &&
            (record.get('productRevision') >= dSS.config.PushbuttonMinFW || record.get('isVdcDevice'))
        ) {
            stdForms.push({
                xtype: 'dssdevgrouppushbuttonform',
                device: record,
                product: type,
                productNumber: deviceNumber,
                matrix: matrix,
                group: override,
                hidden: matrix.enabletypehack
            });

            stdForms.push({
                xtype: 'dssdevbuttonusageform',
                matrix: matrix,
                group: override,
                device: record
            });

            loaders.push(new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.BUTTON_USAGE,
                url: '/json/device/getButtonUsage',
                params: {
                    dsuid: dSUID
                }
            }));

            if (!deviceIsOfGrKl300Type || !deviceIsOfGrKl300Type.Hkl) {
                if (matrix.pushbadvanced) {
                    advancedForms.push({
                        xtype: 'dssdevgrouppushbuttonadvancedform',
                        device: record,
                        matrix: matrix,
                        product: type,
                        productNumber: deviceNumber,
                        group: override
                    });
                } else {
                    stdForms.push({
                        xtype: 'dssdevgrouppushbuttonadvancedform',
                        device: record,
                        matrix: matrix,
                        product: type,
                        productNumber: deviceNumber,
                        group: override,
                        disabled: true,
                        hidden: true
                    });
                }
            }

            addPBOutModeLoader = true;
            addLocalPrioLoader = true;
        }

        if (deviceIsOfGrKl300Type && !deviceIsOfGrKl300Type.sensor) {
            stdForms.push(
                new dSS.devprops.AttachedShadeTerminalDeviceConfigForm({
                    device: record,
                    matrix: matrix
                })
            );

            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.ATTACHED_TERMINAL_DEVICE,
                    url: '/api/v1/apartment/dsDevices/' + record.get('dsuid')
                })
            );
        }

        var renderMultibuttonConfigFormAtTheBottom = false;
        if (matrix.pushbutton &&
            matrix.twowayconfig &&
            dSS.util.deviceHasMultipleButtons(functionId)) {
            var buttonInputIndex = record.get('buttonInputIndex');
            if (buttonInputIndex === 0 || buttonInputIndex === 2) {
                renderMultibuttonConfigFormAtTheBottom = true;
            }
        }
        if (type === dSS.config.deviceTypeNum.UMR) {
            renderMultibuttonConfigFormAtTheBottom = record.get('dSUIDIndex') === 0;
        }
        if (type === dSS.config.deviceTypeNum.SKM) {
            renderMultibuttonConfigFormAtTheBottom = record.get('dSUIDIndex') % 2 !== 1;
        }

        if (renderMultibuttonConfigFormAtTheBottom) {
            stdForms.push(
                new dSS.devprops.MultiButtonConfigForm({
                    device: record,
                    group: color,
                    matrix: matrix,
                    hidden: matrix.enabletypehack
                })
            );
        }

        if (matrix.inputmode) {
            stdForms.push({
                xtype: 'dssdevinputmodeform',
                dSUID: dSUID,
                matrix: matrix
            });
            addPBOutModeLoader = true;
        }

        if (matrix.akmsensor) {
            stdForms.push(
                new dSS.devprops.AKMSensorConfigForm({
                    device: record,
                    group: color,
                    matrix: matrix
                })
            );

            loaders.push(new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.AKM_BINARY_INPUTS,
                url: '/json/device/getBinaryInputs',
                params: {
                    dsuid: dSUID
                }
            }));
        }

        if (matrix.akminput) {
            stdForms.push({
                xtype: 'dssakminputconfigform',
                device: record,
                matrix: matrix
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.AKM_INPUT_PROPERTY,
                    url: '/json/property/getString',
                    params: {
                        path: '/apartment/zones/zone0/devices/' + dSUID + '/AKMInputProperty'
                    }
                })
            );
        }

        if (matrix.akmdelay) {
            stdForms.push({
                xtype: 'dssakmdelayconfigform',
                device: record,
                group: color,
                matrix: matrix
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.AKM_DELAY_CONFIG,
                    url: '/json/device/getAKMInputTimeouts',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (!deviceIsOfGrKl300Type && (
            matrix.outmode ||
            matrix.outmodeswitch ||
            matrix.umroutmode ||
            matrix.outmodegeneric ||
            matrix.outmodeenoceanvalve ||
            matrix.heatingoutmode && record.get('VdcHardwareModelGuid') === '' ||
            matrix.extradimmer
        )) {
            stdForms.push({
                xtype: 'dssdevoutputmodeform',
                device: record,
                group: override,
                matrix: matrix
            });
            addPBOutModeLoader = true;

            if (matrix.outmodeauto) {
                loaders.push(
                    new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.CURRENT_OUTPUT_AUTO_MODE,
                        url: '/json/device/getOutputValue',
                        params: {
                            dsuid: dSUID,
                            type: 'currentOutputMode'
                        }
                    })
                );
            }
        }

        if (matrix.umvrelay) {
            stdForms.push({
                xtype: 'dssdevumvrelayconfigform',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.UMV_RELAY,
                    url: '/json/device/getUMVRelayValue',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.ftwtempcontrolventilationselect) {
            stdForms.push({
                xtype: 'dssdevftwtempcontrolconfig',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.FTW_MODE,
                    url: '/json/device/getSK204Config',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.ventconfig) {
            // hack in order to be able to comply with the new spec
            var usedSUID = dSUID;
            var usePresent = record.get('isPresent');
            var useGroup = dSS.util.getJokerGroup(groups);

            if (record.get('dSUIDIndex') === 0) {
                var paired = record.get('pairedDevices');
                if (paired.length) {
                    usedSUID = paired[0];
                    var store = dSS.data.Loader.getDeviceStore();
                    var index = store.find('dsuid', paired[0]);
                    if (index !== -1) {
                        var devrec = store.getAt(index);
                        usePresent = devrec.get('isPresent');
                        useGroup = dSS.util.getJokerGroup(devrec.get('groups'));
                    } else {
                        usePresent = false;
                    }
                }
            }

            stdForms.push({
                xtype: 'dssdevventilationconfig',
                dSUID: usedSUID,
                present: usePresent,
                group: useGroup
            });
        }

        if (matrix.ftwdisplaysettings) {
            stdForms.push({
                xtype: 'dssdevftwdisplaysettings',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.FTW_DISPLAY_SETTINGS,
                    url: '/json/device/getSK204DisplayMode',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.ftwbacklighttimeout) {
            stdForms.push({
                xtype: 'dssdevftwbacklighttimeout',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.FTW_BACKLIGHT_TIMEOUT,
                    url: '/json/device/getSK204BacklightTimeout',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.temperatureoffset) {
            var sensorIndex = dSS.util.GetSensorIndex(record, dSS.config.SensorTypes.TEMPERATURE_INDOORS);

            if (sensorIndex !== -1) {
                stdForms.push({
                    xtype: 'temperatureoffsetconfigform',
                    device: record
                });
                loaders.push(
                    new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.SENSOR_VALUE,
                        url: '/json/device/getSensorValue',
                        params: {
                            dsuid: dSUID,
                            sensorIndex: sensorIndex
                        }
                    })
                );
                loaders.push(
                    new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.TEMPERATURE_OFFSET,
                        url: '/json/device/getTemperatureOffset',
                        params: {
                            dsuid: dSUID
                        }
                    })
                );
            }
        }

        if (matrix.blinkconfig) {
            blinkForms.push({
                xtype: 'outputblinkconfigform',
                device: record,
                matrix: matrix
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.BLINK_CONFIG,
                    url: '/json/device/getBlinkConfig',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.impulseconfig) {
            blinkForms.push({
                xtype: 'dssdevoutputimpulseconfigform',
                device: record,
                matrix: matrix
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.OUTPUT_AFTER_IMPULSE,
                    url: '/json/device/getOutputAfterImpulse',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.dimtimeconfig) {
            transitionForms.push({
                xtype: 'dssramptimeconfigform',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.DIMTIME0,
                    url: '/json/device/getTransitionTime',
                    params: {
                        dsuid: dSUID,
                        dimtimeIndex: 0
                    }
                })
            );
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.DIMTIME2,
                    url: '/json/device/getTransitionTime',
                    params: {
                        dsuid: dSUID,
                        dimtimeIndex: 2
                    }
                })
            );
        }

        if (matrix.locationconfig) {
            stdForms.push({
                xtype: 'dssdevlocationform',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.LOCATION,
                    url: '/json/device/getCardinalDirection',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.windprotectionconfigawning || matrix.windprotectionconfigblind) {
            stdForms.push({
                xtype: 'dssdevwindprotectionclassform',
                device: record,
                matrix: matrix
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.WIND_PROTECTION,
                    url: '/json/device/getWindProtectionClass',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (deviceIsOfGrKl300Type && !deviceIsOfGrKl300Type.sensor) {
            stdForms.push({
                xtype: 'dssdevgrkl300outputmodeform',
                device: record,
                group: override,
                matrix: matrix
            });
            addPBOutModeLoader = true;
        }

        if (addPBOutModeLoader) {
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.COMBINED_PB_OUTMODE,
                    url: '/json/property/query',
                    params: {
                        query: '/apartment/zones/zone0/devices/' + dSUID +
                            '(outputMode)/button(id,inputMode,setsLocalPriority,callsPresent,activeGroup)'
                    }
                })
            );
        }

        if (addLocalPrioLoader) {
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SCENE_CONFIGURATION,
                    url: '/json/device/getSceneMode',
                    params: {
                        sceneID: dSS.config.scenes.LOCAL_ON,
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.outconfigswitch) {
            advancedForms.push({
                xtype: 'outputswitchtresholdconfigform',
                device: record
            });
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.OUTPUT_SWITCH_THRESHOLD,
                    url: '/json/device/getSwitchThreshold',
                    params: {
                        dsuid: dSUID
                    }
                })
            );
        }

        if (matrix.operationlock && !deviceIsOfGrKl300Type) {
            advancedForms.push({
                xtype: 'operationlockignoreconfigform',
                device: record
            });
        }
    } // CIRCUITS

    var additionalInfo = [];
    var tabHeight;
    var devDsuid;
    var devDsid;

    var reason = dSS._('Modifying settings for this device is not allowed!');
    var confLock = record.get('configurationLocked');
    var oemState = record.get('OemStatus');
    if (confLock && (oemState === 'Unknown' || oemState === 'Loading')) {
        reason = dSS._('Modifying settings for this device is not allowed, because device configuration has not been read out yet!');
    }

    var customTitle;
    if (featureConfig === dSS.config.FeatureConfig.SCENE) {
        customTitle = dSS._('Activity settings');
        tabHeight = 170;
        var scene;
        additionalInfo.push({
            key: dSS._('Output mode'),
            value: dSS._(dSS.config.OutModeStrings[record.get('outputMode')])
        });
        try {
            scene = scope.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._('Error'), err);
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var sceneId = record.get('zoneId') + '-' + cls[0] + '-' + scene;
        var sceneRec = sceneStore.getById(sceneId);
        var sceneName = sceneRec
            ? sceneRec.get('name')
            : '';
        var extraTitle = sceneName.length
            ? ' - ' + sceneName
            : ' - ' + dSS._(dSS.config.sceneNames[scene]);

        if (dSS.devprops.CheckShowSceneProperties(matrix)) {
            stdForms.push({
                xtype: 'dssdevscenepropsform',
                deviceRecord: record,
                scene: scene,
                visibilityMatrix: matrix
            });
            advancedForms.push({
                xtype: 'dssdevscenepropsadvancedform',
                deviceRecord: record,
                scene: scene,
                visibilityMatrix: matrix
            });
            if (record.get('isPresent')) {
                loaders.push(
                    new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.SCENE_MODE,
                        url: '/json/device/getSceneMode',
                        params: {
                            sceneID: scene,
                            dsuid: dSUID
                        }
                    })
                );
            }
        }

        if (scene >= dSS.config.scenes.PRESENT || scene === dSS.config.scenes.PANIC) {
            tabHeight = 445;
            stdForms.push({
                xtype: 'dssdevicesettingswrapper',
                deviceRecord: record,
                scene: scene,
                visibilityMatrix: matrix
            });

            if (record.get('isPresent')) {
                if (matrix.outputchannels) {
                    var supportedChannels = record.json.outputChannels
                        .map(function (channel) {
                            return channel.channelId;
                        })
                        .join(';');
                    loaders.push(
                        new dSS.devprops.Loader({
                            loaderId: dSS.devprops.Loaders.SCENE_VALUE_OUTPUT_CHANNELS,
                            url: '/json/device/getOutputChannelSceneValue',
                            params: {
                                sceneNumber: scene,
                                channels: supportedChannels,
                                dsuid: dSUID
                            }
                        })
                    );
                } else if (matrix.fcu) {
                    loaders.push(
                        new dSS.devprops.Loader({
                            loaderId: dSS.devprops.Loaders.SCENE_VALUE2_OUTPUT_CHANNELS,
                            url: '/json/device/getOutputChannelSceneValue2',
                            params: {
                                sceneNumber: scene,
                                channels: 'airFlowIntensity;airLouverPosition',
                                dsuid: dSUID
                            }
                        })
                    );
                } else if (matrix.outvalue8) {
                    loaders.push(
                        new dSS.devprops.Loader({
                            loaderId: dSS.devprops.Loaders.SCENE_VALUE,
                            url: '/json/device/getSceneValue',
                            params: {
                                sceneID: scene,
                                dsuid: dSUID
                            }
                        })
                    );
                }
            }
        }

        // check if device is locked in an apartment scene
        if (record.get('clusterLock')) {
            var lockedScenes = record.get('lockedScenes');
            if (lockedScenes.indexOf(scene) !== -1) {
                confLock = true;
            }
        }

        // otherwise we could be in some zone scene which contains a
        // locked device (not necessarily ours)
        if (!confLock) {
            var zoneID = record.get('zoneId');
            if (zoneID !== 0) {
                var zoneRecord = dSS.util.getZoneById(zoneID);
                if (zoneRecord) {
                    var locks;
                    if (!dSS.util.isApartmentScene(scene)) {
                        locks = zoneRecord.get('clusterLocks');
                        Ext.each(locks, function (lock) {
                            if (lock.deviceClass == cls[0] ||
                                cls[0] === dSS.config.deviceClass.BL && lock.deviceClass === dSS.util.getHeatingGroup(groups) ||
                                cls[0] === dSS.config.deviceClass.SW && lock.deviceClass === dSS.util.getJokerGroup(groups)) {
                                if (lock.lockedScenes.indexOf(scene) !== -1) {
                                    confLock = true;
                                }
                            }
                        });
                    } else {
                        locks = record.get('lockedApartmentScenes');
                        if (locks.indexOf(scene) !== -1) {
                            confLock = true;
                        }
                    }
                }
            }
        }
    } else {
        devDsuid = dSUID;
        devDsid = record.get('dsid');
    }

    var tabForms = [];

    if (stdForms.length) {
        tabForms.push({
            title: dSS._('Settings'),
            forms: stdForms
        });
    }

    if (advancedForms.length) {
        tabForms.push({
            title: dSS._('Advanced settings'),
            forms: advancedForms
        });
    }

    if (blinkForms.length) {
        var tabtitle = dSS._('Blink');
        if (type === dSS.config.deviceTypeNum.UMR) {
            tabtitle = dSS._('Impulse');
        }
        tabForms.push({
            title: tabtitle,
            forms: blinkForms
        });
    }

    if (transitionForms.length) {
        tabForms.push({
            title: dSS._('Transition'),
            forms: transitionForms
        });
        width = 450;
    }

    if (!advancedForms.length && !blinkForms.length) {      // TODO: what about checking if there are transition forms?
        return {
            panel: new dSS.devprops.DevicePropertiesPanel({
                dSUID: devDsuid,
                dSID: devDsid,
                DisplayID: record.get('DisplayID'),
                name: record.get('name'),
                showDefaults: true,
                additionalInfo: additionalInfo,
                standardForms: stdForms,
                loaders: loaders,
                tabHeight: tabHeight,
                stayMasked: confLock,
                maskReason: reason
            }),
            title: customTitle,
            extraTitle: extraTitle,
            windowWidth: width
        };
    } else {
        return {
            panel: new dSS.devprops.DevicePropertiesMultitabPanel({
                dSUID: devDsuid,
                dSID: devDsid,
                DisplayID: record.get('DisplayID'),
                name: record.get('name'),
                showDefaults: true,
                additionalInfo: additionalInfo,
                tabForms: tabForms,
                loaders: loaders,
                tabHeight: tabHeight,
                stayMasked: confLock,
                maskReason: reason
            }),
            title: customTitle,
            extraTitle: extraTitle,
            windowWidth: width
        };
    }
};
