dSS.devprops.CreatePanelForHeatingProperties = function (record) {
    var matrix;
    try {
        matrix = dSS.devprops.GetVisibleProperties(record);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (!matrix.heatingprops) {
        Ext.Msg.alert(dSS._('Error'), dSS._('Climate properties can only be configured for blue klemme M devices'));
        return;
    }

    var standardForms = [];
    var forms = [];
    var heatingloaders = [];
    var dsuid = record.get('dsuid');

    // hack for the UMR with new firmware
    if (!matrix.umroutmodeswitch2x) {
        heatingloaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.VALVE_CONTROL_MODE,
                url: '/json/device/getValveControlMode',
                params: {
                    dsuid: dsuid
                }
            })
        );
    }

    var outputMode = record.get('outputMode');
    switch (outputMode) {
        case dSS.config.deviceParams.OUTPUT_MODE_PWM:
        case dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM:
            heatingloaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.VALVE_TIMER_MODE,
                    url: '/json/device/getValveTimerMode',
                    params: {
                        dsuid: dsuid
                    }
                })
            );
        // fall through since PWM control mode needs both loaders
        case dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_2_STEPS:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_3_STEPS:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_PARALLEL:
            heatingloaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.VALVE_PWM_MODE,
                    url: '/json/device/getValvePwmMode',
                    params: {
                        dsuid: dsuid
                    }
                })
            );
            break;
        default:
            break;
    }

    if (!matrix.umroutmodeswitch2x) {
        standardForms = [ {
            xtype: 'heatingpropsform',
            dSUID: dsuid,
            outputMode: outputMode
        } ];

        forms.push({
            title: dSS._('Settings'),
            forms: standardForms
        });
    }

    var tabHeight = dSS.util.isHeatingPWMMode(outputMode)
        ? 510
        : 350;

    if (dSS.util.isHeatingPWMMode(outputMode) ||
        outputMode === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED ||
        outputMode === dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL) {
        var advancedForms = [ {
            xtype: 'heatingpropsadvancedform',
            dSUID: dsuid,
            outputMode: outputMode,
            matrix: matrix
        } ];

        forms.push({
            title: dSS._('Advanced settings'),
            forms: advancedForms
        });
    }

    return {
        panel: new dSS.devprops.DevicePropertiesMultitabPanel({
            DisplayID: record.get('DisplayID'),
            name: record.get('name'),
            showDefaults: true,
            tabForms: forms,
            loaders: heatingloaders,
            tabHeight: tabHeight
        }),
        extraTitle: dSS._('climate')
    };
};