// add advanced form, make sure it can communicate with the standard form

dSS.devprops.ShadePropertiesAdvancedForm = Ext.extend(dSS.devprops.BaseShadePropertiesForm, {
    constructor: function (config) {
        if (config.dSUID == null) {
            Ext.MessageBox.alert(dSS._('Error'), ('Missing dSUID'));
            return;
        }

        if (config.matrix == null) {
            Ext.Msg.alert(dSS._('Error'), ('Missing matrix'));
            return;
        }

        this.dSUID = config.dSUID;

        this.addEvents('formsubmitted', 'formloaded', 'sharedata', 'formretry', 'formchangd'); // TODO: formchangd is most probably typo, should be formchangd - see also heating form!!!

        this.columnWidths = [ 260, 170, 20 ];

        var tmotionWhenBlinkingLabel = new Ext.form.Label({
            text: dSS._('Motion when blinking') + ':'
        });
        this.motionWhenBlinking = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_MotionWhenBlinkingNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 8415, // 0xff * 33ms
        });
        var motionWhenBlinkingUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        this.maximumMotionTimeLabel = new Ext.form.Label({
            text: dSS._('Maximum motion time') + ':',
            disabled: true
        });
        this.maximumMotionTime = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_MaxMotionTimeNumberField',

            bubbleEvents: [ 'change' ],
            disabled: true,
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 655
        });
        this.maximumMotionTimeUnitLabel = new Ext.form.Label({
            text: dSS._('s'),
            disabled: true
        });

        var reverseTimeOnDirectionChangeLabel = new Ext.form.Label({
            text: dSS._('Reverse time on direction change') + ':'
        });
        this.reverseTimeOnDirectionChange = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_ReverseTimeOnDirectionChangeNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 2550
        });
        var reverseTimeOnDirectionChangeUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var followUpTimeAtEndPositionLabel = new Ext.form.Label({
            text: dSS._('Follow-up time at end position') + ':'
        });
        this.followUpTimeAtEndPosition = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_FollowUpTimeAtEndPositionNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 25500 // ms, 0xff * 0.1s
        });
        var followUpTimeAtEndPositionUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var thresholdOfEndSwitchRecognitionLabel = new Ext.form.Label({
            text: dSS._('Threshold of end switch recognition') + ':'
        });
        this.thresholdOfEndSwitchRecognition = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_ThresholdOfEndSwitchRecognitionNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 14790 // 0xff * 58
        });
        var thresholdOfEndSwitchRecognitionUnitLabel = new Ext.form.Label({
            text: dSS._('mA')
        });

        var numberOfStepsForFineTuningAdjustmentLabel = new Ext.form.Label({
            text: dSS._('Number of steps for fine tuning adjustment') + ':',
        });
        this.numberOfStepsForFineTuningAdjustment = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_NumberOfStepsForFineTuningNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 0xff,
        });

        var motionTimeForFineTuningAdjustmentLabel = new Ext.form.Label({
            text: dSS._('Motion time for fine tuning adjustment') + ':'
        });
        this.motionTimeForFineTuningAdjustment = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesAdvancedForm_MotionTimeForFineTuningNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 2550,
        });
        var motionTimeForFineTuningAdjustmentUnitLabel = new Ext.form.Label({
            text: dSS._('ms'),
        });

        var items = [
            this.makeRow([
                tmotionWhenBlinkingLabel,
                this.motionWhenBlinking,
                motionWhenBlinkingUnitLabel
            ]),
            this.makeRow([
                this.maximumMotionTimeLabel,
                this.maximumMotionTime,
                this.maximumMotionTimeUnitLabel
            ]),
            this.makeRow([
                reverseTimeOnDirectionChangeLabel,
                this.reverseTimeOnDirectionChange,
                reverseTimeOnDirectionChangeUnitLabel
            ]),
            this.makeRow([
                followUpTimeAtEndPositionLabel,
                this.followUpTimeAtEndPosition,
                followUpTimeAtEndPositionUnitLabel
            ]),
            this.makeRow([
                thresholdOfEndSwitchRecognitionLabel,
                this.thresholdOfEndSwitchRecognition,
                thresholdOfEndSwitchRecognitionUnitLabel
            ]),
            this.makeRow([
                numberOfStepsForFineTuningAdjustmentLabel,
                this.numberOfStepsForFineTuningAdjustment,
                null
            ]),
            this.makeRow([
                motionTimeForFineTuningAdjustmentLabel,
                this.motionTimeForFineTuningAdjustment,
                motionTimeForFineTuningAdjustmentUnitLabel
            ])
        ];

        Ext.apply(this, {
            cls: 'dsClsShadePropertiesAdvancedForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'sharedata', 'formretry', 'formchanged' ],
            padding: 5,
            scope: this,
            border: false,
            items: items
        });

        this.resetLoaded();

        dSS.devprops.ShadePropertiesAdvancedForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.on('change', function () {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            var val;
            var ok = data && data.json && data.json.ok;
            if (ok) {
                val = data.json.result.value;
            }

            switch (data.loader) {
                case dSS.devprops.Loaders.SHADE_MTIME_BLINK:
                    this.loaded.motionTimeBlink = true;
                    if (ok) {
                        this.motionWhenBlinking.setValue(val * 33);
                        dSS.util.ResetDirtyField(this.motionWhenBlinking);
                    } else {
                        this.motionWhenBlinking.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH:
                    this.loaded.reverseTimeOnDirectionChange = true;
                    if (ok) {
                        var rt = (val & 0x0000ff00) >> 8;
                        this.reverseTimeOnDirectionChange.setValue(rt * 10);
                        dSS.util.ResetDirtyField(this.reverseTimeOnDirectionChange);
                    } else {
                        this.reverseTimeOnDirectionChange.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW:
                    this.loaded.fTimeEndposThresholdEndsw = true;
                    if (ok) {
                        var followUp = val & 0x000000ff;
                        var threshold = (val & 0x0000ff00) >> 8;
                        this.followUpTimeAtEndPosition.setValue(followUp * 100);
                        dSS.util.ResetDirtyField(this.followUpTimeAtEndPosition);
                        this.thresholdOfEndSwitchRecognition.setValue(threshold * 58);
                        dSS.util.ResetDirtyField(this.thresholdOfEndSwitchRecognition);
                    } else {
                        this.followUpTimeAtEndPosition.markInvalid();
                        this.thresholdOfEndSwitchRecognition.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE:
                    this.loaded.fineTuningAdjustment = true;
                    if (ok) {
                        var steps = val & 0x000000ff;
                        var mtime = (val & 0x0000ff00) >> 8;
                        this.numberOfStepsForFineTuningAdjustment.setValue(steps);
                        dSS.util.ResetDirtyField(this.numberOfStepsForFineTuningAdjustment);
                        this.motionTimeForFineTuningAdjustment.setValue(mtime * 10);
                        dSS.util.ResetDirtyField(this.motionTimeForFineTuningAdjustment);
                    } else {
                        this.numberOfStepsForFineTuningAdjustment.markInvalid();
                        this.motionTimeForFineTuningAdjustment.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_POS_TIME_MAX:
                    this.loaded.maximumMotionTime = true;
                    if (ok) {
                        var supported = data.json.result.supported;
                        val = data.json.result.value;
                        this.maximumMotionTimeLabel.setDisabled(!supported);
                        this.maximumMotionTime.setDisabled(!supported);
                        this.maximumMotionTimeUnitLabel.setDisabled(!supported);
                        if (supported) {
                            this.maximumMotionTime.setValue(val);
                        }
                        dSS.util.ResetDirtyField(this.maximumMotionTime);
                    }
                    break;
                default:
                    return;
            }

            if (this.isAllLoaded()) {
                this.fireEvent('formloaded', true);
                this.resetLoaded();
            }
            if (this.loaded.motionTimeBlink &&
                this.loaded.maximumMotionTime &&
                this.loaded.reverseTimeOnDirectionChange &&
                this.loaded.fTimeEndposThresholdEndsw &&
                this.loaded.fineTuningAdjustment) {
                this.fireEvent('formloaded', true);

                this.loaded = {
                    motionTimeBlink: false,
                    maximumMotionTime: false,
                    reverseTimeOnDirectionChange: false,
                    fTimeEndposThresholdEndsw: false
                };
            }
        }, this);

        this.on('beforedestroy', function () {
            this.closed = true;
        }, this);

        this.motionWhenBlinking.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.maximumMotionTime.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.reverseTimeOnDirectionChange.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.followUpTimeAtEndPosition.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.thresholdOfEndSwitchRecognition.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.numberOfStepsForFineTuningAdjustment.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.motionTimeForFineTuningAdjustment.on('change', function () {
            this.fireEvent('sharedata', 'shadepropadvancedchanged');
        }, this);

        this.on('submit', function () {
            if (!this.getForm().isValid()) {
                Ext.Msg.alert(
                    dSS._('Error'),
                    dSS._('Please correct values in the red marked fields and try again')
                );
                this.fireEvent('formretry');
                return;
            }

            var save = [];

            if (this.motionWhenBlinking.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x11,
                        dsuid: this.dSUID,
                        value: Math.floor(this.motionWhenBlinking.getValue() / 33)
                    },
                    failureMsg: dSS._('Could not save blinking motion time settings')
                });
            }

            if (this.maximumMotionTime.isDirty()) {
                save.push({
                    url: '/json/device/setMaxMotionTime',
                    scope: this,
                    params: {
                        dsuid: this.dSUID,
                        seconds: this.maximumMotionTime.getValue()
                    },
                    failureMsg: dSS._('Could not save maximum motion time settings')
                });
            }

            if (this.reverseTimeOnDirectionChange.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x37,
                        dsuid: this.dSUID,
                        value: Math.floor(this.reverseTimeOnDirectionChange.getValue() / 10)
                    },
                    failureMsg: dSS._('Could not save reverse time on direction change settings')
                });
            }

            if (this.followUpTimeAtEndPosition.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x38,
                        dsuid: this.dSUID,
                        value: Math.floor(this.followUpTimeAtEndPosition.getValue() / 100)
                    },
                    failureMsg: dSS._('Could not save follow-up time at end position settings')
                });
            }

            if (this.thresholdOfEndSwitchRecognition.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x39,
                        dsuid: this.dSUID,
                        value: Math.floor(this.thresholdOfEndSwitchRecognition.getValue() / 58)
                    },
                    failureMsg: dSS._('Could not save threshold of end switch recognition settings')
                });
            }

            if (this.numberOfStepsForFineTuningAdjustment.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3a,
                        dsuid: this.dSUID,
                        value: this.numberOfStepsForFineTuningAdjustment.getValue()
                    },
                    failureMsg: dSS._('Could not save number of steps for fine tuning adjustment settings')
                });
            }

            if (this.motionTimeForFineTuningAdjustment.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3b,
                        dsuid: this.dSUID,
                        value: Math.floor(this.motionTimeForFineTuningAdjustment.getValue() / 10)
                    },
                    failureMsg: dSS._('Could not save motion time for fine tuning adjustment settings')
                });
            }

            this.saveOK = true;
            var call = this.saveData.createDelegate(this, [ save, true ]);
            call();
        }, this);
    },

    resetLoaded: function () {
        this.loaded = {
            motionTimeBlink: false,
            reverseTimeOnDirectionChange: false,
            fTimeEndposThresholdEndsw: false,
            maximumMotionTime: false,
            fineTuningAdjustment: false
        };
    },

    saveDataCallback: function (records, state) {
        var call = this.saveData.createDelegate(this, [ records, state ]);
        call();
    },

    saveData: function (records, ok) {
        if (!ok) {
            this.saveOK = false;
        }
        if (!records.length) {
            if (!this.saveOK) {
                dSS.MessageBus.fireEvent(
                    'statusmessage',
                    dSS._('Error') + ': ' + dSS._('Some advanced properties could not be saved')
                );
            }
            this.fireEvent('formsubmitted');
            return;
        }

        var record = records.shift();

        var cb = function (state) {
            var call = this.saveDataCallback.createDelegate(this, [ records, state ]);
            call();
        };
        dSS.util.genericRequest(record, cb.createDelegate(this));
    }
});

Ext.reg('shadepropsadvancedform', dSS.devprops.ShadePropertiesAdvancedForm);
