// parameters:
//      dSUID: device dSUID

/* globals dSS, Ext */

dSS.devprops.FinsCalibrationWizard = Ext.extend(Ext.Window, {
    constructor: function (config) {

        if (config.dSUID == null) {
            throw dSS._('Missing dSUID');
        }

        if (config.mtimeadjust == null ||       // DimStep length
            config.numsteps == null ||          // DimStep steps no.
            config.turntime == null) {
            throw dSS._('FinsCalibrationWizard: called with missing parameters');
        }

        this.loaded = {
            turntime: config.turntime,
            numsteps: config.numsteps,
            mtimeadjust: config.mtimeadjust
        };

        this.currentStep = 0;

        this.FINS_CONFIG_TIME = 50;
        this.BOUNDARY_1 = 0x9f00;
        this.BOUNDARY_2 = 0x8100;
        this.dSUID = config.dSUID;
        this.tempTurnTime = 0;
        this.incStepCounter = 0;

        this.finsAngle = {
            OPEN: 0,
            CENTER: 1,
            CLOSED: 2
        };

        this.angle = this.finsAngle.CLOSED;

        var devStore = dSS.data.Loader.getDeviceStore();
        var record = devStore.getById(this.dSUID);
        if (record == null) {
            throw dSS._('Device not found');
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        if (cls[0] !== dSS.config.deviceClass.GR) {
            throw dSS._('Unsupported device group');
        }

        if (dSS.config.deviceTypes[type] !== 'KL' || [220, 230].indexOf(number) === -1) {
            throw dSS._('Unsupported device type');
        }

        var text1 = dSS._('Fins: no text given 1');
        var text2 = dSS._('Fins: no text given 2');
        this.buttonWidth = 80;

        this.cancelButton = new Ext.Button({
            cls: 'dsClsFinsCalibrationWizard_CancelButton',

            xtype: 'button',
            text: dSS._('Cancel'),
            width: this.buttonWidth,
            scope: this,
            handler: this.cancelWizard.createDelegate(this)
        });

        this.contButton = new Ext.Button({
            cls: 'dsClsFinsCalibrationWizard_ContinueButton',

            text: dSS._('Continue'),
            handler: this.nextStep.createDelegate(this),
            disabled: true,
            width: this.buttonWidth
        });

        Ext.apply(this, {
            cls: 'dsClsFinsCalibrationWizard',

            layout: 'fit',
            closable: false,
            width: dSS.gthelper.getDialogConfig('finsCalibrationWindowWidth'),
            height: 380,
            resizable: false,
            modal: true,
            title: dSS._('Calibration of fins turn time: step 1 of 2'),
            items: [
                {
                    layout: 'card',
                    ref: 'main',
                    padding: 10,
                    border: false,
                    bodyCfg: {
                        cls: 'x-window-body x-window-mc'
                    },
                    items: [
                        {
                            cls: 'dsClsFinsCalibrationWizard_Step1Panel',

                            xtype: 'panel',
                            ref: 'step1',
                            height: 350,
                            layout: 'border',
                            border: false,
                            items: [
                                {
                                    xtype: 'panel',
                                    region: 'north',
                                    height: 270,
                                    html: text1,
                                    border: false,
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    }
                                },
                                {
                                    xtype: 'panel',
                                    border: false,
                                    ref: 'action',
                                    region: 'center',
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    },
                                    layout: 'hbox',
                                    items: [
                                        {
                                            xtype: 'label',
                                            width: 120,
                                            ref: 'numsteps',
                                            text: dSS._('Number of steps') + ': 0'
                                        },
                                        {
                                            xtype: 'box',
                                            flex: 3
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step1Panel_UpButton',

                                            xtype: 'button',
                                            iconCls: 'buttonUpIcon',
                                            ref: 'incButton',
                                            disabled: true,
                                            width: 79,
                                            handler: this.handleInc.createDelegate(this)
                                        },
                                        {
                                            xtype: 'box',
                                            width: 6
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step1Panel_StopButton',

                                            xtype: 'button',
                                            ref: 'stopButton',
                                            text: dSS._('Stop'),
                                            width: 79,
                                            handler: this.handleStop.createDelegate(this)
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            cls: 'dsClsFinsCalibrationWizard_Step2Panel',

                            xtype: 'panel',
                            hidden: true,
                            layout: 'border',
                            border: false,
                            height: 350,
                            scope: this,
                            ref: 'step2',
                            items: [
                                {
                                    xtype: 'panel',
                                    region: 'north',
                                    height: 210,
                                    html: text2,
                                    border: false,
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    }
                                },
                                {
                                    xtype: 'panel',
                                    region: 'center',
                                    height: 100,
                                    border: false,
                                    ref: 'action',
                                    bodyCfg: {
                                        cls: 'x-window-body x-window-mc'
                                    },
                                    layout: 'absolute',
                                    items: [
                                        {
                                            xtype: 'label',
                                            text: dSS._('Fins turn time') + ':',
                                            x: 0,
                                            y: 4
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_FinsTurnTimeSpinnerField',

                                            xtype: 'spinnerfield',
                                            width: 50,
                                            ref: 'turnspinner',
                                            allowBlank: false,
                                            minValue: 0,
                                            allowDecimals: true,
                                            accelerate: false,
                                            defaultValue: 0,
                                            splitterClass: ' ',
                                            incrementValue: 0.05,
                                            decimalPrecision: 2,
                                            disabled: false,
                                            allowEmpty: false,
                                            x: 150,
                                            y: 0
                                        },
                                        {
                                            xtype: 'label',
                                            text: dSS._('s'),
                                            x: 220,
                                            y: 4
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_OpenFinsButton',

                                            xtype: 'button',
                                            text: dSS._('Open fins'),
                                            width: 131,
                                            scope: this,
                                            handler: this.handleOpen.createDelegate(this),
                                            x: 250,
                                            y: 0
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_UpButton',

                                            xtype: 'button',
                                            iconCls: 'buttonUpIcon',
                                            ref: 'upButton',
                                            width: 79,
                                            handler: this.handleStepUp.createDelegate(this),
                                            x: 387,
                                            y: 0
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_CenterFinsButton',

                                            xtype: 'button',
                                            text: dSS._('Center fins'),
                                            width: 131,
                                            handler: this.handleCenter.createDelegate(this),
                                            x: 250,
                                            y: 30
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_CloseFinsButton',

                                            xtype: 'button',
                                            text: dSS._('Close fins'),
                                            width: 131,
                                            handler: this.handleClose.createDelegate(this),
                                            x: 250,
                                            y: 60
                                        },
                                        {
                                            cls: 'dsClsFinsCalibrationWizard_Step2Panel_DownButton',

                                            xtype: 'button',
                                            iconCls: 'buttonDownIcon',
                                            ref: 'downButton',
                                            width: 79,
                                            handler: this.handleStepDown.createDelegate(this),
                                            x: 387,
                                            y: 60
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            buttons: [
                this.contButton,
                this.cancelButton
            ]
        });

        dSS.devprops.FinsCalibrationWizard.superclass.constructor.apply(this, arguments);

        this.addEvents('ready2read', 'position1reached', 'position2reached');

        this.on('afterrender', function () {
            this.step2KeyMap = new Ext.KeyMap(this.getId(), [
                {
                    key: Ext.EventObject.UP,
                    fn: this.handleOpen.createDelegate(this),
                    scope: this,
                    stopEvent: true
                },
                {
                    key: Ext.EventObject.DOWN,
                    fn: this.handleClose.createDelegate(this),
                    scope: this,
                    stopEvent: true
                },
                {
                    key: [ Ext.EventObject.LEFT, Ext.EventObject.RIGHT ],
                    fn: this.handleCenter.createDelegate(this),
                    scope: this,
                    stopEvent: true
                }
            ]);

            this.step2KeyMap.disable();

            this.main.step1.show();
            this.main.step2.hide();

            this.setConfig(
                true, 3, 0x3f, ((this.FINS_CONFIG_TIME >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (this.FINS_CONFIG_TIME & 0xff),
                        dSS.util.genericRequest.createDelegate(this, [
                            {
                                url: '/json/device/callScene',
                                params: {
                                    dsuid: this.dSUID,
                                    sceneNumber: 14
                                },
                                scope: this,
                                evt: 'ready2read'
                            }
                        ]) // genericRequest.createDelegate
                    ], 1
                ) // ^^ append args at position, i.e. after the OK but
                // replace json data from the callback
            );
        }, this);

        this.on('ready2read', function (ok) {
            if (this.closed) {
                return;
            }

            if (!ok) {
                this.main.el.mask();
                Ext.Msg.alert(dSS._('Error'), dSS._('Could not call scene on device'));
                return;
            }

            this.readValue(true, 1);
        }, this);

        this.on('position1reached', function () {
            this.setConfig(
                true, 64, 0x03, 0x80,
                this.setConfig.createDelegate(
                    this, [
                        64, 0x02, 0x00,
                        this.setConfig.createDelegate(
                            this, [
                                64, 0x04, 0x00,
                                this.readValue.createDelegate(this, [ 0 ], 1) // append args at position 1
                            ], 1
                        ) // append args at position 1
                    ], 1
                ) // ^^ append args at position 1
            );
        }, this);

        this.on('position2reached', function () {
            var t = 600;
            this.setConfig(
                true, 3, 0x3f, ((t >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (t & 0xff),
                        this.prepareStep2.createDelegate(this)
                    ], 1
                ) // ^^ append args at position 1
            );
        }, this);
    },

    prepareStep2: function (ok) {
        if (ok) {
            this.contButton.enable();
            this.main.step1.action.incButton.enable();
        }
    },

    handleStop: function () {
        Ext.Ajax.abort(this.tId);
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
    },

    handleInc: function () {
        this.main.step1.action.incButton.disable();
        dSS.util.callScene(this.dSUID, dSS.config.scenes.INC);
        this.tempTurnTime += this.loaded.mtimeadjust;
        this.incStepCounter++;
        this.main.step1.action.numsteps.setText(dSS._('Number of steps') + ': ' + this.incStepCounter);
        this.main.step1.action.incButton.enable.defer(
            this.loaded.mtimeadjust + 800,
            this.main.step1.action.incButton
        );
    },

    genericCallback: function (ok) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            this.contButton.disable();
        } else {
            this.unmask2();
            this.step2KeyMap.enable();
            this.contButton.enable();
        }
    },

    handleOpen: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var commitOpen = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }
            this.angle = this.finsAngle.OPEN;
            this.setConfig(true, 64, 0x04, 0xff, this.genericCallback.createDelegate(this));
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.contButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (this.tempTurnTime & 0xff),
                        commitOpen.createDelegate(this)
                    ], 1
                ) // ^^ append args at position 1
            );
        } else {
            var keepScope = commitOpen.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    handleClose: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var commitClose = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }
            this.angle = this.finsAngle.CLOSED;
            this.setConfig(true, 64, 0x04, 0x00, this.genericCallback.createDelegate(this));
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.contButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (this.tempTurnTime & 0xff),
                        commitClose.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        } else {
            var keepScope = commitClose.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    handleCenter: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var commitCenter = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }
            this.angle = this.finsAngle.CENTER;
            this.setConfig(true, 64, 0x04, 0x80, this.genericCallback.createDelegate(this));
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.contButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (this.tempTurnTime & 0xff),
                        commitCenter.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        } else {
            var keepScope = commitCenter.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    handleStepUp: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var finalizeStepUp = function (val) {
            this.angle = this.finsAngle.OPEN;
            var newPos = val + 0x0800;
            if (newPos > 0xffff) {
                newPos = 0xffff;
            }

            this.setConfig(
                true, 64, 0x03, ((newPos >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        64, 0x02, (newPos & 0xff),
                        this.genericCallback.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        };

        var maskedWait = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }

            if (this.angle === this.finsAngle.CENTER) {
                this.readValue.defer(
                    Math.round(this.tempTurnTime / 2.0) + 300,
                    this,
                    [ true, 2, finalizeStepUp.createDelegate(this) ]
                );
            } else if (this.angle === this.finsAngle.CLOSED) {
                this.readValue.defer(
                    this.tempTurnTime + 300,
                    this,
                    [ true, 2, finalizeStepUp.createDelegate(this) ]
                );
            } else {
                this.readValue(true, 2, finalizeStepUp.createDelegate(this));
            }
        };

        var commitUp = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }

            if (this.angle !== this.finsAngle.OPEN) {
                this.setConfig(true, 64, 0x04, 0xff, maskedWait.createDelegate(this));
            } else {
                var ks = maskedWait.createDelegate(this, [ true ]);
                ks();
            }
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.contButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (this.tempTurnTime & 0xff),
                        commitUp.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        } else {
            var keepScope = commitUp.createDelegate(this, [ true ]);
            keepScope();
        }
    },
    handleStepDown: function () {
        var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);

        var finalizeStepDown = function (val) {
            this.angle = this.finsAngle.CLOSED;
            var newPos = val - 0x0800;
            if (newPos < 0) {
                newPos = 0;
            }

            this.setConfig(
                true, 64, 0x03, ((newPos >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        64, 0x02, (newPos & 0xff),
                        this.genericCallback.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        };

        var maskedWait = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }

            if (this.angle === this.finsAngle.CENTER) {
                this.readValue.defer(
                    Math.round(this.tempTurnTime / 2.0) + 300,
                    this,
                    [ true, 2, finalizeStepDown.createDelegate(this) ]
                );
            } else if (this.angle === this.finsAngle.OPEN) {
                this.readValue.defer(
                    this.tempTurnTime + 300,
                    this,
                    [ true, 2, finalizeStepDown.createDelegate(this) ]
                );
            } else {
                this.readValue(true, 2, finalizeStepDown.createDelegate(this));
            }
        };

        var commitDown = function (ok) {
            if (!ok) {
                this.unmask2();
                this.main.el.mask();
                return;
            }

            if (this.angle !== this.finsAngle.CLOSED) {
                this.setConfig(true, 64, 0x04, 0x00, maskedWait.createDelegate(this));
            } else {
                var ks = maskedWait.createDelegate(this, [ true ]);
                ks();
            }
        };

        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        this.step2KeyMap.disable();
        this.contButton.disable();
        if (this.tempTurnTime !== t) {
            this.tempTurnTime = t;
            this.setConfig(
                true, 3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                this.setConfig.createDelegate(
                    this, [
                        3, 0x3e, (this.tempTurnTime & 0xff),
                        commitDown.createDelegate(this)
                    ], 1
                ) // append args at position 1
            );
        } else {
            var keepScope = commitDown.createDelegate(this, [ true ]);
            keepScope();
        }
    },

    unmask2: function () {
        this.main.el.unmask();
    },

    nextStep: function () {
        switch (this.currentStep) {
            case 0:
                this.currentStep++;
                this.main.step2.show();
                this.main.step1.hide();
                this.main.step2.action.turnspinner.setValue(this.tempTurnTime / 100);
                this.setTitle(dSS._('Calibration of fins turn time: step 2 of 2'));
                this.contButton.setText(dSS._('Finish'));
                this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
                this.setConfig(true, 64, 0x04, 0x00, this.delayStep4.createDelegate(this));
                break;
            case 1:
                this.contButton.disable();
                this.cancelButton.disable();
                this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');

                var t = Math.round(this.main.step2.action.turnspinner.getValue() * 100);
                if (this.tempTurnTime !== t) {
                    this.tempTurnTime = t;
                    this.setConfig(
                        true, 3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                        this.setConfig.createDelegate(this, [
                            3, 0x3e, (this.tempTurnTime & 0xff),
                            this.finishWizard.createDelegate(this)
                        ], 1) // append args at position 1
                    );
                } else {
                    var finishWizard = this.finishWizard.createDelegate(this);
                    finishWizard(true);
                }
                break;
            default:
                this.closed = true;
                this.close();
                break;
        }
    },

    finishWizard: function (ok) {
        if (ok) {
            this.setConfig(true, 3, 0x00, 33, this.closeWizard.createDelegate(this));
        } else {
            var keepScope = this.closeWizard.createDelegate(this);
            keepScope();
        }
    },

    cancelWizard: function () {
        this.contButton.disable();
        this.main.el.mask(dSS._('Please wait') + '...', 'x-mask-loading');
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
        this.setConfig(true, 3, 0x3f, ((this.loaded.turntime >> 8) & 0xff),
            this.setConfig.createDelegate(this, [
                true, // close for sure
                3, 0x3e, (this.loaded.turntime & 0xff),
                this.closeWizard.createDelegate(this)
            ]));
    },

    closeWizard: function () {
        this.closed = true;
        this.close();
    },

    setConfig: function (ok, cls, index, value, callback) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.tId = dSS.util.genericRequest({
            url: '/json/device/setConfig',
            params: {
                class: cls,
                index: index,
                dsuid: this.dSUID,
                value: value
            },
            failureMsg: dSS._('Could not write device configuration'),
            scope: this
        }, callback);
    },

    // direction 0: read until position <=0x8100
    // direction 1: read until position >=0x9F00
    // direction 2: read position and do callback callback(value)
    readValue: function (ok, direction, callback) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.tId = Ext.Ajax.request({
            url: '/json/device/getConfigWord',
            method: 'GET',
            params: {
                class: 64,
                index: 0x06,
                dsuid: this.dSUID
            },
            scope: this,
            success: function (response) {
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (!jsonData || !jsonData.ok) {
                        Ext.Msg.alert(
                            dSS._('Error'),
                            dSS._('Could not load current fin position'), ': ' + dSS._(jsonData.message)
                        );
                        this.unmask2();
                        this.main.el.mask();
                        return;
                    }

                    if (!isNaN(jsonData.result.value)) {
                        if (direction === 1) {
                            if (jsonData.result.value >= this.BOUNDARY_1) {
                                this.fireEvent('position1reached');
                                return;
                            }
                        } else if (direction === 0) {
                            if (jsonData.result.value <= this.BOUNDARY_2) {
                                this.fireEvent('position2reached');
                                this.tempStartPos = jsonData.result.value;
                                return;
                            }
                        } else if (callback !== undefined) {
                            callback(jsonData.result.value);
                            return;
                        }
                        this.readValue.defer(500, this, [ true, direction ]);
                    } else {
                        throw dSS._('received invalid value');
                    }
                } catch (err) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not load current fin position') + ': ' + err
                    );
                    this.main.el.mask();
                }
            },
            failure: function (response) {
                if (this.closed) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                        response.responseText
                    );
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not load current fin position') + ': ' + dSS._(jsonData.message)
                    );
                } catch (err) {
                    Ext.Msg.alert(
                        dSS._('Error'),
                        dSS._('Could not load current fin position') + ': ' + err
                    );
                }
                this.main.el.mask();
            }
        });
    },

    delayStep4: function (ok) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.configStep4.defer(this.tempTurnTime + 300, this);
    },

    configStep4: function () {
        if (this.closed) {
            return;
        }

        var newPos = this.tempStartPos - 0x0800;
        if (newPos < 0) {
            newPos = 0;
        }

        this.setConfig(
            true, 64, 0x03, ((newPos >> 8) & 0xff),
            this.setConfig.createDelegate(this, [
                64, 0x02, (newPos & 0xff),
                this.setConfig.createDelegate(this, [
                    3, 0x3f, ((this.tempTurnTime >> 8) & 0xff),
                    this.setConfig.createDelegate(this, [
                        3, 0x3e, (this.tempTurnTime & 0xff),
                        this.configStep4Done.createDelegate(this)
                    ], 1) // append args at position 1
                ], 1) // append args at position 1
            ], 1) // append args at position 1
        );
    },

    configStep4Done: function (ok) {
        if (!ok) {
            this.unmask2();
            this.main.el.mask();
            return;
        }

        if (this.closed) {
            return;
        }

        this.main.step2.action.turnspinner.setValue(this.tempTurnTime / 100);
        this.step2KeyMap.enable();
        this.unmask2();
    }
});

Ext.reg('dssdevfinscalibrationwizard', dSS.devprops.FinsCalibrationWizard);
