/* globals Ext, dSS */

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.GRKL300ShadePropertiesAdvancedForm = Ext.extend(dSS.devprops.BaseShadePropertiesForm, {
    constructor: function (config) {
        if (config.dSUID == null) {
            Ext.MessageBox.alert(dSS._('Error'), ('Missing dSUID'));
            return;
        }

        if (config.matrix == null) {
            Ext.Msg.alert(dSS._('Error'), ('Missing matrix'));
            return;
        }

        var me = this;
        this.dSUID = config.dSUID;

        this.addEvents('formsubmitted', 'formloaded', 'sharedata', 'formretry', 'formchanged'); // TODO: verify after formchangd -> formchanged, see also heating form!!!

        this.motionTimeAndDirectionCalibrationStatus = null;
        this.directionSwapped = false;
        this.columnWidths = [ 260, 170, 20 ];

        var directionOfTravelLabel = new Ext.form.Label({
            text: dSS._('Direction of travel') + ':'
        });
        this.directionOfTravel = new Ext.form.ComboBox({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_DirectionOfTravelCombo',

            name: 'direction',
            typeAhead: false,
            lazyRender: true,
            mode: 'local',
            forceSelection: true,
            editable: false,
            triggerAction: 'all',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: [
                    [ 0, dSS._('default') ],
                    [ 1, dSS._('reverse') ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        var motionTimeOpenCloseLabel = new Ext.form.Label({
            text: dSS._('Motion time open-close') + ':'
        });
        this.motionTimeOpenClose = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_OpenCloseNumberField',

            bubbleEvents: [ 'change' ],
            name: 'openclose',
            allowBlank: false,
            allowDecimals: true,
            forceDecimalPrecision: true,
            allowNegative: false,
            maxValue: 655
        });
        var motionTimeOpenCloseUnitLabel = new Ext.form.Label({
            text: dSS._('s')
        });

        var motionTimeCloseOpenLabel = new Ext.form.Label({
            text: dSS._('Motion time close-open') + ':'
        });
        this.motionTimeCloseOpen = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_CloseOpenNumberField',

            name: 'closeopen',
            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: true,
            forceDecimalPrecision: true,
            allowNegative: false,
            maxValue: 655
        });
        var motionTimeCloseOpenUnitLabel = new Ext.form.Label({
            text: dSS._('s')
        });

        this.finsTurnTimeLabel = new Ext.form.Label({
            text: dSS._('Fins turn time') + ':',
            disabled: true
        });
        this.finsTurnTime = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_TurnTimeFinsNumberField',

            bubbleEvents: [ 'change' ],
            name: 'fins',
            allowBlank: false,
            allowDecimals: true,
            forceDecimalPrecision: true,
            allowNegative: false,
            disabled: true,
            maxValue: 655
        });
        this.finsTurnTimeUnitLabel = new Ext.form.Label({
            text: dSS._('s'),
            disabled: true
        });

        var reverseTimeOnDirectionChangeLabel = new Ext.form.Label({
            text: dSS._('Reverse time on direction change') + ':'
        });
        this.reverseTimeOnDirectionChange = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_ReverseTimeOnDirectionChangeNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            minValue: 500,
            maxValue: 2550,
            listeners: {
                render: function (c) {
                    me.reverseTimeOnDirectionChangeTip = new Ext.ToolTip({
                        target: c.getEl(),
                        html: String.format(dSS._('Value will be rounded to a multiple of {0}'), '10 ' + dSS._('ms'))
                    });
                }
            }
        });
        var reverseTimeOnDirectionChangeUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var deadTimeWhenStartingToMoveLabel = new Ext.form.Label({
            text: dSS._('Dead time when starting to move') + ':'
        });
        this.deadTimeWhenStartingToMove = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_DeadTimeWhenStartingToMoveNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            minValue: 0,
            maxValue: 2550,
            listeners: {
                render: function (c) {
                    me.deadTimeWhenStartingToMoveTip = new Ext.ToolTip({
                        target: c.getEl(),
                        html: String.format(dSS._('Value will be rounded to a multiple of {0}'), '10 ' + dSS._('ms'))
                    });
                }
            }
        });
        var deadTimeWhenStartingToMoveUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var followUpTimeAtEndPositionLabel = new Ext.form.Label({
            text: dSS._('Follow-up time at end position') + ':'
        });
        this.followUpTimeAtEndPosition = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_FollowUpTimeAtEndPositionNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 25500, // ms, 0xff * 0.1s
            listeners: {
                render: function (c) {
                    me.followUpTimeAtEndPositionTip = new Ext.ToolTip({
                        target: c.getEl(),
                        html: String.format(dSS._('Value will be rounded to a multiple of {0}'), '100 ' + dSS._('ms'))
                    });
                }
            }
        });
        var followUpTimeAtEndPositionUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var thresholdOfEndSwitchRecognitionLabel = new Ext.form.Label({
            text: dSS._('Threshold of end switch recognition') + ':'
        });
        this.thresholdOfEndSwitchRecognition = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_ThresholdOfEndSwitchRecognitionNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 14790, // 0xff * 58
            listeners: {
                render: function (c) {
                    me.thresholdOfEndSwitchRecognitionTip = new Ext.ToolTip({
                        target: c.getEl(),
                        html: String.format(dSS._('Value will be rounded to a multiple of {0}'), '58 ' + dSS._('mA'))
                    });
                }
            }
        });
        var thresholdOfEndSwitchRecognitionUnitLabel = new Ext.form.Label({
            text: dSS._('mA')
        });

        var numberOfStepsForFineTuningAdjustmentLabel = new Ext.form.Label({
            text: dSS._('Number of steps for fine tuning adjustment') + ':'
        });
        this.numberOfStepsForFineTuningAdjustment = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_NumberOfStepsForFineTuningNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            maxValue: 0xff
        });

        var motionTimeForFineTuningAdjustmentLabel = new Ext.form.Label({
            text: dSS._('Motion time for fine tuning adjustment') + ':'
        });
        this.motionTimeForFineTuningAdjustment = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_MotionTimeForFineTuningNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            minValue: 100,
            maxValue: 2550,
            listeners: {
                render: function (c) {
                    me.motionTimeForFineTuningAdjustmentTip = new Ext.ToolTip({
                        target: c.getEl(),
                        html: String.format(dSS._('Value will be rounded to a multiple of {0}'), '10 ' + dSS._('ms'))
                    });
                }
            }
        });
        var motionTimeForFineTuningAdjustmentUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var pauseTimeForFineTuningAdjustmentLabel = new Ext.form.Label({
            text: dSS._('Pause time for fine tuning adjustment') + ':'
        });
        this.pauseTimeForFineTuningAdjustment = new Ext.form.NumberField({
            cls: 'dsClsGRKL300ShadePropertiesAdvancedForm_PauseTimeForFineTuningNumberField',

            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: false,
            allowNegative: false,
            minValue: 200,
            maxValue: 2550,
            listeners: {
                render: function (c) {
                    me.pauseTimeForFineTuningAdjustmentTip = new Ext.ToolTip({
                        target: c.getEl(),
                        html: String.format(dSS._('Value will be rounded to a multiple of {0}'), '10 ' + dSS._('ms'))
                    });
                }
            }
        });
        var pauseTimeForFineTuningAdjustmentUnitLabel = new Ext.form.Label({
            text: dSS._('ms')
        });

        var items = [
            this.makeRow([
                directionOfTravelLabel,
                this.directionOfTravel,
                null
            ]),
            this.makeRow([
                motionTimeOpenCloseLabel,
                this.motionTimeOpenClose,
                motionTimeOpenCloseUnitLabel
            ]),
            this.makeRow([
                motionTimeCloseOpenLabel,
                this.motionTimeCloseOpen,
                motionTimeCloseOpenUnitLabel
            ]),
            this.makeRow([
                this.finsTurnTimeLabel,
                this.finsTurnTime,
                this.finsTurnTimeUnitLabel
            ]),
            this.makeRow([
                reverseTimeOnDirectionChangeLabel,
                this.reverseTimeOnDirectionChange,
                reverseTimeOnDirectionChangeUnitLabel
            ]),
            this.makeRow([
                deadTimeWhenStartingToMoveLabel,
                this.deadTimeWhenStartingToMove,
                deadTimeWhenStartingToMoveUnitLabel
            ]),
            this.makeRow([
                followUpTimeAtEndPositionLabel,
                this.followUpTimeAtEndPosition,
                followUpTimeAtEndPositionUnitLabel
            ]),
            this.makeRow([
                thresholdOfEndSwitchRecognitionLabel,
                this.thresholdOfEndSwitchRecognition,
                thresholdOfEndSwitchRecognitionUnitLabel
            ]),
            this.makeRow([
                numberOfStepsForFineTuningAdjustmentLabel,
                this.numberOfStepsForFineTuningAdjustment,
                null
            ]),
            this.makeRow([
                motionTimeForFineTuningAdjustmentLabel,
                this.motionTimeForFineTuningAdjustment,
                motionTimeForFineTuningAdjustmentUnitLabel
            ]),
            this.makeRow([
                pauseTimeForFineTuningAdjustmentLabel,
                this.pauseTimeForFineTuningAdjustment,
                pauseTimeForFineTuningAdjustmentUnitLabel
            ])
        ];

        Ext.apply(this, {
            cls: 'dsClsGRKL300PropertiesAdvancedForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'sharedata', 'formretry', 'formchanged' ],
            padding: 5,
            scope: this,
            border: false,
            items: items
        });

        this.resetLoaded();

        dSS.devprops.GRKL300ShadePropertiesAdvancedForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.directionOfTravel.on('select', function () {
                this.fireEvent('formchanged');
            }, this);

            this.on('change', function () {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            var val;
            var ok = data && data.json && data.json.ok;
            if (ok) {
                val = data.json.result && data.json.result.value;
            }

            switch (data.loader) {
                case dSS.devprops.Loaders.ATTACHED_TERMINAL_DEVICE:
                    this.loaded.attachedTerminalDevice = true;
                    if (data.json.data) {
                        var deviceHasFins = [
                            dSS.config.AttachedShadeTerminalDevice.EXTERNAL_BLINDS,
                            dSS.config.AttachedShadeTerminalDevice.INTERNAL_BLINDS
                        ].indexOf(data.json.data.attributes.shadeAttributes.shadeType) !== -1;
                        if (deviceHasFins) {
                            this.finsTurnTimeLabel.enable();
                            this.finsTurnTime.enable();
                            this.finsTurnTimeUnitLabel.enable();
                        }
                    }
                    break;
                case dSS.devprops.Loaders.CALIBRATION_STATUS:
                    this.loaded.motionTimeAndDirectionCalibrationStatus = true;
                    if (ok) {
                        this.motionTimeAndDirectionCalibrationStatus = dSS.devprops.ShadeCalibration.getMotionAndDirectionCalibrationStatus(val);  // lower nibble
                    } else {
                        this.motionTimeAndDirectionCalibrationStatus = dSS.devprops.ShadeCalibration.motionAndDirectionStatus.NEVER_CALIBRATED;
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH:
                    this.loaded.directionOfTravel = true;
                    if (ok) {
                        if ((val & 0x000000ff) > 0) {
                            this.directionOfTravel.setValue(1);
                            if (!this.directionSwapped) {
                                // swap fields, #1972
                                this.directionSwapped = true;
                                var oc = this.items.get(1);
                                var co = this.items.get(2);
                                this.items.replace(this.items.getKey(oc), co);
                                this.items.replace(this.items.getKey(co), oc);
                                this.doLayout();
                            }
                        } else {
                            this.directionOfTravel.setValue(0);
                        }
                    } else {
                        this.directionOfTravel.markInvalid();
                    }

                    this.loaded.reverseTimeOnDirectionChange = true;
                    if (ok) {
                        var rt = (val & 0x0000ff00) >> 8;
                        this.reverseTimeOnDirectionChange.setValue(rt * 10);
                        dSS.util.ResetDirtyField(this.reverseTimeOnDirectionChange);
                    } else {
                        this.reverseTimeOnDirectionChange.markInvalid();
                    }

                    break;
                case dSS.devprops.Loaders.SHADE_DEAD_TIME:
                    this.loaded.deadTimeWhenStartingToMove = true;
                    if (ok) {
                        this.deadTimeWhenStartingToMove.setValue(val * 10);
                        dSS.util.ResetDirtyField(this.deadTimeWhenStartingToMove);
                    } else {
                        this.deadTimeWhenStartingToMove.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_OPEN_CLOSE:
                    this.loaded.motionTimeOpenClose = true;
                    if (ok) {
                        this.motionTimeOpenClose.setValue(val * 10 / 1000);
                        dSS.util.ResetDirtyField(this.motionTimeOpenClose);
                    } else {
                        this.motionTimeOpenClose.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_CLOSE_OPEN:
                    this.loaded.motionTimeCloseOpen = true;
                    if (ok) {
                        this.motionTimeCloseOpen.setValue(val * 10 / 1000);
                        dSS.util.ResetDirtyField(this.motionTimeCloseOpen);
                    } else {
                        this.motionTimeCloseOpen.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_MTIME_FINS:
                    this.loaded.finsTurnTime = true;
                    if (ok) {
                        // this.turnTime = val;
                        this.finsTurnTime.setValue(val * 10 / 1000);
                        dSS.util.ResetDirtyField(this.finsTurnTime);
                    } else {
                        this.finsTurnTime.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW:
                    this.loaded.followUpTimeAtEndPosition = true;
                    this.loaded.thresholdOfEndSwitchRecognition = true;
                    if (ok) {
                        var followUp = val & 0x000000ff;
                        var threshold = (val & 0x0000ff00) >> 8;
                        this.followUpTimeAtEndPosition.setValue(followUp * 100);
                        dSS.util.ResetDirtyField(this.followUpTimeAtEndPosition);
                        this.thresholdOfEndSwitchRecognition.setValue(threshold * 58);
                        dSS.util.ResetDirtyField(this.thresholdOfEndSwitchRecognition);
                    } else {
                        this.followUpTimeAtEndPosition.markInvalid();
                        this.thresholdOfEndSwitchRecognition.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE:
                    this.loaded.numberOfStepsForFineTuningAdjustment = true;
                    this.loaded.motionTimeForFineTuningAdjustment = true;
                    if (ok) {
                        var steps = val & 0x000000ff;
                        var mtime = (val & 0x0000ff00) >> 8;
                        this.numberOfStepsForFineTuningAdjustment.setValue(steps);
                        dSS.util.ResetDirtyField(this.numberOfStepsForFineTuningAdjustment);
                        this.motionTimeForFineTuningAdjustment.setValue(mtime * 10);
                        dSS.util.ResetDirtyField(this.motionTimeForFineTuningAdjustment);
                    } else {
                        this.numberOfStepsForFineTuningAdjustment.markInvalid();
                        this.motionTimeForFineTuningAdjustment.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_PTIME_FTUNE:
                    this.loaded.pauseTimeForFineTuningAdjustment = true;
                    if (ok) {
                        this.pauseTimeForFineTuningAdjustment.setValue(val * 10);
                        dSS.util.ResetDirtyField(this.pauseTimeForFineTuningAdjustment);
                    } else {
                        this.pauseTimeForFineTuningAdjustment.markInvalid();
                    }
                    break;
                default:
                    return;
            }

            if (this.isAllLoaded()) {
                this.fireEvent('formloaded', true);
                this.resetLoaded();
            }
        }, this);

        this.on('beforedestroy', function () {
            this.closed = true;
        }, this);


        // block calibration in Settings tab if there are unsaved changes in Advanced tab (same for other fields):
        this.directionOfTravel.on('change', function () {
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.motionTimeCloseOpen.on('change', function () {
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.motionTimeOpenClose.on('change', function () {
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.finsTurnTime.on('change', function () {
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.reverseTimeOnDirectionChange.on('change', function () {
            if (this.reverseTimeOnDirectionChange.value > this.reverseTimeOnDirectionChange.maxValue ||
                this.reverseTimeOnDirectionChange.value < this.reverseTimeOnDirectionChange.minValue) {
                me.reverseTimeOnDirectionChangeTip.disable();
            } else {
                me.reverseTimeOnDirectionChangeTip.enable();
            }
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.deadTimeWhenStartingToMove.on('change', function () {
            if (this.deadTimeWhenStartingToMove.value > this.deadTimeWhenStartingToMove.maxValue ||
                this.deadTimeWhenStartingToMove.value < this.deadTimeWhenStartingToMove.minValue) {
                me.deadTimeWhenStartingToMoveTip.disable();
            } else {
                me.deadTimeWhenStartingToMoveTip.enable();
            }
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.followUpTimeAtEndPosition.on('change', function () {
            if (this.followUpTimeAtEndPosition.value > this.followUpTimeAtEndPosition.maxValue) {
                me.followUpTimeAtEndPositionTip.disable();
            } else {
                me.followUpTimeAtEndPositionTip.enable();
            }
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.thresholdOfEndSwitchRecognition.on('change', function () {
            if (this.thresholdOfEndSwitchRecognition.value > this.thresholdOfEndSwitchRecognition.maxValue) {
                me.thresholdOfEndSwitchRecognitionTip.disable();
            } else {
                me.thresholdOfEndSwitchRecognitionTip.enable();
            }
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.numberOfStepsForFineTuningAdjustment.on('change', function () {
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.motionTimeForFineTuningAdjustment.on('change', function () {
            if (
                this.motionTimeForFineTuningAdjustment.value > this.motionTimeForFineTuningAdjustment.maxValue ||
                this.motionTimeForFineTuningAdjustment.value < this.motionTimeForFineTuningAdjustment.minValue
            ) {
                me.motionTimeForFineTuningAdjustmentTip.disable();
            } else {
                me.motionTimeForFineTuningAdjustmentTip.enable();
            }
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.pauseTimeForFineTuningAdjustment.on('change', function () {
            if (
                this.pauseTimeForFineTuningAdjustment.value > this.pauseTimeForFineTuningAdjustment.maxValue ||
                this.pauseTimeForFineTuningAdjustment.value < this.pauseTimeForFineTuningAdjustment.minValue
            ) {
                me.pauseTimeForFineTuningAdjustmentTip.disable();
            } else {
                me.pauseTimeForFineTuningAdjustmentTip.enable();
            }
            this.fireEvent('sharedata', 'advancedvalueschanged', this.getForm().isDirty());
        }, this);

        this.on('submit', function () {
            if (!this.getForm().isValid()) {
                Ext.Msg.alert(
                    dSS._('Error'),
                    dSS._('Please correct values in the red marked fields and try again')
                );
                this.fireEvent('formretry');
                return;
            }

            var save = [];
            var val;
            var dec;

            if (this.directionOfTravel.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x36,
                        dsuid: this.dSUID,
                        value: this.directionOfTravel.getValue()
                    },
                    failureMsg: dSS._('Could not save direction of travel settings')
                });
            }

            if (this.motionTimeOpenClose.isDirty()) {
                val = Math.floor(this.motionTimeOpenClose.getValue() * 1000 / 10);
                dec = dSS.util.decode16(val);
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x32,
                        dsuid: this.dSUID,
                        value: dec.low
                    },
                    failureMsg: dSS._('Could not save motion time open-close settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x33,
                        dsuid: this.dSUID,
                        value: dec.high
                    },
                    failureMsg: dSS._('Could not save motion time open-close settings')
                });
            }

            if (this.motionTimeCloseOpen.isDirty()) {
                val = Math.floor(this.motionTimeCloseOpen.getValue() * 1000 / 10);
                dec = dSS.util.decode16(val);
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x30,
                        dsuid: this.dSUID,
                        value: dec.low
                    },
                    failureMsg: dSS._('Could not save motion time close-open settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x31,
                        dsuid: this.dSUID,
                        value: dec.high
                    },
                    failureMsg: dSS._('Could not save motion time close-open settings')
                });
            }

            if (this.finsTurnTime.isDirty()) { // can only be dirty if input is enabled
                val = Math.floor(this.finsTurnTime.getValue() * 1000 / 10);
                dec = dSS.util.decode16(val);

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 1,
                        index: 0x5c,
                        dsuid: this.dSUID,
                        value: 0b01100000 + this.motionTimeAndDirectionCalibrationStatus
                    },
                    failureMsg: dSS._('Could not save motion time fins settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3f,
                        dsuid: this.dSUID,
                        value: dec.high
                    },
                    failureMsg: dSS._('Could not save motion time fins settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3e,
                        dsuid: this.dSUID,
                        value: dec.low
                    },
                    failureMsg: dSS._('Could not save motion time fins settings')
                });

                save.push({
                    url: '/json/device/setConfig',      // set turn time calibration status to "manually calibrated"
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x00,
                        dsuid: this.dSUID,
                        value: 33
                    },
                    failureMsg: dSS._('Could not change calibration status')
                });
            }

            if (this.reverseTimeOnDirectionChange.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x37,
                        dsuid: this.dSUID,
                        value: Math.floor(this.reverseTimeOnDirectionChange.getValue() / 10)
                    },
                    failureMsg: dSS._('Could not save reverse time on direction change settings')
                });
            }

            if (this.deadTimeWhenStartingToMove.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x34,
                        dsuid: this.dSUID,
                        value: Math.floor(this.deadTimeWhenStartingToMove.getValue() / 10)
                    },
                    failureMsg: dSS._('Could not save reverse time on direction change settings')
                });
            }

            if (this.followUpTimeAtEndPosition.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x38,
                        dsuid: this.dSUID,
                        value: Math.floor(this.followUpTimeAtEndPosition.getValue() / 100)
                    },
                    failureMsg: dSS._('Could not save follow-up time at end position settings')
                });
            }

            if (this.thresholdOfEndSwitchRecognition.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x39,
                        dsuid: this.dSUID,
                        value: Math.floor(this.thresholdOfEndSwitchRecognition.getValue() / 58)
                    },
                    failureMsg: dSS._('Could not save threshold of end switch recognition settings')
                });
            }

            if (this.numberOfStepsForFineTuningAdjustment.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3a,
                        dsuid: this.dSUID,
                        value: this.numberOfStepsForFineTuningAdjustment.getValue()
                    },
                    failureMsg: dSS._('Could not save number of steps for fine tuning adjustment settings')
                });
            }

            if (this.motionTimeForFineTuningAdjustment.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3b,
                        dsuid: this.dSUID,
                        value: Math.floor(this.motionTimeForFineTuningAdjustment.getValue() / 10)
                    },
                    failureMsg: dSS._('Could not save motion time for fine tuning adjustment settings')
                });
            }

            if (this.pauseTimeForFineTuningAdjustment.isDirty()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3c,
                        dsuid: this.dSUID,
                        value: Math.floor(this.pauseTimeForFineTuningAdjustment.getValue() / 10)
                    },
                    failureMsg: dSS._('Could not save pause time for fine tuning adjustment settings')
                });
            }

            this.saveOK = true;
            var call = this.saveData.createDelegate(this, [ save, true ]);
            call();
        }, this);
    },

    resetLoaded: function () {
        this.loaded = {
            motionTimeAndDirectionCalibrationStatus: false,
            attachedTerminalDevice: false,
            directionOfTravel: false,
            motionTimeOpenClose: false,
            motionTimeCloseOpen: false,
            finsTurnTime: false,
            reverseTimeOnDirectionChange: false,
            deadTimeWhenStartingToMove: false,
            followUpTimeAtEndPosition: false,
            thresholdOfEndSwitchRecognition: false,
            numberOfStepsForFineTuningAdjustment: false,
            motionTimeForFineTuningAdjustment: false,
            pauseTimeForFineTuningAdjustment: false
        };
    },

    saveDataCallback: function (records, state) {
        var call = this.saveData.createDelegate(this, [ records, state ]);
        call();
    },

    saveData: function (records, ok) {
        if (!ok) {
            this.saveOK = false;
        }
        if (!records.length) {
            if (!this.saveOK) {
                dSS.MessageBus.fireEvent(
                    'statusmessage',
                    dSS._('Error') + ': ' + dSS._('Some advanced properties could not be saved')
                );
            }
            this.fireEvent('formsubmitted');
            return;
        }

        var record = records.shift();

        var cb = function (state) {
            var call = this.saveDataCallback.createDelegate(this, [ records, state ]);
            call();
        };
        dSS.util.genericRequest(record, cb.createDelegate(this));
    }
});

Ext.reg('grkl300shadepropsadvancedform', dSS.devprops.GRKL300ShadePropertiesAdvancedForm);
