/* globals dSS, Ext */

Ext.namespace('dSS', 'dSS.config', 'dSS.devprops', 'dSS.gthelper');
// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID

dSS.devprops.ShadePropertiesForm = Ext.extend(dSS.devprops.BaseShadePropertiesForm, {
    constructor: function (config) {
        if (config.dSUID == null) {
            Ext.MessageBox.alert(dSS._('Error'), ('Missing dSUID'));
            return;
        }

        if (config.matrix == null) {
            Ext.Msg.alert(dSS._('Error'), ('Missing matrix'));
            return;
        }

        var that = this;

        this.dSUID = config.dSUID;
        this.matrix = config.matrix;

        this.finsResetted = false;
        this.finsManual = false;
        this.directionSwapped = false;

        this.buttonWidth = 80;
        this.columnWidths = [ 265, 170, 20 ];

        this.addEvents('formsubmitted', 'formloaded', 'forcereload', 'sharedevent', 'formretry', 'formchanged');

        this.finsTip = new Ext.ToolTip({
            cls: 'dsClsShadePropertiesForm_FinsToolTip',

            html: dSS._('Changes must be saved first.'),
            title: dSS._('Notice'),
            closable: true,
            draggable: true,
            dismissDelay: 4000,
            anchor: 'bottom'
        });

        this.directionTip = new Ext.ToolTip({
            cls: 'dsClsShadePropertiesForm_DirectionsToolTip',

            html: dSS._('Values were adjusted according to the direction of travel settings.'),
            title: dSS._('Notice'),
            closable: true,
            draggable: true,
            dismissDelay: 4000,
            anchor: 'bottom'
        });

        var motionTimeOpenCloseLabel = new Ext.form.Label({
            text: dSS._('Motion time open-close') + ':'
        });
        this.motionTimeOpenClose = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesForm_OpenCloseNumberField',

            bubbleEvents: [ 'change' ],
            name: 'openclose',
            allowBlank: false,
            allowDecimals: true,
            forceDecimalPrecision: true,
            allowNegative: false,
            maxValue: 655
        });
        var motionTimeOpenCloseUnitLabel = new Ext.form.Label({
            text: dSS._('s')
        });

        var motionTimeCloseOpenLabel = new Ext.form.Label({
            text: dSS._('Motion time close-open') + ':'
        });
        this.motionTimeCloseOpen = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesForm_CloseOpenNumberField',

            name: 'closeopen',
            bubbleEvents: [ 'change' ],
            allowBlank: false,
            allowDecimals: true,
            forceDecimalPrecision: true,
            allowNegative: false,
            maxValue: 655
        });
        var motionTimeCloseOpenUnitLabel = new Ext.form.Label({
            text: dSS._('s'),
        });

        var turnTimeFinsLabel = new Ext.form.Label({
            text: dSS._('Fins turn time') + ':',
            disabled: !this.matrix.motiontimefins
        });
        this.turnTimeFins = new Ext.form.NumberField({
            cls: 'dsClsShadePropertiesForm_TurnTimeFinsNumberField',

            bubbleEvents: [ 'change' ],
            name: 'fins',
            allowBlank: false,
            allowDecimals: true,
            forceDecimalPrecision: true,
            allowNegative: false,
            disabled: !this.matrix.motiontimefins,
            maxValue: 655
        });
        var turnTimeFinsUnitLabel = new Ext.form.Label({
            text: dSS._('s'),
            disabled: !this.matrix.motiontimefins
        });

        var directionOfTravelLabel = new Ext.form.Label({
            text: dSS._('Direction of travel') + ':'
        });
        this.directionOfTravel = new Ext.form.ComboBox({
            cls: 'dsClsShadePropertiesForm_DirectionOfTravelCombo',

            name: 'direction',
            typeAhead: false,
            lazyRender: true,
            mode: 'local',
            forceSelection: true,
            editable: false,
            triggerAction: 'all',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields: [
                    'id',
                    'displayText'
                ],
                data: [
                    [ 0, dSS._('default') ],
                    [ 1, dSS._('reverse') ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        this.protectCalibrationValues = new Ext.form.Checkbox({
            cls: 'dsClsShadePropertiesForm_protectCalibrationValuesCheckbox',
        });
        this.protectCalibrationValuesLabel = new Ext.form.Label({
            text: dSS._('Protect calibration values'),
            style: { marginTop: 5 },

            listeners: {
                afterrender: function (component) {
                    component.getEl().on('click', function () {
                        that.protectCalibrationValues.fireEvent('check', that.protectCalibrationValues, !that.protectCalibrationValues.getValue());
                    });
                }
            }
        });

        var calibrateMotionTimeAndDirectionLabel = new Ext.form.Label({
            text: dSS._('Calibrate motion time and direction')
        });
        this.calibrateMotionTimeAndDirection = new Ext.Button({
            cls: 'dsClsShadePropertiesForm_StartMotionTImeAndDirectionCalibrationButton',

            text: dSS._('Start'),
            disabled: true,
            handler: this.prepareWizard.createDelegate(this),
            minWidth: this.buttonWidth
        });

        var calibrateTurnTimeLabel = new Ext.form.Label({
            disabled: !this.matrix.motiontimefins,
            text: dSS._('Calibrate turn time')
        });
        this.calibrateTurnTime = new Ext.Button({
            cls: 'dsClsShadePropertiesForm_StartTurnTimeCalibrationButton',

            text: dSS._('Start'),
            disabled: true,
            scope: this,
            handler: this.startFinsWizard.createDelegate(this),
            minWidth: this.buttonWidth
        });
        this.calibrateTurnTimeReset = new Ext.Button({
            cls: 'dsClsShadePropertiesForm_ResetButton',

            text: dSS._('Reset'),
            disabled: true,
            handler: this.calibrateTurnTimeReset.createDelegate(this),
            minWidth: this.buttonWidth
        });

        var items = [
            this.makeRow([
                motionTimeOpenCloseLabel,
                that.motionTimeOpenClose,
                motionTimeOpenCloseUnitLabel,
            ]),
            this.makeRow([
                motionTimeCloseOpenLabel,
                that.motionTimeCloseOpen,
                motionTimeCloseOpenUnitLabel,
            ]),
            this.makeRow([
                turnTimeFinsLabel,
                that.turnTimeFins,
                turnTimeFinsUnitLabel,
            ]),
            this.makeRow([
                directionOfTravelLabel,
                that.directionOfTravel,
                null
            ]),
            this.makeRow([
                null,
                this.makeColumns([
                    that.protectCalibrationValues,
                    that.protectCalibrationValuesLabel,
                ]),
            ]),
            this.makeRow([
                calibrateMotionTimeAndDirectionLabel,
                this.makeColumns([
                    that.calibrateMotionTimeAndDirection,
                ])
            ]),
            this.makeRow([
                calibrateTurnTimeLabel,
                this.makeColumns([
                    that.calibrateTurnTime,
                    that.calibrateTurnTimeReset,
                ], 10),
            ])
        ];

        Ext.apply(this, {
            cls: 'dsClsShadePropertiesForm',

            bubbleEvents: [ 'formsubmitted', 'formloaded', 'forcereload', 'formretry', 'formchanged' ],
            padding: 5,
            scope: this,
            trackResetOnLoad: true,
            items: items
        });

        this.resetLoaded();

        dSS.devprops.ShadePropertiesForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function () {
            this.directionOfTravel.on('select', function () {
                this.fireEvent('formchanged');
            }, this);

            this.protectCalibrationValues.on('check', function () {
                this.fireEvent('formchanged');
            }, this);

            this.on('change', function () {
                this.fireEvent('formchanged');
            }, this);
        }, this);

        this.protectCalibrationValues.on('check', function (checkbox, checked) {
            checkbox.setValue(checked);
            if (checked) {
                this.calibrateMotionTimeAndDirection.disable();
                this.calibrateTurnTime.disable();
                this.calibrateTurnTimeReset.disable();
            } else {
                this.calibrateMotionTimeAndDirection.enable();
                if (this.matrix.motiontimefins) {
                    // TODO: toggling works weirdly on this button
                    if (this.finsManual && !this.finsResetted) {
                        this.calibrateTurnTime.enable();
                    }
                    this.calibrateTurnTimeReset.enable();
                }
            }
        }, this);

        this.on('dataloaded', function (data) {
            if (this.closed) {
                return;
            }

            var val;
            var ok = data && data.json && data.json.ok;
            if (ok) {
                val = data.json.result.value;
            }

            switch (data.loader) {
                case dSS.devprops.Loaders.SHADE_OPEN_CLOSE:
                    this.loaded.motionTimeOpenClose = true;
                    if (ok) {
                        this.motionTimeOpenClose.setValue(val * 10 / 1000);
                        dSS.util.ResetDirtyField(this.motionTimeOpenClose);
                    } else {
                        this.motionTimeOpenClose.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_CLOSE_OPEN:
                    this.loaded.motionTimeCloseOpen = true;
                    if (ok) {
                        this.motionTimeCloseOpen.setValue(val * 10 / 1000);
                        dSS.util.ResetDirtyField(this.motionTimeCloseOpen);
                    } else {
                        this.motionTimeCloseOpen.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_MTIME_FINS:
                    this.loaded.turnTimeFins = true;
                    if (ok) {
                        this.turnTime = val;
                        this.turnTimeFins.setValue(val * 10 / 1000);
                        dSS.util.ResetDirtyField(this.turnTimeFins);
                    } else {
                        this.turnTimeFins.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH:
                    this.loaded.directionOfTravel = true;
                    if (ok) {
                        this.directionOfTravelInitialValue = (val & 0x000000ff);
                        if (this.directionOfTravelInitialValue > 0) {
                            this.directionOfTravel.setValue(1);
                            if (!this.directionSwapped) {
                                this.directionSwapped = true;
                                // swap fields, #1972
                                var oc = this.motionTimeOpenClose.getPosition(true);
                                var co = this.motionTimeCloseOpen.getPosition(true);
                                this.motionTimeOpenClose.setPosition(co[0], co[1]);
                                this.motionTimeCloseOpen.setPosition(oc[0], oc[1]);
                            }
                        } else {
                            this.directionOfTravel.setValue(0);
                        }
                    } else {
                        this.directionOfTravel.markInvalid();
                    }
                    break;
                case dSS.devprops.Loaders.SHADE_CALIBRATION_ALLOWED:
                    this.loaded.callibrationAllowed = true;
                    if (ok) {
                        this.calibrationValue = val;
                        // 1-bit
                        if ((this.calibrationValue & 1) > 0) {
                            this.protectCalibrationValues.setValue(false);
                            this.calibrateMotionTimeAndDirection.enable();
                            if (this.matrix.motiontimefins) {
                                this.calibrateTurnTime.enable();
                                this.calibrateTurnTimeReset.enable();
                            }
                        } else {
                            this.protectCalibrationValues.setValue(true);
                        }
                    } else {
                        this.directionOfTravel.markInvalid();
                        this.calibrateMotionTimeAndDirection.disable();
                        this.calibrateTurnTime.disable();
                        this.calibrateTurnTimeReset.disable();
                    }
                    break;

                case dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE:
                    this.loaded.numStepsMTimeFinetune = true;
                    if (ok) {
                        this.numSteps = val & 0x000000ff;
                        this.mTimeAdjust = (val & 0x0000ff00) >> 8;
                    }
                    break;

                default:
                    return;
            }

            if (this.isAllLoaded()) {
                this.fireEvent('formloaded', true);
                this.resetLoaded();
            }
        }, this);

        this.on('beforedestroy', function () {
            this.closed = true;
        }, this);

        this.on('submit', function () {
            if (!this.getForm().isValid()) {
                Ext.Msg.alert(
                    dSS._('Error'),
                    dSS._('Please correct values in the red marked fields and try again')
                );
                this.fireEvent('formretry');
                return;
            }

            var save = [];
            var val;
            var dec;

            if (this.motionTimeOpenClose.isDirty()) {
                val = Math.floor(this.motionTimeOpenClose.getValue() * 1000 / 10);
                dec = dSS.util.decode16(val);
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x32,
                        dsuid: this.dSUID,
                        value: dec.low
                    },
                    failureMsg: dSS._('Could not save motion time open-close settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x33,
                        dsuid: this.dSUID,
                        value: dec.high
                    },
                    failureMsg: dSS._('Could not save motion time open-close settings')
                });
            }

            if (this.motionTimeCloseOpen.isDirty()) {
                val = Math.floor(this.motionTimeCloseOpen.getValue() * 1000 / 10);
                dec = dSS.util.decode16(val);
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x30,
                        dsuid: this.dSUID,
                        value: dec.low
                    },
                    failureMsg: dSS._('Could not save motion time close-open settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x31,
                        dsuid: this.dSUID,
                        value: dec.high
                    },
                    failureMsg: dSS._('Could not save motion time close-open settings')
                });
            }

            if (this.matrix.motiontimefins && this.turnTimeFins.isDirty()) {
                val = Math.floor(this.turnTimeFins.getValue() * 1000 / 10);
                dec = dSS.util.decode16(val);
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3e,
                        dsuid: this.dSUID,
                        value: dec.low
                    },
                    failureMsg: dSS._('Could not save motion time fins settings')
                });

                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3f,
                        dsuid: this.dSUID,
                        value: dec.high
                    },
                    failureMsg: dSS._('Could not save motion time fins settings')
                });
            }

            if (this.directionOfTravelInitialValue != this.directionOfTravel.getValue()) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x36,
                        dsuid: this.dSUID,
                        value: this.directionOfTravel.getValue()
                    },
                    failureMsg: dSS._('Could not save direction of travel settings')
                });
            }

            var d = this.calibrationValue;

            if (this.protectCalibrationValues.getValue()) {
                d &= ~1;
            } else {
                d |= 1;
            }

            if (this.calibrationValue != d) {
                save.push({
                    url: '/json/device/setConfig',
                    scope: this,
                    params: {
                        class: 3,
                        index: 0x3d,
                        dsuid: this.dSUID,
                        value: d
                    },
                    failureMsg: dSS._('Could not save calibration settings')
                });
            }

            if (this.finsResetted) {
                save.push({
                    url: '/json/device/setConfig',
                    params: {
                        class: 3,
                        index: 0,
                        dsuid: this.dSUID,
                        value: 42
                    },
                    failureMsg: dSS._('Could not save output mode settings')
                });
            }

            if (this.finsManual) {
                save.push({
                    url: '/json/device/setConfig',
                    params: {
                        class: 3,
                        index: 0,
                        dsuid: this.dSUID,
                        value: 33
                    },
                    failureMsg: dSS._('Could not save output mode settings')
                });
            }

            this.saveOK = true;
            var call = this.saveData.createDelegate(this, [ save, true ]);
            call();
        }, this);

        this.turnTimeFins && this.turnTimeFins.on('change', this.handleFinsChange.createDelegate(this), this);
        this.motionTimeOpenClose && this.motionTimeOpenClose.on('change', this.handleChange.createDelegate(this), this);
        this.motionTimeCloseOpen && this.motionTimeCloseOpen.on('change', this.handleChange.createDelegate(this), this);
        this.direction && this.directionOfTravel.on('change', this.handleDirectionChange.createDelegate(this), this);
        this.on('sharedevent', function (name) {
            if (name === 'shadepropadvancedchanged') {
                this.handleChange();
            }
        }, this);
    },

    resetLoaded () {
        this.loaded = {
            motionTimeOpenClose: false,
            motionTimeCloseOpen: false,
            turnTimeFins: !this.matrix.motiontimefins,
            directionOfTravel: false,
            callibrationAllowed: false,
            numStepsMTimeFinetune: false
        };
    },

    handleDirectionChange: function (combo, newVal, oldVal) {
        if (newVal === oldVal) {
            this.handleChange();
            return;
        }

        // swap fields, #1972
        var oc = this.motionTimeOpenClose.getPosition(true);
        var co = this.motionTimeCloseOpen.getPosition(true);
        this.motionTimeOpenClose.setPosition(co[0], co[1]);
        this.motionTimeCloseOpen.setPosition(oc[0], oc[1]);
        if (this.motionTimeOpenClose.getValue() !== this.motionTimeCloseOpen.getValue()) {
            this.showValueAdjustmentTip();
        }
        this.handleChange();
    },

    handleFinsChange: function () {
        this.finsManual = true;
        this.handleChange();
    },

    handleChange: function () {
        this.finsResetted = false;
        this.calibrateTurnTime.disable();
        this.showFinsTip.defer(2000, this);
    },

    showFinsTip: function () {
        if (this.closed) {
            return;
        }

        this.finsTip.initTarget(this.calibrateTurnTime.getId());
        this.finsTip.enable();
        this.finsTip.show();
    },

    showValueAdjustmentTip: function () {
        if (this.closed) {
            return;
        }

        var oc = this.motionTimeOpenClose.getPosition();
        var co = this.motionTimeCloseOpen.getPosition();
        if (oc[1] < co[1]) {
            this.directionTip.initTarget(this.motionTimeOpenClose.getId());
        } else {
            this.directionTip.initTarget(this.motionTimeCloseOpen.getId());
        }

        this.directionTip.enable();
        this.directionTip.show();
    },

    saveDataCallback: function (records, state) {
        var call = this.saveData.createDelegate(this, [ records, state ]);
        call();
    },

    saveData: function (records, ok) {
        if (!ok) {
            this.saveOK = false;
        }
        if (!records.length) {
            if (!this.saveOK) {
                dSS.MessageBus.fireEvent(
                    'statusmessage',
                    dSS._('Error') + ': ' + dSS._('Some properties could not be saved')
                );
            }
            this.fireEvent('formsubmitted');
            return;
        }

        var record = records.shift();

        var cb = function (state) {
            var call = this.saveDataCallback.createDelegate(this, [ records, state ]);
            call();
        };
        dSS.util.genericRequest(record, cb.createDelegate(this));
    },

    prepareWizard: function () {
        var d = this.calibrationValue;

        if (this.protectCalibrationValues.getValue()) {
            d &= ~1;
        } else {
            d |= 1;
        }

        var cb = this.startWizard.createDelegate(this);

        if (this.calibrationValue != d) {
            dSS.util.genericRequest({
                url: '/json/device/setConfig',
                scope: this,
                params: {
                    class: 3,
                    index: 0x3d,
                    dsuid: this.dSUID,
                    value: d
                },
                failureMsg: dSS._('Could not save calibration settings'),
                maskEl: this.ownerCt.el,
                maskMsg: dSS._('Loading') + '...'
            },
            function (success) {
                if (success) {
                    cb();
                }
            });
        } else {
            cb();
        }
    },

    startWizard: function () {
        var dSUID = this.dSUID;

        var reload = this.reloadData.createDelegate(this);
        var scope = this;
        var callback = function () {
            scope.on('dataloaded', function () {
                // reenable save button that is reset by the
                // reload action, #13291
                scope.fireEvent('formchanged');
            }, scope, { scope: scope, single: true });
            reload();
        };

        dSS.util.genericRequest({
            url: '/json/device/setProgMode',
            params: {
                dsuid: dSUID,
                mode: 'enable'
            },
            maskEl: this.ownerCt.el,
            maskMsg: dSS._('Loading') + '...',
            scope: this,
            failureMsg: dSS._('Could not start programming mode')
        },
        function (success) {
            if (success) {
                try {
                    var wizard = new dSS.devprops.ShadeCalibrationWizard({
                        dSUID: dSUID
                    });

                    wizard.on('close', callback);
                    wizard.show();
                } catch (error) {
                    Ext.Msg.alert(dSS._('Error'), error);
                }
            }
        });
    },

    startFinsWizard: function () {
        var dSUID = this.dSUID;

        var reload = this.reloadData.createDelegate(this);
        var scope = this;
        var callback = function () {
            scope.on('dataloaded', function () {
                // reenable save button that is reset by the
                // reload action, #13291
                scope.fireEvent('formchanged');
            }, scope, { scope: scope, single: true });
            reload();
        };

        try {
            var wizard = new dSS.devprops.FinsCalibrationWizard({
                dSUID: dSUID,
                mtimeadjust: this.mTimeAdjust,
                numsteps: this.numSteps,
                turntime: this.turnTime
            });
            wizard.on('close', callback);
            wizard.show();
        } catch (e) {
            Ext.Msg.alert(dSS._('Error'), error);
        }
    },

    reloadData: function () {
        this.finsResetted = false;
        this.finsManual = false;
        this.fireEvent('forcereload');
    },

    calibrateTurnTimeReset: function () {
        this.turnTimeFins.setValue(3);
        this.calibrateTurnTime.disable();
        this.finsResetted = true;
        this.finsManual = false;
        this.showFinsTip();
    }
});

Ext.reg('shadepropsform', dSS.devprops.ShadePropertiesForm);
