//= require <dSS/util/IdentifyDevice>

/* globals Ext, dSS, dSA, dSS11 */

Ext.namespace('dSS', 'dSS.grid', 'dSS.data', 'dSS.config', 'dSA', 'dSS11.config', 'dSS.devprops', 'dSS.util', 'dSS.util.IdentifyDevice');

Ext.override(Ext.ToolTip, {
    onTargetOver: function (e) {
        if (this.disabled || e.within(this.target.dom, true)) {
            return;
        }
        this.clearTimer('hide');
        this.triggerElement = e.getTarget();
        this.targetXY = e.getXY();
        this.delayShow();
    },

    onMouseMove: function (e) {
        this.triggerElement = e.getTarget();
        this.targetXY = e.getXY();
        if (!this.hidden && this.trackMouse) {
            this.setPagePosition(this.getTargetXY());
        }
    }
});

dSS.grid.ModulatorPanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function () {
        var dSUIDrenderer = function (value, metaData, record) {
            var shortId = record.get('DisplayID');
            if (record.get('isPresent') && !record.get('isValid')) {
                shortId = dSS._('Loading') + ' ... ' + shortId;
            }
            return String.format('<div ext:qtip="dSUID: {0}">{1}</div>', record.get('dsuid'), shortId);
        };

        var deviceRegistrationRenderer = function (value, metaData, record) {
            var busMemberType = record.get('busMemberType');
            if (
                busMemberType !== dSS.config.BusMemberTypes.dSM11 &&
                busMemberType !== dSS.config.BusMemberTypes.dSM12 &&
                busMemberType !== dSS.config.BusMemberTypes.dSM20 &&
                busMemberType !== dSS.config.BusMemberTypes.dSM25
            ) {
                return '';
            }
            if (record.get('ignoreActionsFromNewDevices')) {
                return dSS._('enabled');
            } else {
                return dSS._('disabled');
            }
        };

        var ledCodeRenderer = function (value, metaData, record) {
            var code = record.get('ledCode');
            if (code < 0) {
                return '';
            }

            var cls = dSS.config.dSM_LED_CSS[code];
            if (cls == null) {
                return '';
            }

            metaData.css = cls;
            return String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);
        };

        var firmwareRendererARM = function (value, metaData, record) {
            var busMemberType = record.get('busMemberType');
            if (
                (
                    busMemberType === dSS.config.BusMemberTypes.dSM11 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM12 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM20 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM25 ||
                    busMemberType === dSS.config.BusMemberTypes.VDC
                ) &&
                record.get('isPresent') &&
                record.get('isValid') &&
                !record.get('isUpToDate')
            ) {
                metaData.attr = 'style="color:red;"';
                return String.format('<div ext:qtip="{0}">{1}</div>', dSS._('dSM firmware needs to be updated!'), record.get('swVersion'));
            }
            return record.get('swVersion');
        };


        var consumptionRenderer = function (value, metaData, record) {
            if (!record.get('hasMetering')) {
                return '';
            }
            var consumption = record.get('consumption');
            var production = record.get('production');
            if (production) {
                var cellContent = ' <span style="background:#00523b;padding:0 0.25em;color:white;border-radius:0.25em;">' + (-production) + '</span>';
                return String.format('<div ext:qtip="' + dSS._('production') + ': {0} W">{1}</div>', production, cellContent);
            } else {
                return String.format('<div ext:qtip="' + dSS._('Consumption') + ': {0} W">{0}</div>', consumption);
            }
        };

        var modulatorColumns = [];

        if (dSA.module) {
            modulatorColumns.push(
                {
                    header: dSS._('Status'),
                    tooltip: dSS._('Status'),
                    width: 20,
                    sortable: false,
                    dataIndex: 'ledCode',
                    id: 'ledCode',
                    renderer: ledCodeRenderer
                }
            );
        }

        modulatorColumns = modulatorColumns.concat([
            {
                id: 'name',
                header: dSS._('Name'),
                tooltip: dSS._('Name'),
                width: 50,
                sortable: true,
                dataIndex: 'name'
            },
            {
                id: 'dsuid',
                header: dSS._('dSM ID'),
                tooltip: dSS._('dSM ID'),
                width: 50,
                sortable: true,
                dataIndex: 'dsuid',
                renderer: dSUIDrenderer
            },
            {
                id: 'numdevices',
                header: dSS._('Number of devices'),
                tooltip: dSS._('Number of devices'),
                width: 30,
                sortable: true,
                dataIndex: 'numDevices'
            },
            {
                id: 'deviceregistration',
                header: dSS._('Lock new devices'),
                tooltip: dSS._('Lock new devices'),
                width: 30,
                sortable: true,
                dataIndex: 'ignoreActionsFromNewDevices',
                renderer: deviceRegistrationRenderer
            },
            {
                id: 'consumption',
                header: dSS._('Power (W)'),
                tooltip: dSS._('Consumption / production (as negative number)'),
                width: 15,
                sortable: true,
                dataIndex: 'consumption',
                renderer: consumptionRenderer
            },
            {
                id: 'armSwVersion',
                header: dSS._('ARM software version'),
                tooltip: dSS._('ARM software version'),
                width: 60,
                sortable: true,
                dataIndex: 'armSwVersion',
                renderer: firmwareRendererARM
            },
            {
                id: 'hwVersion',
                header: dSS._('HW version'),
                tooltip: dSS._('Hardware version'),
                width: 30,
                sortable: true,
                dataIndex: 'hwVersionString'
            },
            {
                id: 'hardwareInfo',
                header: dSS._('HW info'),
                tooltip: dSS._('Hardware info'),
                width: 50,
                sortable: true,
                dataIndex: 'hwName'
            }
        ]);

        var rowSelect = new Ext.grid.RowSelectionModel({
            singleSelect: false
        });

        this.heightConfigured = false;
        this.selectionBlocked = true;

        Ext.apply(this, {
            cls: 'dsClsModulatorPanel',

            store: dSS.data.Loader.getModulatorStore(),
            columns: modulatorColumns,
            enableDragDrop: false,
            stripeRows: true,
            title: dSS._('Meters & Controllers'),
            selModel: rowSelect,
            viewConfig: {
                forceFit: true,
                getRowClass: function (record) {
                    if (!record.get('isPresent')) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.ModulatorPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function () {
            if (!this.heightConfigured) {
                this.heightConfigured = true;
                this.setHeight(this.findParentByType('dsmoverviewpanel').getInnerHeight() / 2);
            }
        });

        this.on('viewready', function () {
            this.selectionBlocked = false;
            this.setInitialSelection();
        });

        dSS.MessageBus.on('structureloaded', function () {
            this.setInitialSelection();
        }, this);

        dSS.MessageBus.on('updatedsmdignoredevflag', function (dSUID, flag) {
            var mrec = this.getStore().getById(dSUID);
            if (mrec) {
                mrec.set('ignoreActionsFromNewDevices', flag);
                mrec.commit();
            }
        }, this);

        dSS.data.Loader.getDeviceStore().on('load', function () {
            this.setInitialSelection();
        }, this);

        this.on('rowcontextmenu', function (grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({
                    cls: 'dsClsModulatorPanel_ContextMenu'
                });
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
            }

            var showRemoveMenuItem = false;
            var showMeterSettings = false;

            var records = sel.getSelections();

            var i;
            var record;
            for (i = 0; i < records.length; i++) {
                record = records[i];

                // if at least one record can be removed - set the flag
                if (!record.get('isPresent')) {
                    showRemoveMenuItem = true;
                    break;
                }
            }
            for (i = 0; i < records.length; i++) {
                record = records[i];

                // at least one selection is a dSMxx
                var busMemberType = record.get('busMemberType');
                if (
                    busMemberType === dSS.config.BusMemberTypes.dSM11 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM12 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM20 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM25
                ) {
                    showMeterSettings = true;
                    break;
                }
            }

            var menuItem;

            var blinkDisabled = true;
            if (record.get('hasBlinking')) {
                blinkDisabled = false;
            }

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_BlinkLedItem',

                iconCls: 'blinkdSMIcon',
                text: dSS._('Blink LED'),
                disabled: blinkDisabled,
                handler: this.blinkdSM.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_TestTranmissionQualityItem',

                iconCls: 'transmissionQuality',
                text: dSS._('Test transmission quality'),
                handler: this.testTransmissionQuality.createDelegate(this, rowIndex, true),
                disabled: !showMeterSettings
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_CheckDevicePresenceItem',

                iconCls: 'checkDevicePresenceIcon',
                text: dSS._('Check device presence'),
                handler: this.checkDevicePresence.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_ReregisterAllDevicesItem',

                iconCls: 'reregisterDevicesIcon',
                text: dSS._('Reregister all devices'),
                handler: this.restartRegistration.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_IdentifyDeviceItem',

                iconCls: 'identifyDeviceToolbarIcon',
                text: dSS._('Identify device'),
                handler: this.identifyDevice.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_EditPropertiesItem',

                text: dSS._('Edit properties'),
                iconCls: 'dSMSettings',
                handler: this.dSMSettings.createDelegate(this, rowIndex, true),
                disabled: !showMeterSettings
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_RenameItem',

                text: dSS._('Rename'),
                iconCls: 'editIcon',
                handler: this.renameMeter.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_ViewCommunicationLogItem',

                iconCls: 'viewCommunicationLogIcon',
                text: dSS._('View communication log'),
                disabled: (record.get('busMemberType') === dSS.config.BusMemberTypes.VDC),
                handler: this.viewCommLog.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                cls: 'dsClsModulatorPanel_ContextMenu_RemoveDSMItem',

                text: dSS._('Remove dS-Meter'),
                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeMeter.createDelegate(this, rowIndex, true),
                disabled: !showRemoveMenuItem
            });
            this.contextMenu.add(menuItem);

            var current = sel.getSelected();
            if (current) {
                var url = String(current.get('VdcConfigURL'));
                if (url.length > 0) {
                    this.contextMenu.add('-');
                    menuItem = new Ext.menu.Item({
                        cls: 'dsClsModulatorPanel_ContextMenu_OpenVDCInterfaceItem',

                        text: dSS._('Open vdc interface'),
                        iconCls: 'openVdcInterface',
                        href: url,
                        hrefTarget: '_blank'
                    });
                    this.contextMenu.add(menuItem);
                }
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        }, this);

        rowSelect.on('rowselect', function (selModel, rowIndex, dataRecord) {
            dSS.MessageBus.fireEvent('filter_devices_circuit', dataRecord);
            this.lastSelectedDSUID = dataRecord.get('dsuid');
            this.getTopToolbar().removeMeterButton.setDisabled(dataRecord.get('isPresent'));
            var canBlink = Boolean(dataRecord.get('hasBlinking'));
            this.getTopToolbar().blinkMeterButton.setDisabled(!canBlink);
            var busMemberType = dataRecord.get('busMemberType');
            this.getTopToolbar().viewCommLogButton.setDisabled(busMemberType === dSS.config.BusMemberTypes.VDC);
            var showMeterSettings = false;
            if (
                busMemberType === dSS.config.BusMemberTypes.dSM11 ||
                busMemberType === dSS.config.BusMemberTypes.dSM12 ||
                busMemberType === dSS.config.BusMemberTypes.dSM20 ||
                busMemberType === dSS.config.BusMemberTypes.dSM25
            ) {
                showMeterSettings = true;
            }

            this.getTopToolbar().dSMSettings.setDisabled(!showMeterSettings);
            this.getTopToolbar().transmissionQuality.setDisabled(!showMeterSettings);
        }, this);

        this.on('render', function () {
            this.tip = new Ext.ToolTip({
                cls: 'dsClsModulatorPanel_Tip',

                target: this.getView().mainBody,
                trackMouse: true,
                renderTo: document.body,
                autoHide: true,
                dismissDelay: 0,
                autoWidth: true,
                listeners: {
                    beforeshow: this.updateTipBody.createDelegate(this),
                    move: this.updateTipBody.createDelegate(this)
                }
            });
        }, this);
    },
    updateTipBody: function (tip) {
        if (
            !Ext.DomQuery.is(tip.triggerElement, 'td.x-grid3-td-ledCode') &&
            !Ext.DomQuery.is(tip.triggerElement, 'div.x-grid3-col-ledCode')
        ) {
            return false;
        }
        var rowIndex = this.getView().findRowIndex(tip.triggerElement);
        tip.update(this.getStore().getAt(rowIndex).get('ledStatus'));
    },
    buildTopToolbar: function () {
        return [
            {
                cls: 'dsClsModulatorPanel_TopToolBar_BlinkLedIcon',

                iconCls: 'blinkdSMIcon',
                scope: this,
                handler: this.blinkdSM,
                ref: 'blinkMeterButton',
                tooltip: dSS._('Blink LED')
            },
            '-',
            {
                cls: 'dsClsModulatorPanel_TopToolBar_TestTransmissionQualityIcon',

                iconCls: 'transmissionQuality',
                scope: this,
                ref: 'transmissionQuality',
                handler: this.testTransmissionQuality,
                tooltip: dSS._('Test transmission quality')
            },
            {
                cls: 'dsClsModulatorPanel_TopToolBar_CheckDevicePresenceIcon',

                iconCls: 'checkDevicePresenceIcon',
                scope: this,
                handler: this.checkDevicePresence,
                tooltip: dSS._('Check device presence')
            },
            {
                cls: 'dsClsModulatorPanel_TopToolBar_ReregisterAllDevicesIcon',

                iconCls: 'reregisterDevicesIcon',
                scope: this,
                handler: this.restartRegistration,
                tooltip: dSS._('Reregister all devices')
            },
            {
                cls: 'dsClsModulatorPanel_TopToolBar_IdentifyDeviceIcon',

                iconCls: 'identifyDeviceToolbarIcon',
                scope: this,
                handler: this.identifyDevice,
                tooltip: dSS._('Identify device')
            },
            '-',
            {
                cls: 'dsClsModulatorPanel_TopToolBar_EditPropertiesIcon',

                iconCls: 'dSMSettings',
                ref: 'dSMSettings',
                scope: this,
                handler: this.dSMSettings,
                tooltip: dSS._('Edit properties')
            },
            {
                cls: 'dsClsModulatorPanel_TopToolBar_RenameIcon',

                iconCls: 'editIcon',
                scope: this,
                handler: this.renameMeter,
                tooltip: dSS._('Rename')
            },
            '->',
            {
                cls: 'dsClsModulatorPanel_TopToolBar_ViewDeviceUpdateStatusReportIcon',

                iconCls: 'viewDeviceUpdateReportIcon',
                scope: this,
                handler: this.viewDeviceUpdateTaskReport,
                tooltip: dSS._('View device update status report')
            },
            '-',
            {
                cls: 'dsClsModulatorPanel_TopToolBar_ViewCommunicationLogIcon',

                iconCls: 'viewCommunicationLogIcon',
                scope: this,
                handler: this.viewCommLog,
                ref: 'viewCommLogButton',
                tooltip: dSS._('View communication log')
            },
            '-',
            {
                cls: 'dsClsModulatorPanel_TopToolBar_RemoveDSMIcon',

                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeMeter,
                ref: 'removeMeterButton',
                tooltip: dSS._('Remove dS-Meter'),
                scope: this
            }];
    },

    renameMeter: function () {
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._('Error'), dSS._('No meter selected, please select a meter from the list!'));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: true,
            title: dSS._('Rename'),
            featureConfig: dSS.config.FeatureConfig.CIRCUIT,
            callerScope: this,
            width: 350,
            dialogType: dSS.config.dialogTypes.RENAME_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    setInitialSelection: function () {
        if (this.selectionBlocked) {
            return;
        }

        if (!this.ownerCt.isVisible()) {
            return;
        }

        if (this.getStore().getCount() === 0) {
            this.getStore().clearFilter();
            return;
        }

        var selModel = this.getSelectionModel();
        var record;
        var index;
        var selected = false;

        if (selModel.hasSelection()) {
            record = selModel.getSelected();
            index = this.getStore().indexOf(record);
            if (index !== -1) {
                selected = true;
            } else {
                record = null;
            }
        } else if (this.lastSelectedDSUID != null) {
            index = this.getStore().findExact('dsuid', this.lastSelectedDSUID);
            if (index !== -1) {
                record = this.getStore().getAt(index);
                selected = true;
            } else {
                record = null;
            }
        }

        if (!selected) {
            if (this.getView().getRow(0) != null) {
                this.getView().focusRow(0);
                selModel.selectFirstRow();
                record = this.getStore().getAt(0);
                index = 0;
            }
        }

        if (record) {
            this.lastSelectedDSUID = record.get('dsuid');
            if (selected) {
                this.getView().focusRow(index);
                selModel.selectRecords([ record ], false);
            }
        }
    },
    rescanCircuit: function (item, event, rowIndex) {
        var record = this.getStore().getAt(rowIndex);
        Ext.Ajax.request({
            url: '/json/circuit/rescan',
            disableCaching: true,
            method: 'GET',
            params: {
                dsuid: record.get('dsuid')
            },
            success: function (result) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(dSS._('Error'), dSS._('Rescan failed'));
                    }
                } catch (err) {
                    Ext.MessageBox.alert(dSS._('Error'), dSS._('Rescan failed') + ': ' + err);
                }
            },
            failure: function () {
                Ext.MessageBox.alert(dSS._('Error'), dSS._('Rescan failed'));
            }
        });
    },
    showCopyWindow: function () {
        var tsvwindow = new dSS.util.GridToSVWindow({
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 120,
            separator: '\t'
        });
        tsvwindow.show(this);
    },

    removeMeter: function () {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no meter selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._('Error'), dSS._('No meter selected, please select a meter from the list!'));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        for (var i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent')) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length) {
            this.deleteMeter(notPresent);
        } else if (skippedPresent) {
            Ext.MessageBox.alert(dSS._('Error'), dSS._('Present meters can not be removed!'));
        }
    },
    deleteMeter: function (records) {
        var store = this.getStore();
        if (!records.length) {
            this.el.unmask();
            return;
        }

        var record = records.shift();

        this.el.mask(dSS._('Deleting meter') + ': ' + record.get('dsuid'));
        Ext.Ajax.request({
            url: '/json/apartment/removeMeter',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                dsuid: record.get('dsuid')
            },
            success: function (result) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.remove(record);
                        dSS.MessageBus.fireEvent(
                            'statusmessage',
                            dSS.gt.strargs(dSS._('digitalSTROM Meter %1 was removed'), [record.get('DisplayID')])
                        );
                        this.deleteMeter(records);
                    } else {
                        this.el.unmask();
                        Ext.MessageBox.alert(dSS._('Could not remove'), dSS._(jsonData.message));
                    }
                } catch (err) {
                    this.el.unmask();
                    Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send remove meter command to dSS') + ': ' + err);
                }
            },
            failure: function () {
                this.el.unmask();
                Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send remove meter command to dSS'));
            }
        });
    },
    removeInactiveMeters: function () {
        Ext.Ajax.request({
            url: '/json/apartment/removeInactiveMeters',
            disableCaching: true,
            method: 'GET',
            scope: this,
            success: function (result) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        dSS.data.Loader.load();
                    } else {
                        Ext.MessageBox.alert(dSS._('Could not remove inactive meters'), jsonData.message);
                    }
                } catch (err) {
                    Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send remove inactive meters command to dSS') + ': ' + err);
                }
            },
            failure: function () {
                Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send remove inactive meters command to dSS'));
            }
        });
    },
    getMeter: function (rowIndex) {
        var record;

        if (rowIndex == null) {
            // we were called from the toolbar, so its possible that
            // there is no selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._('Error'), dSS._('No meter selected, please select a meter from the list!'));
                return null;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        // this should never happen
        if (record == null) {
            Ext.Msg.alert(dSS._('Error'), dSS._('dSM not found'));
            return null;
        }

        return record;
    },
    getMeter4dSA: function (rowIndex, ignoreNotPresent, notPresentMsg) {
        if (!dSA.module) {
            Ext.Msg.alert(dSS._('Notice'),
                dSS._('This feature requires presence of the digitalSTROM Assistant, the setup is only available on the dSS11 hardware'));
        } else {
            var record = this.getMeter(rowIndex);
            if (record) {
                if (!ignoreNotPresent && !record.get('isPresent')) {
                    Ext.Msg.alert(dSS._('Error'), notPresentMsg);
                    return null;
                }
            }
            return record;
        }
        return null;
    },
    restartRegistration: function (item, event, rowIndex) {
        var controller = this.getMeter(rowIndex);
        Ext.Ajax.request({
            url: '/json/circuit/reregisterDevices',
            disableCaching: true,
            method: 'GET',
            params: {
                dsuid: controller.get('dsuid')
            },
            scope: this,
            success: function (result) {
                var jsonData;
                try {
                    jsonData = Ext.util.JSON.decode(result.responseText);
                } catch (err) {
                    // nothing to handle
                }
                if (jsonData && jsonData.ok) {
                    dSS.MessageBus.fireEvent(
                        'statusmessage',
                        dSS.gt.strargs(dSS._('Reregistering devices for controller %1 started'), [ controller.get('DisplayID') ])
                    );
                    setTimeout(function () {
                        dSS.MessageBus.fireEvent('reloadapartment');
                    }, 5000);
                } else {
                    Ext.MessageBox.alert(dSS._('Error'), dSS._('Reregistering devices failed'));
                }
            },
            failure: function () {
                Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send reregister command to dSS'));
            }
        });
    },
    identifyDevice1: function () {
        if (!dSA.module) {
            Ext.Msg.alert(
                dSS._('Notice'),
                dSS._('This feature requires presence of the digitalSTROM Assistant, the setup is only available on the dSS11 hardware')
            );
            return;
        }
        dSA.Reactor.fireEvent(
            'identifydevice',
            dSS.data.Loader.getDeviceStore(),
            this.getStore(),
            this.findParentByType(Ext.Viewport).el
        );
    },
    identifyDevice2: function () {
        var webSocket;
        try {
            webSocket = new WebSocket(((window.location.protocol === 'https:') ? 'wss://' : 'ws://') + window.location.host + '/bus');
        } catch (err) {
            this.identifyDevice1();
            return;
        }
        var maintain;

        var record = this.getMeter();

        var isVDC = false;
        if (record.get('busMemberType') === dSS.config.BusMemberTypes.VDC) {
            isVDC = true;
        }

        var win = new dSS.util.IdentifyDevice.Window(
            {
                devstore: dSS.data.Loader.getDeviceStore(),
                dsmRecord: record,
                maskElement: this.findParentByType(Ext.Viewport).el,
                webSocket: webSocket,
                isVDC: isVDC
            }
        );

        var requestToSend = {
            url: 'apartment/identifyDevices',
            type: 'request',
            id: '1'
        };
        var ping = {
            url: 'apartment/identifyDevices',
            type: 'event',
            id: '2'
        };

        webSocket.onopen = function () {
            webSocket.send(JSON.stringify(requestToSend));
            win.show();
            maintain = setInterval(function () {
                webSocket.send(JSON.stringify(ping));
            }, 5000);
        };

        webSocket.onmessage = function (message) {
            var data = JSON.parse(message.data);
            win.checkResults(true, data.result);
        };

        var panel = this;
        webSocket.onerror = function () {
            panel.identifyDevice1();
        };

        webSocket.onclose = function () {
            clearInterval(maintain);
            win.close();
        };
    },
    identifyDevice: function () {
        this.identifyDevice2();
    },
    blinkdSM: function (item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._('Could not blink, dSM is not present!'));

        if (record) {
            if (record.get('hasBlinking')) {
                var busMemberType = record.get('busMemberType');
                if (
                    busMemberType === dSS.config.BusMemberTypes.dSM11 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM12 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM20 ||
                    busMemberType === dSS.config.BusMemberTypes.dSM25
                ) {
                    dSA.Reactor.fireEvent('blinkdsmled', record.get('dsuid'));
                } else {
                    Ext.Ajax.request({
                        url: '/json/circuit/identify',
                        disableCaching: true,
                        method: 'GET',
                        params: {
                            dsuid: record.get('dsuid')
                        },
                        scope: this,
                        success: function (result) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    dSS.MessageBus.fireEvent(
                                        'statusmessage',
                                        dSS.gt.strargs(dSS._('Controller %1 is identifying itself'), [ record.get('DisplayID') ])
                                    );
                                } else {
                                    Ext.MessageBox.alert(dSS._('Controller can not identify') + ': ', jsonData.message);
                                }
                            } catch (err) {
                                Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send identify command to dSS') + ': ' + err);
                            }
                        },
                        failure: function (result) {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            Ext.MessageBox.alert(dSS._('Error'), dSS._('Could not send identify command to dSS') + ': ' + jsonData.message);
                        }
                    });
                }
            } else {
                Ext.Msg.alert(
                    dSS._('Notice'),
                    dSS._('Blinking LED is not supported by this device')
                );
            }
        }
    },
    testTransmissionQuality: function (item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._('Can not start transmission quality test, dSM is not present!'));
        if (record) {
            var busMemberType = record.get('busMemberType');
            if (
                busMemberType === dSS.config.BusMemberTypes.dSM11 ||
                busMemberType === dSS.config.BusMemberTypes.dSM12 ||
                busMemberType === dSS.config.BusMemberTypes.dSM20 ||
                busMemberType === dSS.config.BusMemberTypes.dSM25
            ) {
                dSA.Reactor.fireEvent(
                    'checktransmissionquality',
                    record.get('DisplayID'), record.get('dsid'),
                    record.get('dsuid'), record.get('name'),
                    this.findParentByType(Ext.Viewport).el
                );
            }
        }
    },
    checkDevicePresence: function (item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._('Can not start device presence test, dSM is not present!'));
        if (record) {
            dSA.Reactor.fireEvent(
                'checkdevicepresence',
                record.get('DisplayID'),
                record.get('dsuid'), record.get('name'),
                this.findParentByType(Ext.Viewport).el
            );
        }
    },
    // only the dSS11 UI is capable of showing logs, so we will send a message
    // with the appropriate dSUID and the dSS11 websetup will take care of the
    // rest
    viewCommLog: function (item, event, rowIndex) {
        if (!dSS11.config.dSSIntegration) {
            Ext.Msg.alert(
                dSS._('Notice'),
                dSS._('This feature is only available in the setup on dSS11 hardware')
            );
        } else {
            var record = this.getMeter(rowIndex);
            if (record) {
                dSS.MessageBus.fireEvent('showlog', record.get('dsuid'));
            }
        }
    },
    viewDeviceUpdateTaskReport: function () {
        if (!dSS11.config.dSSIntegration) {
            Ext.Msg.alert(
                dSS._('Notice'),
                dSS._('This feature is only available in the setup on dSS11 hardware')
            );
        } else {
            dSA.Reactor.fireEvent(
                'showreport',
                dSA.config.TaskReportRequest.TYPE,
                dSA.config.TaskTypes.DEVICE_UPDATE_TASK,
                dSS._('Device update status report')
            );
        }
    },
    dSMSettings: function (item, event, rowIndex) {
        var record = this.getMeter4dSA(rowIndex, false, dSS._('dSM settings are not available because the dSM is not present!'));
        if (record) {
            var busMemberType = record.get('busMemberType');
            if (
                busMemberType === dSS.config.BusMemberTypes.dSM11 ||
                busMemberType === dSS.config.BusMemberTypes.dSM12 ||
                busMemberType === dSS.config.BusMemberTypes.dSM20 ||
                busMemberType === dSS.config.BusMemberTypes.dSM25
            ) {
                dSA.Reactor.fireEvent('dsmsettings', record, this.findParentByType(Ext.Viewport).el);
            }
        }
    }
});

Ext.reg('dssmodulatorpanel', dSS.grid.ModulatorPanel);
