//= require <external/CheckColumn>

Ext.namespace('dSS', 'dSS.grid');

dSS.grid.TokenAuthPanel = Ext.extend(Ext.grid.EditorGridPanel, {
    initComponent: function() {

        var tokenrenderer = function(data, metadata, record, rowIndex,
                                     columnIndex, store) {
            return String.format('<div ext:qtip="{0}">...{1}</div>',
                record.get('token'),
                dSS.util.shortToken(record.get('token')));
        };

        var appnamerenderer = function(data, metadata, record, rowIndex,
                                     columnIndex, store) {
            return String.format('<div ext:qtip="{0}">{1}</div>',
                record.get('application'),
                record.get('application'));
        };

        var columns = [
            {
                header: dSS._("Application"),
                tooltip: dSS._("Application"),
                dataIndex: 'application'
            },
            {
                header: dSS._("Token"),
                tooltip: dSS._("Token"),
                dataIndex: 'token',
                width: 45,
                renderer: tokenrenderer
            },
            {
                xtype: 'checkcolumn',
                header: dSS._("Enable access"),
                tooltip: dSS._("Enable access"),
                dataIndex: 'enabled',
                width: 60
            }
        ];

        var tokenStore = new Ext.data.JsonStore({
            root: 'data',
            fields: [ 
                'application',
                'token',
                { name: 'enabled', type: 'bool'}
            ]
        });

        Ext.apply(this, {
            cls: 'dsClsTokenAuthPanel',

            columns: columns,
            stripeRows: true,
            enableColumnHide: false,
            enableHdMenu: false,
            clicksToEdit: 1,
            store: tokenStore,
            viewConfig: {
                forceFit: true
            },
            buttons: {
                minButtonWidth: 80,
                items: [
                    {
                        text: dSS._("Apply"),
                        handler: this.applyChanges.createDelegate(this)
                    },
                    {
                        text: dSS._("Cancel"),
                        handler: this.reloadTokens.createDelegate(this)
                    }
                ]
            }
        });

        dSS.grid.TokenAuthPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', this.reloadTokens.createDelegate(this),
            this);

        dSS.MessageBus.on('refreshview', this.reloadTokens.createDelegate(this),
            this);
    },

    applyChanges: function() {
        var records = this.getStore().getModifiedRecords();
        var record = records.shift();
        this.handleTokens(record, records);
    },
    handleTokens: function(record, remainingRecords) {
        var maskel = this.findParentByType(Ext.Viewport).el;

        if (record === undefined) {
            maskel.unmask();
            return;
        }
        maskel.mask(dSS._("Applying changes") + ": " +
                    record.get('application'));

        var cmd = 'revokeToken';

        if (record.get('enabled') === true) {
            cmd = 'enableToken';
        }
        Ext.Ajax.request({
            url: '/json/system/' + cmd,
            params: {
                applicationToken: record.get('token')
            },
            method: 'GET',
            disableCaching: true,
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        record.commit();
                        if (cmd === 'revokeToken') {
                            this.getStore().remove(record);
                        }
                        if (remainingRecords.length < 1) {
                            maskel.unmask();
                            return;
                        }
                        var next = remainingRecords.shift();
                        this.handleTokens(next, remainingRecords);
                    } else {
                        throw jsonData.message;
                    }
                } catch (err) {
                    maskel.unmask();
                    this.getStore().rejectChanges();
                    Ext.Msg.show({
                        title: dSS._("Error"),
                        msg: dSS._("Could not send command to dSS") + ": " +
                             err,
                        buttons: Ext.Msg.OK,
                        scope: this,
                        fn: function() {
                            this.reloadTokens();
                        }
                    });
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                this.getStore().rejectChanges();
                this.reloadTokens();
                Ext.Msg.show({
                    title: dSS._("Error"),
                    msg: dSS._("Could not send command to dSS"),
                    buttons: Ext.Msg.OK,
                    scope: this,
                    fn: function() {
                        this.reloadTokens();
                    }
                });

            }
        });
    },
    reloadTokens: function() {
       Ext.Ajax.request({
           url: '/json/property/query',
           params: {
               query: '/system/security/applicationTokens/*/*(token,applicationName)'
           },
           disableCaching: true,
           method: 'GET',
           scope: this,
           success: function(result, request) {
               try {
                   var recs = [];
                   var jsonData = Ext.util.JSON.decode(result.responseText);
                   Ext.each(jsonData.result.pending, function(rec) {
                       if ((rec.applicationName !== "RemoteConnectivity") &&
                           (rec.applicationName !== "dSHub") &&
                           (rec.applicationName !== "vdc")) {
                           recs.push({
                               application: rec.applicationName,
                               token: rec.token,
                               enabled: false
                           });
                       }
                   });

                   Ext.each(jsonData.result.enabled, function(rec) {
                       if ((rec.applicationName !== "RemoteConnectivity") &&
                           (rec.applicationName !== "dSHub") &&
                           (rec.applicationName !== "vdc")) {
                           recs.push({
                               application: rec.applicationName,
                               token: rec.token,
                               enabled: true
                           });
                       }
                   });

                   this.getStore().loadData({ data: recs });

                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                       dSS._("Could not load token information") +
                       ": " + err );
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"), dSS._("Could not load") +
                   ": " + request.url);
            }
        });
    } 
});

Ext.reg('dsstokenauthpanel', dSS.grid.TokenAuthPanel);
